      subroutine bgf2
     & (sntr, zcctr, cmean, crrmn, dbhmn, 
     & ggsu, ggvu, glmp, zz1, zz2, zz3, sths, cths, 
     & sthv, cthv, sphi, cphi, calph, aav, aas, poouj, poodj)
c
c Calculation of bidirectional free lines of sight outside the tree crown,
c one ray exits the crown at level z1, the other ray at level z2, the angle 
c between rays is alph.  A. Kuusk 29.01.2024
c   
c  Input:
c  Output:
c    poouj, poodj
c   pooui - upper hemisphere, poodj - lower hemisphere
c
      include 'par.h'
c
*     data eps1/.1d-3/, eps2/.2d-5/, eps3/.1d-6/, eps/.1d-4/
      data eps/.1d-4/
c
      aell = crrmn
      cell = cmean
*     hbase = max((htrmax - crhmn), 0.d0)
      hbase = max((zcctr - cell), 0.d0)
      htr = zcctr + cell
      alph = acos(calph)
c                                              sun direction
      ugx = ggsu
      sthx = sths
      cthx = cths
      zzx  = zz1
      if (zzx .ge. htr) then
        aas = 1.d0
        szs = 0.d0
        tauzs = 0.d0
      else
        call gfzx
     &   (sntr, zcctr, cell, aell, dbhmn, ugx, 
     &   glmp, zzx, sthx, cthx, sproj, tauz)
        tauzs = tauz
        aas = exp(-tauzs)
        szs = sproj
      endif
c                                            view direction up
      ugx = ggvu
      sthx = sthv
      cthx = cthv
      zzx  = zz2
      if (zzx .ge. htr) then
        aav = 1.d0
        szv = 0.d0
        tauzv = 0.d0
      else
        call gfzx
     &   (sntr, zcctr, cell, aell, dbhmn, ugx, 
     &   glmp, zzx, sthx, cthx, sproj, tauz)
        tauzv = tauz
        aav = exp(-tauzv)
        szv = sproj
      endif
c                crown level hot-spot correction for the bidirectional 
c                free lines of sight outside the tree crown 
      if ((1.d0 - calph) .lt. eps) then
        tausv = tauzs
      else
        if (szv .gt. szs) then
          tauc = tauzs*(calph/(1.d0 + alph))**4
        else
          tauc = tauzv*(calph/(1.d0 + alph))**4
        endif
        tausv = tauzs + tauzv - tauc
      endif
      poouj = exp(-tausv) 
c                                                view direction down
c                           crown projection in view direction (down)
      if (zz3 .gt. hbase) then
        tgthv = sthv/cthv
        call pelld
     &     (tgthv, zz3, htr, cell, aell, szd3, vzdj)
c
        if (szd3 .gt. 0.d0) then
          rllv2d = vzdj/szd3/cthv
          azvd  = exp(-ggvu*rllv2d)
*         sl3  = ((hbase + zz3)*.5d0 - hbase)*tgthv
        else
          azvd  = 0.d0
*         sl3   = 0.d0
        endif
      else
        azvd  = 0.d0
*       sl3   = 0.d0
        szd3  = 0.d0
      endif
c                            stem projection in view direction (down)
      slty3 = min(zz3, hbase)
      zx1 = 0.d0
      zx2 = slty3
      call stem(zx1, zx2, dbhmn, htr, styx)
      sty3  = styx*tgthv
c                            pooj down using binome equation, 28.05.1998
c                            the first coefficient
      b11j = bij(azvd, glmp)
c                                   ***** ! kas seda on üldse vaja? ! ****
*       x7  = x3 - sl3*cphi
*       y7  = y3 - sl3*sphi
*       sl8 = sqrt((x7 - x8)**2 + (y7 - y8)**2)
* c
* c                                  common area of stem projections
*       if (sty3 .gt. 0.d0 .and. sty2 .gt. 0.d0) then
*         if (-cphi .lt. -eps3) then
*           scty = 0.d0
*         else if (abs(y3 - y2) .lt. eps1) then
*           x5  = x2 + slty2
*           x6  = x3 - slty3*cphi
*           if (abs(x3 - x2) .lt. eps1) then
*             tstphi = dbhmn/min(slty3, slty2)
*             if (sphi .gt. tstphi) then
*               scty = dbhmn**2*.5d0/sphi
*             else if ((min(x5, x6) - max(x3, x2)) .gt. 0.d0) then
*               scty = (min(x5, x6) - max(x3, x2))*dbhmn
*             else
*               scty = 0.d0
*             endif
*           else
*             if (sphi .gt. eps3) then
*               scty = 0.d0
*             else if ((min(x5, x6) - max(x3, x2)) .gt. 0.d0) then
*               scty = (min(x5, x6) - max(x3, x2))*dbhmn
*             else
*               scty = 0.d0
*             endif
*           endif
*         else
*           scty = 0.d0
*         endif
*       else
*         scty = 0.d0
*       endif
c                                           ***** ! korda tegemata *****
*     sumd = sntr*(b11j*(szd3 - scomm) +
*    &       b12j*(szu2 - scomm) + b2jz*scomm + styx)
      sumd = sntr*(b11j*szd3 + sty3)
      poodj = exp(-sumd)
c
      return
      end
*
************************************************************************
*
