      function func
     & (nxx, xxf)
c
c  The interface between the main program FRT and subroutines for the
c  calculation of forest reflectance and transmittance, and the merit
c  function for the inversion of the FRT.  A. Kuusk, 24.10.1995 - Nov. 2009
c  In the iversion the reflectance and/or transmittance at several
c  wavelengths and view directions can be used.
c
c     Elliptical LAD for tree foliage   12.2012
c
c     The allometric regressions for tree parameters in the 
c     subroutine regre, Feb. 1998, Dec. 2000
c
c     Fisher's grouping index, March 1998
c
c     Ground albedo is calculated integrating over polar angle 
c     in the principal plane phi = pi/2, A. Kuusk  7.10.1998
c  
c     The expert guess in the merit function, May 2001.
c
c     The 1997 version of PROSPECT, May 2001.
c
c     Two-layer homogeneous model ACRM for the ground vegetation,
c     and two leaf optics models (PROSPECT and LIBERTY), Feb. 2002.
c
c    Exchange of functions and parameters between frt.f and func using 
c    arrays ?_* in common areas
c
      include 'par.h'
      include 'stands.h'
      include 'spectra.h'
      include 'func.h'
c
      save ncl
      save sntr, htrmax
      save htrmn, crhmn, crrmn, dbhmn, tmass
      save tlai, tbai, uuz
      save f
c
      save tlty, pdgfu, pdgfd, ptrsu, ptrsd
c
      save lopt, lstruc, lalbedo
      save x0, xx, xa, xb, xdelt, llxx
      save rsdgrou, rddgrou, ccomp3, nzs
      save rty, utot, rteff, tteff, jwl, sq
      save ggsu, ggvu, ggmean, tsst, tsvt, zcctr, cmean
c
      logical lopt, lstruc, lalbedo 
      logical lmass, lbai, lhtr, lrcrwn, lhc1
      dimension ccomp3(nclmax, ncomp0)
      dimension xx(nvar), xxf(nvar), xa(nvar), xb(nvar), xdelt(nvar), 
     & llxx(nvar), x0(nvar)
      dimension pdgfu(nclmax), pdgfd(nclmax)
      dimension ptrsu(nclmax), ptrsd(nclmax)
      dimension uuz(nzmax), tsz(nzmax), tvz(nzmax)
c
      common /pidr/ pi, dr
      common /par/  zeps, tol, tiny, ftol, alphopt, betopt, gammopt, 
     & at, dxopt, ier, ncount, nfmax, itmax, itbr, nbrak, ilo
c
      data lmass/.true./, lbai/.true./, lrcrwn/.true./
      data lhc1/.true./, lhtr/.true./
      data lstruc/.true./, lopt/.true./, lalbedo/.true./
c
      ncount = ncount + 1
      f   = 0.d0
c
      ijob   = i_job 
      lopt   = l_opt
      lstruc = l_struc
      lalbedo = .true.
c                                                 ! The first pass
      if (ncount .eq. 1) then
        ncl     = n_cl
        nzs     = n_zs
        nbnds   = n_bnds
        bangs   = -0.1d0
        clmpst  = x_c(1, 13)
        call iterats(clmpst, glmp)
c
        if (ncl .gt. 1) then
          do jcl = 2, ncl
            stdns(jcl)  = x_c(jcl, 1)
            htr(jcl)    = x_c(jcl, 2)
            hc1(jcl)    = x_c(jcl, 3)
            hc2(jcl)    = x_c(jcl, 4)
            rcr(jcl)    = x_c(jcl, 5)
            dbh(jcl)    = x_c(jcl, 6)*1.d-2
            rmass(jcl)  = x_c(jcl, 7)
            slwcl(jcl)  = x_c(jcl, 8)
            eln3(jcl)   = x_c(jcl, 9)
            thm3(jcl)   = x_c(jcl, 10)*dr
            shl(jcl)    = x_c(jcl, 11)
            if (shl(jcl) .le. 0.d0) shl(jcl) = 0.01d0
            rbai(jcl)   = x_c(jcl, 12)
            ghg(jcl)    = x_c(jcl, 14)
            cshj        = x_c(jcl, 15)
            clmpsh(jcl) = cshj
            rnlfc(jcl)  = x_c(jcl, 16)
            vaicl(jcl)  = x_c(jcl, 17)/cshj
            dcl(jcl)    = x_c(jcl, 18)
            acl(jcl)    = x_c(jcl, 19)
            do jc = 1, ncomp3(jcl)
              ccomp3(jcl, jc) = x_c(jcl, jc+19)
            enddo
          enddo
        endif
c
        if (ijob .gt. 2) then
          do jx = 1, nvar
            llxx(jx)  = l_lxx(jx)
            x0(jx)    = x_0(jx)
            xx(jx)    = x_x(jx)
            xa(jx)    = x_a(jx)
            xb(jx)    = x_b(jx)
            xdelt(jx) = x_delt(jx)
          enddo
        endif
c
        do jx = 1, nvar
          xx(jx)    = x_x(jx)
        enddo
c                                                   ! endif(ncount=1)
      endif
c                                ! check constraints and scale variables
      if (ijob .gt. 2) then
        do i = 1, nxx
          li  = llxx(i)
          xi3 = xxf(i)
          if (li .eq. 13) lstruc = .true.
          x2  = 0.d0
          if (xi3 .lt. -5.d0) then
             x2  = xi3 + 5.d0
             xi3 =  -5.d0
          else if (xi3 .gt. 5.d0) then
             x2  = xi3 - 5.d0
             xi3 = 5.d0
          endif
          xli    = xi3/xb(li) + xa(li)
          xx(li) = xli
          xi4    = x2*x2*at
          f      = f + xi4 + ((xli - x0(li))/xdelt(li))**2
        enddo
      endif
c
      nbnds = n_bnds
c                                   ! initialisation of model parameters
      stdns1   = xx(1)
      stdns(1) = stdns1
      htr(1)   = xx(2)
      hc1(1)   = xx(3)
c
      if (hc1(1) .gt. htr(1)) then
        hc1(1) = htr(1)
        print *, 'FUNCIFRT:'
        print *, 'Warning: tree height is less than crown length,'
        print *, 'the crown length is adjusted.'
      endif
c
      hc2(1)   = xx(4)
      rcr(1)   = xx(5)
      dbh(1)   = xx(6)*1.d-2
      rmass(1) = xx(7)
      slwcl(1) = xx(8)
      eln3(1)  = xx(9)
      thm3(1)  = xx(10)*dr
      shl(1)   = xx(11)
      rbai(1)  = xx(12)
      if (shl(1) .le. 0.d0) shl(1) = .01d0
c
c
      if (lhtr .or. lhc1 .or. lrcrwn .or. lmass .or. lbai) then
        call regre
     &     (lmass, lbai, lhtr, lrcrwn, lhc1,
     &     xx, rlai, htr, hc1, rcr)
      endif
c
      clmpx  = xx(13)
      clmpst = clmpx
      call iterats(clmpx, fgx)
      glmp      = fgx
      ghg(1)    = xx(14)
      clmpsh(1) = xx(15)
      rnlfc(1)  = xx(16)
      vaicl(1)  = xx(17)/xx(15)
      dcl(1)    = xx(18)
      acl(1)    = xx(18)
c
      do jc = 1, ncomp3(1)
        ccomp3(1, jc) = xx(jc+19)
      enddo
c                        ground vegetation, upper layer
      ul2   = xx(30)
      sl2   = xx(31)
      clmp2 = xx(32)
      szz   = xx(33)
      eln2  = xx(34)
      thm2  = xx(35)*dr
      rnc2  = xx(36)
      slw2  = xx(37)
      vai2  = xx(38)/clmp2
      do jc = 1, ncomp2
        ccomp2(jc) = xx(jc+38)
      enddo
c                        ground vegetation, lower layer
      ul1   = xx(49)
      sl1   = xx(50)
      clmp1 = xx(51)
      eln1  = xx(52)
      thm1  = xx(53)*dr
      rnc1  = xx(54)
      slw1  = xx(55)
      vai1  = xx(56)/clmp1
      do jc = 1, ncomp1
         ccomp1(jc) = xx(jc+56)
      enddo
c                                        soil
      rsl1  = xx(67)
      rsl2  = xx(68)
      rsl3  = xx(69)
      rsl4  = xx(70)
c
        if (ijob .eq. 0 .or. ijob .eq. 2) nbnds = 1
c
        if (lstruc) then
c                             ! strmean depends on the view and sun angle
          do j = 1, 19
            x_m(j) = -99.d0
          enddo
          call strmean
     & (ncl, ths, thv, sntr, htrmn, htrmax, crhmn, cilhmn, 
     & zcctr, cmean, crrmn, dbhmn, shlm, dcmn, acmn, 
     & crcl, cacl, tlty, tlai, tbai, efflai, utot, uuz, tsz, tvz, 
     & ggsu, ggvu, ggmean, tsst, tsvt, tmass)
          x_m(1)  = sntr
          x_m(2)  = htrmn
          x_m(3)  = crhmn
          x_m(4)  = cilhmn
          x_m(5)  = crrmn
          x_m(6)  = dbhmn*100.d0
          x_m(7)  = tmass
*         x_m(11) = shlm
          x_m(12) = tbai
          x_m(13) = clmpst
*         x_m(18) = dcmn
*         x_m(19) = acmn
          t_lai   = tlai
c                              ! calculation of probabilities
          call hetk8s
     & (ncl, ths, thv, phi, nzs, sntr, zcctr, cmean, crrmn, dbhmn, 
     & ggsu, ggvu, aas, aav, pslv, pstr, psvgr, 
     & psgvus, pdgfu, pdgfd, ptrsu, ptrsd)
c
          aavth(jview) = aav
          p11gr(jview) = psvgr
          p11lvs(jview) = pslv
          p11tr(jview) = pstr
          do j = 1, ncl
            u_uu(j) = uuu(j)
          enddo
c                    if ijob=0 provide the vertical profile of foliage
          if (ijob .eq. 0) then
            do j = 1, nzmax
              u_uz(j) = uuz(j)
            enddo
          endif
          lstruc = .false.
        endif
c
c                                        ! Loop for wavelength
        if (lopt) then
        do 44 jbnd = 1, nbnds
c
          lopt = .true.
          rlambda = wlarr(jbnd)
          jwl     = nint((rlambda - (400.d0 - wlstep))/wlstep)
c
c                              ! Mean optical parameters
          call optmean
     & (ncl, rlambda, ccomp3, thknot, p14, tlty, tlai, utot,
     & rleff, tleff, rneff, rty, rteff, tteff)
c
          r_leff  = rleff
          t_leff  = tleff
          r_neff  = rneff
c                                   ! Direct run of the model    *******
c                   ! Albedo of ground vegetation is calculated
c                   ! integrating ACRM in perpendicular plane, phi = pi/2
c
          if (lalbedo) then
            phisave = phi
            phi     = pi/2.d0
            call aground
     & (ths, thv, thknot, phi, p14, 
     & rsogrou, rdsgrou, rsdgrou, rddgrou)
c                        ! rddgrou = albedo of ground vegetation
          endif 
          phi     = phisave
c
      call hetk8o
     & (ncl, pdgfu, pdgfd, ptrsu, ptrsd, bc1u, bi0u, bi0d)
c
c                                     S'/Q - direct to diffuse ratio
      sq = sqa(jwl)
      skyl  = 1.d0 - sq
c                                     diffuse fluxes of tree crowns
      call diffor
     & (ths, thv, skyl, ggmean, efflai, tlty, utot, 
     & tsst, tsvt, rteff, tteff, 
     & rdsgrou, rsdgrou, rddgrou, budif, bddif)
c
c               ! reflectance of ground vegetation r??grou was calculated
c               ! with 2-layer model ACRMf
c
      skylg = bddif/(aas*cos(ths)*pi + bddif)
c  bddif is diffuse downward flux under tree layer
c
      call smcrm
     &  (penalty, bc1, bs1, bdplants, bdsoil)
         sbgr = (1.d0 - skylg)*(bc1 + bs1) + bdplants + bdsoil
c
      psgxx    = psvgr
      br1gr    = sbgr*psgxx
c
      b0       = bi0u + br1gr
      bqq     = sq*b0 + budif
*          write (0, *) thv/dr, budif
c                                          reflectance components
      bqg1 = sq*br1gr
      bqm  = budif
c                                                  radiance down
      bsngld = bi0d
      bdwn = (bsngld + bddif) + aav/pi*skyl
*        print *, thv/dr,
*    & bsngld, bddif, aav/pi*skyl, bdown(j_sun)
c 
      if (ijob .le. 1) then
        lopt = .true.
        jwl   = nint((rlambda - (400.d0 - wlstep))/wlstep)
        skyl  = 1.d0 - sq
        call smcrm
     &     (penalty, bc1, bs1, bdplants, bdsoil)
        sbgr = sq*(bc1 + bs1) + bdplants + bdsoil
        sq = 1.d0 - skyl
      endif
      if (ijob .le. 1) then
        rth(jbnd)   = bqq
        rsth(jbnd)  = sbgr
        rcr1(jbnd)  = bc1u
        rgr1(jbnd)  = bqg1
        rdif(jbnd)  = bqm
        bdown(jbnd) = bdwn
        if (mod(jbnd, 10) .eq. 0)
     &    write (0, '(f8.1, g16.5)') wlarr(jbnd), rth(jbnd)
      endif
      if (ijob .eq. 2) then
        rth(jview)   = bqq
        rsth(jview)  = sbgr
        rcr1(jview)  = bc1u
        rgr1(jview)  = bqg1
        rdif(jview)  = bqm
        bdown(jview) = bdwn
        lstruc = .true.
      endif
c
c                                   ! calculation of the merit function
        if (ijob .gt. 2) then
          xi4  = bqjarr(jbnd) - bqq
*     print *, 'HET48o: bqi, p2bq: ', bqi, p2bq
          if (ijob .eq. 4) then
            f    = f + xi4**2
          else
            f    = f + (xi4/drhoarr(jbnd))**2
          endif
        endif
        r_out(jbnd) = bqq
44    continue
      endif
c
      func  = f
*           print *, 'f  = ', f
      return
      end
*
************************************************************************
*
