      subroutine hetk8o
     & (ncl, pdgfu, pdgfd, ptrsu, ptrsd, bc1u, bi0u, bi0d)
c
c     Replaced the phase function by M. Mõttus by the Henyey-Greenstein
c     phase function, 01.2013
c
c     Phase function of conifer shoots by M. Mõttus (Agricult. & Forest
c     Meteorol., 154-155, 67-74 (2012)), 01.2013
c
c     Single scattering radiance of tree crowns, up and down
c     A. Kuusk  27.11.2000
c
c  Input:
c      & (ncl, jwl, nsangl, thknot, phi, stdns, uuu,
c      & pdgfu, pdgfd, pkhair, rnlf, rtrnk, rrs, ttt,
c  Output:
c      bi0u, bi0d
c
      include 'par.h'
      include 'stands.h'
      include 'spectra.h'
      include 'func.h'
c      
      dimension pdgfu(nclmax), pdgfd(nclmax)
      dimension ptrsu(nclmax), ptrsd(nclmax)
      common /pidr/ pi, dr
c
      data eps/.01d0/, eps2/.0001d0/
c
      jwl   = nint((rlambda - (400.d0 - wlstep))/wlstep)
      sthv  = sin(thv)
      cthv  = cos(thv)
      cphi  = cos(phi)
      sphi  = sin(phi)
      cths  = cos(ths)
      sths  = sin(ths)
      tgths = sths/cths
      calph = sthv*sths*cphi + cthv*cths
      alpha = acos(calph)
      aalpha  = abs(alpha)
      if (aalpha .lt. eps) alpha = 0.d0
      if (alpha .lt. 0.d0) alpha = alpha + pi
      salph   = sin(alpha)
      calp2u  = cos(alpha*.5d0)
      calp2d  = sqrt((1.d0 - calph)/2.d0)
c
      bbbu    = 0.d0
      bbbd    = 0.d0
      bc1u    = 0.d0
c                       ! Summing over tree classes
      do jcl = 1, ncl
        ulj    = uuu(jcl)
        stdnsj = stdns(jcl)
        rdlf   = rrs(jcl, jwl)
        tleaf  = ttt(jcl, jwl)
        rnj    = rnlf(jcl)
        thmj   = thm3(jcl)
        eej    = 1.d0 - exp(-eln3(jcl))
        szg    = 1.d0
        ellmyj = abs(.059d0*eln3(jcl) *(thmj - 1.02d0) + .02d0)
        ellsij = abs(.0107d0 - .0216d0*eln3(jcl)*(thmj - .265d0))
c
c                 ! Phase function of scattering gamma
c                 ! the upper hemisphere (diffuse + specular)
        call gamma
     & (thv, ths, phi, thmj, eej, szg, rdlf, tleaf, rnj,
     & pkhair, ellmyj, ellsij, gg1, gg2, gamm)
c
*       ulgj = ulj*sqrt(gg1*gg2)
        ulgj = ulj*gg2
*                   ! Phase function gamma,  spherical orientation
*                   ! the upper hemisphere
        gammr = (salph + (pi - alpha)*calph)/(3.d0*pi)
        gammt = (salph - alpha*calph)/(3.d0*pi)
c
c  Phase function of specular reflection
        call gmfres(calp2u, rnj, pkhair, gmf)
        if (eej .le. eps) then
           gl  = 1.d0
        else
           y4  = abs(cthv + cths)*.5d0/calp2u
           thp = acos(y4)
           call gleaf(thmj, eej, thp, gl)
        endif
        gammf = gmf*gl*.125d0
        omg   = rdlf + tleaf
c
        if (ghg(jcl) .gt. .0d0) then
c  The Henyey-Greenstein phase function
          ghgj = ghg(jcl)
          gammd = (1.d0 - ghgj*ghgj)/sqrt((1.d0 + ghgj*ghgj -
     &             2.d0*ghgj*calph)**3)/(4.d0*pi)*omg
          gamm  = gammd + gammf
c
        endif
*
*       print *, 'thv, alpha, gamme, gammil, gamm, gl, gammf', 
*    &  thv/dr, alpha/dr, gamme, gammil, gamm, gl, gammf
c
        bfu = pdgfu(jcl)
        bbx = ulgj*bfu*stdnsj/(cths*cthv)
        bc1uj = bbx*gamm
        bc1u = bc1u + bc1uj
c
*           write (0, '(a,3x,6f10.6)')
*    &  "hetk8o: ", thv/dr, ths/dr, psv, gsv
c
c                     ! Phase function gamma, vertical orientation
c                     ! tty = 0, rfr = 0
c                     ! the upper hemisphere
        gammv = rtrnk(jcl)*(sphi - (phi - pi)*cphi)/(2.d0*pi)
c
        btr1u   = ptrsu(jcl)*gammv*stdnsj*
     &             tgths*sthv/cthv
c
        bbbu    = bbbu + bc1uj + btr1u
c
c                                  ! the lower hemisphere
        gammr = (salph + (pi - alpha)*calph)/(3.d0*pi)
        gammt = (salph - alpha*calph)/(3.d0*pi)
        gammd = rdlf*gammt + tleaf*gammr
c
        if (calp2d .lt. eps2) then
          rfr = 0.d0
        else
          call gmfres(calp2d, rnj, pkhair, rfr)
        endif
c
        gammout = gammd + rfr*.125d0
c
        bc1d    = pdgfd(jcl)*gammout*ulgj*stdnsj/
     &            (cths*cthv)
c                    ! Phase function gamma, vertical orientation
c                    ! tty = 0, rfr = 0
c                    ! the lower hemisphere
        gammout = rtrnk(jcl)*(sphi - phi*cphi)/(2.d0*pi)
c
        btr1d   = ptrsd(jcl)*gammout*stdnsj*
     &            tgths*sthv/cthv
        bbbd    = bbbd + bc1d + btr1d

      enddo
c 
      bi0u   = bbbu
      bi0d   = bbbd
c
      return
      end
*
***********************************************************************
*
