      subroutine layer
     & (ths, thv, ggs, ggv, ggj, ul, rrl, ttl,
     & rdd, tdd, rdo, tdo, rsd, tsd, rso)
c
c  Diffuse reflection and transmission of a layer from the SAIL model
c  for elliptical leaf orientation. A. Kuusk  13.01.2000.
c  Approximations for small absorption added 22.07.2009.
c
c  ths - the Sun zenith, thv - view angle
c  ggs, ggv - G-function at ths and thv
c  ggj  - the J-integral in the SAIL model
c  rrl, ttl - leaf reflection and transmission
c  rdd, tdd - diffuse reflectance and transmittance of a layer
c             in 2-stream approximation
c  rdo - diffuse reflectance of the upper surface of the layer,
c  rsd, tsd - diffuse reflectance and transmittance of the layer
c             for direct incoming radiation
c  rso - diffuse reflectance of the layer in view direction
c        for direct incoming radiation
c        the angular dependence from the SAIL model
c
      include 'par.h'
      data eps/.1d-5/
c
      rtp   = (rrl + ttl)/2.d0
      rtm   = (rrl - ttl)/2.d0
      bf    = rtm*ggj
c
      vks   = ggs/cos(ths)
      vkv   = ggv/cos(thv)
      vsig  = rtp + bf
      vatt  = 1.d0 - rtp + bf
      vssf  = vks*rtp - bf
      vssb  = vks*rtp + bf
      vmm2  = vatt**2 - vsig**2
      vmm   = sqrt(vmm2)
      vvv   = vkv*rtp + bf
      vuu   = vkv*rtp - bf
      vh1   = (vatt + vmm)/vsig
      vh2   = 1.d0/vh1
      vcc   = (vssf*vsig - vssb*(vks - vatt))/(vmm2 - vks**2)
      vdd   = (vssb*vsig + vssf*(vks + vatt))/(vmm2 - vks**2)
c
      exku1 = exp(-vks*ul)
      if (vmm .lt. eps) then
c                                                  ! small absorption
        exku2 = exp(-vkv*ul)
        xxx   = (2.d0 + ul*(1.d0 + vmm2))
        yy1   = eint((vkv - vmm)*ul)
        yy2   = eint((vkv + vmm)*ul)
        yy3   = eint(vkv*ul)
        xx3   = (1.d0 - (1.d0 + vkv*ul)*exku2)/(vkv*vkv - vmm2)
        rdd   = ul*(1.d0 - vmm2)/xxx
        tdd   = 2.d0/xxx
        tsd   = vdd*(exku1 - tdd) - 
     &          (1d0 - vmm2)*ul*vcc*exku1/xxx
        rsd   = vcc - (2.d0*vcc*exku1 + vdd*ul*(1.d0 - vmm2))/xxx
        tdo   = (((vuu*(1.d0 + vmm2) + vvv*(1.d0 - vmm2))*xx3) +
     &          ul*vuu*(yy1 + yy2))/xxx
        rso   = (((1.d0 + vkv*ul)*exku2 - 1.d0)*(vvv+vuu)*
     &          (vcc*exku2 - vdd) -
     &          ul*vkv*vkv*(2.d0*vuu*vcc*exku2 + (vuu + vvv)*ul*vdd +
     &          2.d0*vvv*vdd)*yy3)/(2.d0 + ul)/vkv/vkv +
     &          ul*(vvv*vdd + vuu*vcc)*eint((vkv + vks)*ul)
        rdo   = ((vvv + vuu)*((1.d0 + vkv*ul)*exku2 - 1.d0) +
     &          ul*vkv*vkv*(vvv*(2.d0 + ul) + vuu*ul)*yy3)/
     &          (2.d0 + ul)/vkv/vkv
      else
        exmu1 = exp(vmm*ul)
        exmu2 = 1.d0/exmu1
        delt  = vh1*exmu1 - vh2*exmu2
        rdd   = (exmu1 - exmu2)/delt
        tdd   = (vh1 - vh2)/delt
        delta = vh2*vcc*exku1 - vdd*exmu1
        deltb = vdd*exmu2 - vh1*vcc*exku1
        tsd   = (vcc*(exku1*(exmu2 - exmu1)) +
     &          vdd*(vh2 - vh1))/delt + vdd*exku1
        rsd   = (delta + deltb)/delt + vcc
        tdo   = ul*((vuu*vh1 + vvv)*eint((vkv - vmm)*ul) -
     &          (vuu*vh2 + vvv)*eint((vkv + vmm)*ul))/delt
        rso   = ul*((vvv*vdd + vuu*vcc)*eint((vkv + vks)*ul) +
     &          ((vvv*vh1 + vuu)*delta*eint((vkv + vmm)*ul) +
     &          (vvv*vh2 + vuu)*deltb*eint((vkv - vmm)*ul))/delt)
        rdo   = ul*(exmu1*(vvv*vh1 + vuu)*eint((vkv + vmm)*ul) -
     &          exmu2*(vvv*vh2 + vuu)*eint((vkv - vmm)*ul))/delt
      endif
c
      return
      end
*
**********************************************************************
*
