      subroutine optmean
     & (ncl, rlambda, ccomp3, thknot, p14, tlty, tlai, utot,
     & rleff, tleff, rneff, rty, rteff, tteff)
c
c  the mean values of optical parameters    A. Kuusk 23.09.1995
c
c  Input:
c    & (ncl, rlambda, thknot, p14, tlty, tlai, utot
c    
c  Output:
c    tleff, rleff, rneff, rty, rteff, tteff
c
c  utot - total LAI (leaves + branches + stems)
c  rteff - total efffective leaf reflectance
c  tteff - total efffective leaf transmittance
c
      include 'par.h'
      include 'stands.h'
      include 'spectra.h'
      dimension ccomp3(nclmax, ncomp0)
      dimension thknot(nknots), p14(nknots)
      dimension acoef1(nspchnl, ncomp0), acoef2(nspchnl, ncomp0)
      dimension acoef3(nclmax, nspchnl, ncomp0)
      common /pidr/ pi, dr
      common /spect3/ acoef1, acoef2, acoef3
c
      jwl   = nint((rlambda - (400.d0 - wlstep))/wlstep)
      rind = refr(jwl)
c                                  ! Leaf reflectance and transmittance
      do jcl = 1, ncl
        dkl = 0.d0
        do jc = 1, ncomp3(jcl)
          dkl = dkl + ccomp3(jcl, jc)*acoef3(jcl, jwl, jc)
        enddo
        slw = slwcl(jcl)
        vai = vaicl(jcl)
        if (lmod3(jcl) .eq. 'prospect') then
          dkl = dkl/vai*slw*1.d-6
          call prospect(rind, dkl, vai, refl, tran)
        else if (lmod3(jcl) .eq. 'liberty') then
          rinf = -99.d0
          dcell0 = 80.d0
          asp = acl(jcl)
          coeff = dkl*dcl(jcl)/dcell0*slw*1.d-6/vai
          call liberty
     &         (thknot, p14, rind, coeff, vai, asp, rinf, refl, tran)
          if (refl .eq. rinf .or. rinf .lt. 0d0) then
            write (0, '(/a/a, f6.1, f10.4 /)') 'LIBERTY:',
     &        'Cannot calculate leaf reflectance, lambda, r_leaf = ', 
     &        rlambda, refl
            stop
          endif
        else
          print *, 'ERR: No leaf optics model'
          stop
        endif 
c     
        rnlfj      = rnlfc(jcl)*rind
        rnlf(jcl)   = rnlfj
        if (rnlfj .lt. 1.d0) rnlfj = 1.d0
        rspj = ((1.d0 - rnlfj)/(1.d0 + rnlfj))**2
        rlfcl(jcl) = refl - rspj
        tlfcl(jcl) = tran
        rbrnc(jcl) = brrarr(jcl, jwl)
        rtrnk(jcl) = trrarr(jcl, jwl)
      enddo 
c
      sum2   = 0.d0
      do j = 1, ncl
        stdnsj  = stdns(j)
        htrj    = htr(j)
        hcrj    = hc1(j) + hc2(j)
        cli     = stdnsj*dbh(j)*.5d0*(htrj - hcrj)
        sum2    = sum2 + cli*rtrnk(j)
      enddo
      rty    = 0.d0
      if (tlty .ne. 0.d0) rty = sum2*pi/tlty
c
      sul0   = 0.d0
      sumn   = 0.d0
      sut    = 0.d0
      rteff  = 0.d0
      tteff  = 0.d0
c
      do jcl = 1, ncl
        rnlfj        = rnlf(jcl)
        rlaief       = rlai(jcl)*clmpsh(jcl)
        baicl        = rbai(jcl)*rlaief
        claief       = rlaief + baicl
        rdlf         = (rlaief*rlfcl(jcl) + baicl*rbrnc(jcl))
        rrs(jcl, jwl) = rdlf/claief
c
        rteff = rteff + rdlf + rbrnc(jcl)*baicl
        ttl          = rlaief*tlfcl(jcl)
        tteff        = tteff + ttl
        ttt(jcl, jwl) = ttl/claief
c
        sul0    = sul0 + rdlf
        sut     = sut + ttl
        sumn    = sumn + rlai(jcl)*rnlf(jcl)
      enddo
c
      rteff = (rteff + rty*tlty)/(utot + tlty)
      tteff = tteff/(utot + tlty)
      rneff = 1.d0
      if (tlai .ne. 0.d0) rneff = sumn/tlai
      tleff = sut/utot
      r12   = sul0/utot
      rleff = r12 + ((rneff - 1.d0)/(rneff + 1.d0))**2
c
      return
      end
*
************************************************************************
*
