      subroutine rcone
     & (xi, yj, zk, sthrl, cthrl, sphrl, cphrl, 
     &  aell, vhcon, vhcyl, rlout)
c
c   Distance from (xi, yj, zk) to crown perimeter in direction thet, phi
c   cylinder+cone,  A. Kuusk, 7.05.1998
c  Input:
c    xi, yj, zk, sthrl, cthrl, sphrl, cphrl,
c    &  aell, vhcon, vhcyl
c  Output:
c    rlout
c   vhcon - length of cone, aell - crow radius
c   phi - azimuth counted from x-axis
c   sphrl = sin(phi), cphrl = cos(phi)
c   sthrl = sin(thet), cthrl = cos(thet)
c
      include 'par.h'
      data eps/.1d-6/, eps2/.001d0/
c
      z3 = vhcon + eps2
      z0 = zk - eps2
c
      if (cthrl .ge. 0.d0) then
c                                             upper hemisphere
      if (sthrl .lt. eps) then
c                                             vertical ray
         rlout = vhcon - vhcon/aell*sqrt(xi**2 + yj**2) - zk
      else
         ae2 = aell**2
         hh2 = vhcon**2
         if (zk .ge. 0.d0) then
c                                                (x,y,z) in cone
            aaa = ae2/hh2*(cthrl**2) - sthrl**2
            bbb = 2.d0*ae2*cthrl/vhcon*(zk/vhcon - 1.d0) 
     &            - 2.d0*sthrl*(xi*cphrl + yj*sphrl)
            ccc = (aell*(1.d0 - zk/vhcon))**2 - (xi**2 + yj**2)
            det = bbb**2 - 4.d0*aaa*ccc
            if (abs(det) .lt. eps) then
               rlout = -bbb*.5d0/aaa
            else
               rlout = (sqrt(det) - bbb)*.5d0/aaa
               z2    = rlout*cthrl + zk
               if (z2 .lt. z0 .or. z2 .gt. z3) 
     &            rlout = (-sqrt(det) - bbb)*.5d0/aaa
            endif
c
         else
c                                                (x,y,z) in cylinder
            rl1 = 0.d0
            aaa = sthrl**2
            bbb = 2.d0*sthrl*(xi*cphrl + yj*sphrl)
            ccc = xi**2 + yj**2 - ae2
            det = bbb**2 - 4.d0*aaa*ccc
            if (abs(det) .lt. eps) then
               rl1 = -bbb*.5d0/aaa
            else
               rl1 = (sqrt(det) - bbb)*.5d0/aaa
               z2  = rl1*cthrl + zk
               if (z2 .lt. z0) 
     &            rl1 = (-sqrt(det) - bbb)*.5d0/aaa
            endif
c
            z2  = rl1*cthrl + zk
            if (z2 .le. 0.d0) then
c                                                ray exits cylinder wall
               rlout = rl1
            else
c                                                ray exits cone
               rl2 = 0.d0
               rl1 = abs(zk)/cthrl
               x3  = xi + rl1*sthrl*cphrl
               y3  = yj + rl1*sthrl*sphrl
               aaa = ae2/hh2*(cthrl**2) - sthrl**2
               bbb = -2.d0*ae2*cthrl/vhcon 
     &               - 2.d0*sthrl*(x3*cphrl + y3*sphrl)
               ccc = ae2 - (x3**2 + y3**2)
               det = bbb**2 - 4.d0*aaa*ccc
               if (abs(det) .lt. eps) then
                  rl2 = -bbb*.5d0/aaa
               else
                  rl2 = (sqrt(det) - bbb)*.5d0/aaa
                  z2  = rl2*cthrl
                  if (z2 .lt. 0.d0 .or. z2 .gt. z3) 
     &            rl2 = (-sqrt(det) - bbb)*.5d0/aaa
               endif
               rlout = rl1 + rl2
            endif
         endif
      endif
      else
c                                                lower hemisphere
      if (sthrl .lt. eps) then
c                                                vertical ray
         rlout = vhcyl + zk
      else
         ae2 = aell**2 + eps
         if (zk .ge. 0.d0) then
c                                                (x,y,z) in cone
            rl1 = -zk/cthrl
            x2  = xi + rl1*sthrl*cphrl
            y2  = yj + rl1*sthrl*sphrl
            if ((x2**2 + y2**2) .ge. ae2) then
c                                      ray exits cone 
               hh2 = vhcon**2
               aaa = aell**2/hh2*(cthrl**2) - sthrl**2
               bbb = 2.d0*aell**2*cthrl/vhcon*(zk/vhcon - 1.d0) 
     &               - 2.d0*sthrl*(xi*cphrl + yj*sphrl)
               ccc = (aell*(1.d0 - zk/vhcon))**2 - (xi**2 + yj**2)
               det = bbb**2 - 4.d0*aaa*ccc
               if (abs(det) .lt. eps) then
                  rlout = -bbb*.5d0/aaa
               else
                  rlout = (sqrt(det) - bbb)*.5d0/aaa
                  if (rlout .lt. 0.d0) 
     &               rlout = (-sqrt(det) - bbb)*.5d0/aaa
               endif
            else
c                                     ray enters cylinder
               rl1 = -(vhcyl + zk)/cthrl
               x2  = xi + rl1*sthrl*cphrl
               y2  = yj + rl1*sthrl*sphrl
               if ((x2**2 + y2**2) .le. ae2) then
c                                    ray exits cylinder bottom
                  rlout = rl1
               else
c                                    ray exits cylinder wall
                  aaa = sthrl**2
                  bbb = 2.d0*sthrl*(xi*cphrl + yj*sphrl)
                  ccc = xi**2 + yj**2 - aell**2
                  det = bbb**2 - 4.d0*aaa*ccc
                  if (abs(det) .lt. eps) then
                     rl1 = -bbb*.5d0/aaa
                  else
                     rl1 = (sqrt(det) - bbb)*.5d0/aaa
                  endif
                  z2  = rl1*cthrl + zk
                  if (z2 .gt. 0.d0)
     &               rl1 = (-sqrt(det) - bbb)*.5d0/aaa
                  rlout = rl1
               endif
            endif
c
         else
c                                             (x,y,z) in cylinder
            rl1 = -(vhcyl + zk)/cthrl
            x2  = xi + rl1*sthrl*cphrl
            y2  = yj + rl1*sthrl*sphrl
            if ((x2**2 + y2**2) .le. ae2) then
c                                    ray exits cylinder bottom
               rlout = rl1
            else
c                                    ray exits cylinder wall
               aaa = sthrl**2
               bbb = 2.d0*sthrl*(xi*cphrl + yj*sphrl)
               ccc = xi**2 + yj**2 - aell**2
               det = bbb**2 - 4.d0*aaa*ccc
               if (abs(det) .lt. eps) then
                  rl1 = -bbb*.5d0/aaa
               else
                  rl1 = (sqrt(det) - bbb)*.5d0/aaa
               endif
               z2  = rl1*cthrl + zk
               if (z2 .gt. zk + eps)
     &            rl1 = (-sqrt(det) - bbb)*.5d0/aaa
               rlout = rl1
            endif
c
         endif
      endif
      endif
c
*     if (rlout .lt. -.01d0) then
*        print *, 'rlout = ', rlout
*        pause
*     endif
      if (rlout .lt. 0.d0) rlout = 0.d0
      return
      end
*
************************************************************************
*
