      function eint(xy)
c   exponential integral
c
      implicit double precision (a-h, o-z)
c
      data eps/.1d-6/
c
      if (abs(xy) .lt. eps) then
         eint = 1.d0 - xy*(.5d0 - xy/6.d0*(1.d0 - xy*.25d0))
      else
         eint = (1.d0 - exp(-xy))/xy
      endif
      return
      end
*
***********************************************************************
*
      subroutine smcrm
     &  (penalty, bc1, bs1, bdplants, bdsoil)
c
c   Two-layer CR model. A. Kuusk  andres@aai.ee   25.02.2000
c
      include 'par.h'
      include 'spectra.h'
      include 'func.h'
      dimension acoef1(nspchnl, ncomp0), acoef2(nspchnl, ncomp0)
      dimension acoef3(nclmax, nspchnl, ncomp0)
      common /spect3/ acoef1, acoef2, acoef3
c
      data eps2/.1d-5/
c
c                                   ! jw - wavelength counter
      jw   = nint((rlambda - (400.d0 - wlstep))/wlstep)
c                                   ! rsl - soil nadir reflectance
      rsl  = rsl1*phis(jw, 1) + rsl2*phis(jw, 2)
     &     + rsl3*phis(jw, 3) + rsl4*phis(jw, 4)
      if (rsl .lt. 0.d0) rsl = 0.d0
c
      if ((ul1 + ul2) .gt. eps2) then
c
c                        ! leaf reflectance and transmittance
c                        ! penalty was used for debugging
         penalty = 0.d0
         rnnl = refr(jw)
         dkl2 = 0.d0
         do jc = 1, ncomp2
            dkl2 = dkl2 + ccomp2(jc)*acoef2(jw, jc)
         enddo
c
         if (lmod2 .eq. 'prospect') then
            dkl2 = dkl2/vai2*slw2*1.d-6
            call prospect(rnnl, dkl2, vai2, refl2, tran2)
         endif
c
         dkl1 = 0.d0
         do jc = 1, ncomp1
            dkl1 = dkl1 + ccomp1(jc)*acoef1(jw, jc)
         enddo
c
         if (lmod1 .eq. 'prospect') then
            dkl1 = dkl1/vai1*slw1*1.d-6
            call prospect(rnnl, dkl1, vai1, refl1, tran1)
         endif
c
c                                  ! G-function and phase function
c                                  ellmy = (my(eln) - my(eln=0))
c                                  ellsi = (sigma(eln) - sigma(eln=0))
         ellmy1  = abs(.059d0*eln1*(thm1 - 1.02d0) + .02d0)
         ellsi1  = abs(.0107d0 - .0216d0*eln1*(thm1 - .265d0))
         ellmy2  = abs(.059d0*eln2*(thm2 - 1.02d0) + .02d0)
         ellsi2  = abs(.0107d0 - .0216d0*eln2*(thm2 - .265d0))
         ee1     = 1.d0 - exp(-eln1)
         ee2     = 1.d0 - exp(-eln2)
c
         rn1   = max(rnc1*rnnl, 1.d0)
         rrx   = refl1 - ((1.d0 - rn1)/(1.d0 + rn1))**2
         rrl1  = max(rrx, 0.d0)
         ttl1  = tran1
         rn2   = max(rnc2*rnnl, 1.d0)
         rrx   = refl2 - ((1.d0 - rn2)/(1.d0 + rn2))**2
         rrl2  = max(rrx, 0.d0)
         ttl2  = tran2
c
         sz1   = 1.d0
            call gamma
     & (thv, ths, phi, thm1, ee1, sz1, rrl1, ttl1, rn1, 
     & pkhair, ellmy1, ellsi1, gg1s, gg1v, gamm1)
            call gamma
     & (thv, ths, phi, thm2, ee2, szz, rrl2, ttl2, rn2, 
     & pkhair, ellmy2, ellsi2, gg2s, gg2v, gamm2)
      endif
c
c                             ! single scattering reflectance
            call biz2
     & (thv, ths, phi, ul2, sl2, gg2s, gg2v, gamm2,
     & ul1, sl1, gg1s, gg1v, gamm1, thss, rsl, bc1, bs1)
c
c                             ! diffuse fluxes
      rrl1 = refl1
      rrl2 = refl2
c
            call dif2
     & (thv, ths, 
     & gg1s, gg1v, ul1, rrl1, ttl1, ellmy1, ellsi1, thm1,
     & gg2s, gg2v, ul2, rrl2, ttl2, ellmy2, ellsi2, thm2,
     & skyl, rsl, thss, bdplants, bdsoil)
c
*     bq   = sq*(bc1 + bs1) + bdplants + bdsoil
c
      return
      end
*
***********************************************************************
*
      subroutine biz2
     & (th2, th1, phi, ul2, sl2, gg2s, gg2v, gamm2,
     & ul1, sl1, gg1s, gg1v, gamm1, thss, rsl, bc1, bs1)
c     Bidirectional reflectance of single scattering
c     for a two-layer canopy. An extension of the MCRM,
c     Kuusk,  Agricult. Forest Meteorol., 1995, 76(3-4): 221-236.
c     A. Kuusk   20.01.2000
c
      implicit double precision (a-h, o-z)
c
      data eps/.1d-4/, eps2/.1d-5/
c
*     eint(xy) = (1.d0 - exp(-xy))/xy
c
c                                           ! soil BRDF and albedo
      call soil
     & (rsl, th2, th1, thss, phi, rsosoil, rsdsoil, rodsoil, rddsoil)
c
      if ((ul1 + ul2) .gt. eps) goto 2
      bc1   = 0.d0
      bs1   = rsosoil
      goto 1
c
2     continue
      sth2  = sin(th2)
      cth2  = cos(th2)
      sth1  = sin(th1)
      cth1  = cos(th1)
      cp    = cos(phi)
c                                         ! hot-spot correction
      calph = sth2*sth1*cp + cth2*cth1
      catmp = calph
      alph  = acos(catmp)
      deltx = 1.d0/(cth1**2) + 1.d0/(cth2**2) - 2.d0*calph/cth1/cth2
      if (abs(deltx) .lt. eps2) then
         delta = 0.d0
      else
         delta = sqrt(1.d0/(cth1**2) + 1.d0/(cth2**2)
     &           - 2.d0*calph/cth1/cth2)
      endif
c
c                            ! bi-directional gap probabilities in
c                            ! the upper (qq2) and lower (qq1) layers
      xyc1  = delta/sl1
      chsh1 = exp(sqrt(gg1s/cth1*gg1v/cth2)*ul1*eint(xyc1))
      qq1   = exp(-ul1*(gg1s/cth1 + gg1v/cth2))*chsh1
      xyc2  = delta/sl2
      chsh2 = exp(sqrt(gg2s/cth1*gg2v/cth2)*ul2*eint(xyc2))
      qq2   = exp(-ul2*(gg2s/cth1 + gg2v/cth2))*chsh2
c
c                               ! soil contribution
      bs1   = rsosoil*qq1*qq2
c
      if (ul2 .le. 0.d0) then
         rc21 = 0.d0
      else
c                         ! canopy contribution, the upper layer
         call rhoc1
     & (cth2, cth1, alph, ul2, sl2, gg2s, gg2v, gamm2, rc21)
      endif
      if (ul1 .le. 0.d0) then
         rc11 = 0.d0
      else
c                         ! canopy contribution, the lower layer
         call rhoc1
     & (cth2, cth1, alph, ul1, sl1, gg1s, gg1v, gamm1, rc11)
      endif
c
      bc1  = rc21 + rc11*qq2
*     bi   = bc1 + bs1
c
1     continue
c
      return
      end
*
******************************************************************
*
      subroutine rhoc1
     & (cth2, cth1, alph, ul, sl, gg1, gg2, gamm, rc1)
c     Single scattering from leaves in a layer
c     A. Kuusk   20.01.2000
c
      implicit double precision (a-h, o-z)
c
*     eint(xy) = (1.d0 - exp(-xy))/xy
c
      xy1  = ul*(gg1/cth1 + gg2/cth2)
      ss1  = ul/cth1/cth2*eint(xy1)
      xy2  = ul/2.d0*
     &       (gg1/cth1 + gg2/cth2 + alph/(sl*ul*sqrt(cth1*cth2)))
      ss2  = ul/cth1/cth2*eint(xy2)
      xy3  = ul*
     &       (gg1/cth1 + gg2/cth2 + alph/(2.d0*sl*ul*sqrt(cth1*cth2)))
      ss3  = ul/cth1/cth2*eint(xy3)
c
      rc1  = gamm*(ss1 + ss2 - ss3)
c
      return
      end
*
******************************************************************
*
      subroutine gamma
     & (th2, th1, phi, thm, ee, szz, rrl, ttl, rn, rk,
     & ellmy, ellsig, gg1, gg2, gamm)
c  Phase function and G-function for a Markov chain structure
c  J. Ross,  The Radiation Regime and Architecture of Plant Stands
c  Junk,  The Hague,  1981.
c  Kuusk, A. A Markov chain model of canopy reflectance,
c  Agricult. Forest Meteorol., 1995, 76(3-4): 221-236.
c  A. Kuusk    22.03.1988 & 16.06.1992 & 20.01.2000
c  0 <= t11 < t22 <= pi/2, 0 <= phi <= pi
c
c
      include 'par.h'
      common /pidr/ pi, dr
c
      dimension fgamm(5)
      data eps3/.01d0/, eps5/.1d-2/, eps/.1d-5/
c
      pi21  = pi/2.d0
      pi4   = pi*2.d0
      pi12  = 1.d0/pi4
      pi13  = pi12/1.5d0
      pi14  = pi12*4.d0
c
      sth1  = sin(th1)
      sth2  = sin(th2)
      cth1  = cos(th1)
      cth2  = cos(th2)
      calph = sth1*sth2*cos(phi) + cth1*cth2
      alph  = acos(calph)
      salph = sin(alph)
      calp2 = cos(alph/2.d0)
c
c  Phase function of specular reflection
      call gmfres(calp2, rn, rk, gmf)
      if (ee .le. eps3) then
         gl  = 1.d0
      else
         y4  = abs(cth2 + cth1)*.5d0/calp2
         thp = acos(min(1.d0, y4))
         call gleaf(thm, ee, thp, gl)
      endif
      gammf = gmf*gl*.125d0
c
      gr0   = (salph + (pi - alph)*calph)*pi13
      gt0   = (salph - alph*calph)*pi13
c
      if (ee .lt. .4d0) then
         gr   = gr0
         gt   = gt0
         gg2  = .5d0
         gg1  = .5d0
         goto 99
      endif
c
c                            ***  gammad,  ee = 1.  ***
      if (abs(th1) .le. abs(th2)) then
         t11  = th1
         t22  = th2
         jj   = 1
      else
         t11  = th2
         t22  = th1
         jj   = 0
      endif
c
      sg2   = 0.d0
      sg1   = 0.d0
      sgmr  = 0.d0
      sgmt  = 0.d0
      yg    = 0.d0
      i1    = 0
      thmc  = pi21 - thm
      if (abs(thmc) .lt. eps) thmc = abs(thmc)
      st1   = sin(t11)
      ct1   = cos(t11)
      st2   = sin(t22)
      ct2   = cos(t22)
      sthm  = sin(thm)
      cthm  = cos(thm)
      sp    = sin(phi)
      cp    = cos(phi)
      stt1  = st1*st2
      ctt1  = ct1*ct2
      e1    = st1*ct2
      e2    = ct1*st2
      s2    = e1*cp + e2
      s3    = e1*sp
      ctg2  = 1.d30
      ctg1  = 1.d30
      if (st1 .ne. 0.d0) ctg2  = ct1/st1
      if (st2 .ne. 0.d0) ctg1  = ct2/st2
c
      if (thmc .lt. t22) goto 47
c
c  ******************************    tl1 = 0.,  tl2 = pi/2 - th1
c
      xg   = sthm*sthm
      pp   = calph*xg + ctt1*(2.d0 - 3.d0*xg)
      pp   = pp*pi
      yg   = pp
      if (yg .gt. 0.d0) sgmr = sgmr + yg
      if (yg .lt. 0.d0) sgmt = sgmt - yg
      y1  = ct2*cthm
      sg1 = sg1 + abs(y1)
c
46    continue
c
c  ******************************    tl1 = 0,  tl2 = pi/2 - th2
c
      y1    = ct1*cthm
      sg2   = sg2 + abs(y1)
      goto 48
c
47    continue
      if (thmc .lt. t11) goto 50
c
c  *****************************    tl1 = pi/2 - th1,  tl2 = pi/2 - th2
      x2 = cthm/sthm
      xg = -ctg1*x2
      x1 = sqrt(1.d0 - xg*xg)
      fa = atan2(x1, xg)
      fb = pi4 - fa
c
c  *********************************    p(fa, fb)
      xg   = fb - fa
      if (xg .gt. eps5) then
         if ((pi4 - xg) .lt. eps5) then
            xg   = sthm*sthm
            pp   = calph*xg + ctt1*(2.d0 - 3.d0*xg)
            pp   = pp*pi
            goto 172
         endif
c
         sfa  = sin(fa)
         sfb  = sin(fb)
         cfa  = cos(fa)
         cfb  = cos(fb)
         pp   = xg*ctt1*cthm*cthm
         y1   = xg + sfb*cfb - sfa*cfa
         xg   = cfa - cfb
         y1   = y1*cp + sp*xg*(cfa + cfb)
         pp   = pp + stt1*.5d0*y1*sthm*sthm
         y1   = s2*(sfb - sfa) + s3*xg
         pp   = pp + y1*sthm*cthm
         goto 172
      endif
      pp   = 0.d0
c
172    continue
      yg  = pp
      if (yg .gt. 0.d0) sgmr = sgmr + yg
      if (yg .lt. 0.d0) sgmt = sgmt - yg
      xg   = sthm*sthm
      pp   = calph*xg + ctt1*(2.d0 - 3.d0*xg)
      pp   = pp*pi
      yg   = pp - yg
      if (yg .gt. 0.d0) sgmr = sgmr + yg
      if (yg .lt. 0.d0) sgmt = sgmt - yg
c
c  ******************************    tl1 = pi/2 - th1,  tl2 = pi/2
      x2   = ct2*cthm
      x1   = st2*sthm/x2
      x1   = sqrt(x1*x1 - 1.d0)
      xg   = atan2(1.d0, x1)
      xg   = (xg + x1)*x2
      yg   = xg*pi14
      sg1  = sg1 + abs(yg)
      goto 46
c
50    continue
c
c  *********************************   tl1 = pi/2 - th2,  tl2 = pi/2
      x2       = cthm/sthm
      xg       = -ctg1*x2
      x1       = sqrt(1.d0 - xg*xg)
      fa       = atan2(x1, xg)
      fgamm(2) = fa
      fgamm(3) = pi4 - fa
      xg       = -ctg2*x2
      x1       = sqrt(1.d0 - xg*xg)
      fa       = atan2(x1, xg)
      fb       = phi - fa
      if (fb .lt. 0.d0) fb = fb + pi4
      fgamm(4) = fb
      fgamm(5) = phi + fa
c
      do ii = 2, 4
        i1 = ii + 1
        do j = i1, 5
          fa = fgamm(ii)
          fb = fgamm(j)
          if (fb .gt. fa) goto 75
          fgamm(ii) = fb
          fgamm(j)  = fa
75        continue
        enddo
      enddo
c
      fgamm(1) = fgamm(5) - pi4
      i1       = 1
c
76    continue
      ii   = i1
      i1   = ii + 1
      fa   = fgamm(ii)
      fb   = fgamm(i1)
c
c  *********************************    p(fa, fb)
c
      xg   = fb - fa
      if (xg .gt. eps5) then
         if ((pi4 - xg) .lt. eps5) then
            xg   = sthm*sthm
            pp   = calph*xg + ctt1*(2.d0 - 3.d0*xg)
            pp   = pp*pi
            goto 74
         endif
c
         sfa  = sin(fa)
         sfb  = sin(fb)
         cfa  = cos(fa)
         cfb  = cos(fb)
         pp   = xg*ctt1*cthm*cthm
         y1   = xg + sfb*cfb - sfa*cfa
         xg   = cfa - cfb
         y1   = y1*cp + sp*xg*(cfa + cfb)
         pp   = pp + stt1*.5d0*y1*sthm*sthm
         y1   = s2*(sfb - sfa) + s3*xg
         pp   = pp + y1*sthm*cthm
         goto 74
      endif
c
      pp   = 0.d0
c
74    continue
      yg    = pp
      if (yg .gt. 0.d0) sgmr = sgmr + yg
      if (yg .lt. 0.d0) sgmt = sgmt - yg
      if (i1 .le. 4) goto 76
c
      x2   = ct1*cthm
      x1   = st1*sthm/x2
      x1   = sqrt(x1*x1 - 1.d0)
      xg   = atan2(1.d0, x1)
      xg   = (xg + x1)*x2
      yg   = xg*pi14
      sg2  = sg2 + abs(yg)
c
c  ******************************    tl1 = pi/2 - th1,  tl2 = pi/2
      x2   = ct2*cthm
      x1   = st2*sthm/x2
      x1   = sqrt(x1*x1 - 1.d0)
      xg   = atan2(1.d0, x1)
      xg   = (xg + x1)*x2
      yg   = xg*pi14
      sg1  = sg1 + abs(yg)
c
48    continue
      gr1 = sgmr*pi12
      gt1 = sgmt*pi12
      grx = gr0 - .0102d0  +
     &      (1.742d0*ellmy - .4557d0*ellsig)*(gr1 - gr0)
      gr  = max(grx, 0.d0)
      gtx = gt0 + 1.14d-4 +
     &      (.2693d0*ellmy + 5.821d0*ellsig)*(gt1 - gt0)
      gt  = max(gtx, 0.d0)
      gg2 = 
     & max((2.653d0*ellmy+1.432d0*ellsig)*(sg2-.5d0)+.50072d0, 0.d0)
      gg1 = 
     & max((2.653d0*ellmy+1.432d0*ellsig)*(sg1-.5d0)+.50072d0, 0.d0)
      if (jj .eq. 1) then
         ggx  = gg1
         gg1  = gg2
         gg2  = ggx
      endif
c
99    continue
c
      gammd = gr*rrl + gt*ttl
c                                  ! Regularity parameter 
c                                  ! as a function of zenith angle
c                                  ! 1 <= szz <= 2
      szs = (szz - 1.d0)*(1.d0 - cth1) + 1.d0
      szv = (szz - 1.d0)*(1.d0 - cth2) + 1.d0
c
c                         The effective G-function
      gg1  = gg1*szs
      gg2  = gg2*szv
c
      gamm = gammd + gammf
c
*     print *, gg1, gg2, gamm
      return
      end
*
******************************************************************
*
      subroutine gleaf (thm, ee, thl, gl)
c  elliptical distribution
c  A. Kuusk   1.03.1988
c
c  thm - the modal leaf angle, ee - eccentrisity (epsilon)
c
      implicit double precision (a-h, o-z)
c
      save bb, es, tms
c
      data bb/1.d0/, es/0.d0/, tms/0.d0/, eps/.1d0/,
     &     sthm/0.d0/, cthm/1.d0/
c
      if (ee .lt. eps) then
         gl = 1.d0
         return
      endif
c
      if (ee .eq. 1.d0) ee = .999999d0
      if (ee .ne. es .or. thm .ne. tms) then
         sthm   = sin(thm)
         cthm   = cos(thm)
         u1     = ee*cthm
         u3     = ee*sthm
         u2     = sqrt(1.d0 - u1*u1)
         u4     = sqrt(1.d0 - u3*u3)
         xg     = log((u4 + u1)/(u2 - u3))
         x1     = atan2(u3, u4) - atan2(u1, u2)
         x2     = sthm*xg - cthm*x1
         bb     = ee/x2
         es     = ee
         tms    = thm
      endif
c
      gl = bb/sqrt(1.d0 - (ee*cos(thm - thl))**2)
c
      return
      end
*
******************************************************************
*
      subroutine gmfres (calp2, rn, pkhair, gmf)
c  Fresnel' reflection                    A. Kuusk 02.01.1991
c  input parameters are calp2 = cos(th_incident),  rn = refract_ind.,
c  pkhair = leaf hair index
c
c
      include 'par.h'
      common /pidr/ pi, dr
c
      pi12 = pi/2.d0
      x2  = calp2*calp2
      ag  = x2*2.d0 - 1.d0 + rn*rn
      bg  = 1.d0 + (ag - 2.d0)*x2
      xy  = ag - x2
      cg  = 2.d0*calp2*sqrt(xy)
      sa2 = 1.d0 - x2
      yg  = (bg + sa2*cg)*(ag + cg)
      yg  = (ag - cg)*bg/yg
      yy  = sqrt(sa2)/pi12/calp2*pkhair
      gmf = exp(-yy)*yg
c
      return
      end
*
******************************************************************
*
      subroutine soil
     & (rsl, th2, th1, thss, phi, rsosoil, rsdsoil, rodsoil, rddsoil)
c
c   Soil reflectance, Walthall et al., Appl. Opt., 1985, 24: 383-387
c   th2, th1, thss, phi in radians
c   rsosoil  -  bidirectional reflectance
c   r?dsoil  -  directional-hemispherical reflectance
c   rddsoil  -  hemispherical-hemispherical reflectance  (albedo)
c   A.Kuusk     1.03.1988, 07.04.2000
c
c
      implicit double precision (a-h, o-z)
c
      save asl, bsl, csl, cts, ths1, thsss
c
      data asl/.45098d0/, bsl/5.7829d0/, csl, cts/2*13.7575d0/
      data ths1, thsss/2*.785398163d0/
c
      if (thss .ne. thsss) then
         cts   = 16.41d0 - thss*thss*4.3d0
         thsss = thss
      endif
c
      if (th1 .ne. ths1) then
         ths1 = th1
         asl  = th1*th1*7.702d0 - 4.3d0
         bsl  = th1*7.363d0
         csl  = 16.41d0 - th1*th1*4.3d0
      endif
c
      cp      = cos(phi)
      rslcts  = rsl/cts
      rsosoil = ((asl*th2 + bsl*cp)*th2 + csl)*rslcts
      rsdsoil = (th1*th1*5.651d0 + 10.1d0)*rslcts
      rodsoil = (th2*th2*5.651d0 + 10.1d0)*rslcts
      rddsoil = 14.25d0*rslcts
c
      return
      end
*
******************************************************************
*
