      subroutine rtsafe(xxin, xx1, xx2, xacc, yyout)
c
c  Newton-Raphson method. Numerical Recipes, 9.4
c
*        Using a combination of Newton-Raphson and bisection,        *)
*        find the root of a function bracketed between xx1 and xx2.  *)
*        The root, returned as yyout                                 *)
*        will be refined until its accuracy is known within +-xacc.  *)
*        funcd is a user-supplied subroutine which returns both the  *)
*        function value and the first derivative of the function.    *)
c
      include 'par.h'
      external funcd
c
      call funcd(xxin, xx1, fl, df)
      call funcd(xxin, xx2, fh, df)
c
      if (fl .eq. 0.d0) then
         yyout = xx1
         goto 12
      else if (fh .eq. 0.d0) then
         yyout = xx2
         goto 12
      else if (fl .lt. 0.d0) then
*                             ! Orient the search so that f(xl) < 0. *)
         xl = xx1
         xh = xx2
      else
         xh = xx1
         xl = xx2
      endif
      yyout  = .5d0*(xx1 + xx2)
      dxold  = abs(xx2 - xx1)
      dx     = dxold
*                             ! Initialize the guess for root,  *)
*                             ! the "stepsize before last,"     *)
*                             !  and the last step.             *)
c
      call funcd(xxin, yyout, f, df)
c
      do 11 j = 1, maxit
*                             !  Loop over allowed iterations.   *)
        if (((yyout - xh)*df - f)*((yyout - xl)*df - f) .ge. 0.d0
     &  .or. abs(2.d0*f) .gt. abs(dxold*df)) then
            dxold = dx
*                             ! Bisect if Newton out of range,   *)
          dx = 0.5d0*(xh - xl)
*                             ! or not decreasing fast enough.   *)
          yyout = xl + dx
          if (xl .eq. yyout) goto 12
*                             ! Change in root is negligible. *)
        else
*                             ! Newton step acceptable. Take it. *)
           dxold = dx
           dx    = f/df
           temp  = yyout
           yyout = yyout - dx
           if (temp .eq. yyout) goto 12
        endif
        if (abs(dx).lt.xacc) goto 12
c
        call funcd(xxin, yyout, f, df)
c
*                             !Convergence criterion.       *)
*                             !The one new function evalution per iteration.*)
*                             !Maintain the bracket on the root *)
        if (f .lt. 0.d0) then
           xl = yyout
        else
           xh = yyout
        endif
11    continue
12    continue
      return
      end
*
************************************************************************
*
