      subroutine strmean
     & (ncl, ths, thv, sntr, htrmn, htrmax, crhmn, cilhmn, 
     & zcctr, cmean, crrmn, dbhmn, shlm, dcmn, acmn, 
     & crcl, cacl, tlty, tlai, tbai, efflai, utot, uuz, tsz, tvz, 
     & ggsu, ggvu, ggmean, tsst, tsvt, tmass)
c
c  the mean values of structure parameters   
c  A. Kuusk  23.09.1995, 9.01.2024, 4.11.2024
c
c  Input:
c     ncl, ths, thv. 
c     Other input parameters are in the common /stndvars/,
c     stands.h, read in in the subroutine stands.
c
c  Output:
c    & sntr, htrmn, htrmax, crhmn, cilhmn, 
c    & zcctr, cmean, crrmn, dbhmn, shlm, dcmn, acmn,
c    & crcl, cacl, tlty, tlai, tbai, utot, uuz, tsz, tvz,
c    & ggsu, ggvu, ggeu, tsst, tsvt, tmass)
c
      include 'par.h'
      include 'stands.h'
      common /pidr/ pi, dr
c      
      dimension uuz(nzmax), rlaiz(nzmax), tsz(nzmax), tvz(nzmax)
c
      data thseff/.698d0/
c       
      dz = 0.1d0
      cths = cos(ths)
      cthv = cos(thv)
      sthe = sin(thseff)
      cthe = cos(thseff)
      tgthe = sthe/cthe
c                                             ! total values
      sntr   = 0.d0                           ! #_of_trees
      tmass  = 0.d0                           ! total leaf mass
      tvol   = 0.d0                           ! total crown volume
      tlai   = 0.d0
      tbai   = 0.d0
      tlty   = 0.d0
      utot   = 0.d0
c                                             ! mean values
      htrmn  = 0.d0                           ! h_tree
      htrmax = 0.d0
      crhmn  = 0.d0                           ! crown length
      cilhmn = 0.d0
      crrmn  = 0.d0                           ! crown radius
      dbhmn  = 0.d0                           ! DBH
      shlm   = 0.d0                           ! shl
      hcbmin = 50.d0
c                                             ! LIBERTY parameters
      dcmn   = 0.d0
      acmn   = 0.d0
c
      scrtot = 0.d0
      scacl  = 0.d0
c
      do jz = 1, nzmax
         uuz(jz) = 0.d0
        tsz(jz) = 1.d0
        tvz(jz) = 1.d0
      enddo
c                                            ! total values, mean values
      clmpj  = clmpst
      do jcl = 1, ncl
        lellj = lelli(jcl)
        stdj  = stdns(jcl)
        rcrj  = rcr(jcl)
        scrj  = pi*rcrj**2
        htrj  = htr(jcl)
        if (htrj .gt. htrmax) htrmax = htrj
        hc1j  = hc1(jcl)
        hc2j  = hc2(jcl)
        crhj  = hc1j + hc2j
        hcbj  = htrj - crhj
        if (hcbj .lt. hcbmin) hcbmin = hcbj
        dbhj  = dbh(jcl)
        rmj   = rmass(jcl)
        slwj  = slwcl(jcl)
        cshj  = clmpsh(jcl)
c
        sntr   = sntr + stdj
        tmass  = tmass + stdj*rmj
c                                           ! crown volume
        if (lellj) then
          volj = (2.d0/3.d0)*scrj*hc1j
        else
          volj = scrj*(hc1j/3.d0 + hc2j)
        endif
        tvol   = tvol + stdj*volj
c                                           ! leaf area & leaf area density
        rltree = rmj*1.d3/slwj
        baitree = rbai(jcl)*rltree
        rlaij  = rmj*1.d3/slwj*stdj
        rlai(jcl) = rlaij
        tlai   = tlai + rlaij
        baij = rbai(jcl)*rlaij
        tbai   = tbai + baij*stdj
        utot   = utot + rlaij*cshj + baij
        uuu(jcl) = (rltree + baitree)/volj
c
        htrmn  = htrmn + stdj*htrj
        crhmn  = crhmn + stdj*crhj
        cilhmn = cilhmn + stdj*hc2j
        crrmn  = crrmn + stdj*rcrj
        dbhmn  = dbhmn + stdj*dbhj
        tlty   = tlty + stdj*dbhj*(htrj - crhj)
        shlm   = shlm + rlaij*shl(jcl)
c                                           ! cacl & crcl
c                       ! the regularity parameter of the 1st tree class 
c                       ! is used for all classes
        crclj  = stdj*scrj
        scrtot = scrtot + crclj
        xxy = stdj*clmpj*scrj
        caclj  = 1.d0 - exp(-xxy)
        if (caclj .gt. crclj) then
          wvliit = crclj
          wcano  = caclj
          jwcl = jcl
        endif
        scacl  = scacl + clmpj*crclj
c                                           ! leaf area projection
        thmj  = thm3(jcl)
        elnj  = eln3(jcl)
        eej  = 1.d0 - exp(-elnj)
        ellmyj = abs(.059d0*eln3(jcl) *(thmj - 1.02d0) + .02d0)
        ellsij = abs(.0107d0 - .0216d0*eln3(jcl)*(thmj - .265d0))
        call ggg1(ths, thmj, eej, ellmyj, ellsij, ggs)
        gg3sj = ggs*cshj
        gg3s(jcl) = gg3sj
        call ggg1(thv, thmj, eej, ellmyj, ellsij, ggv)
        gg3vj = ggv*cshj
        gg3v(jcl) = gg3vj
      enddo
c
      htrmn  = htrmn/sntr
      crhmn  = crhmn/sntr
      cilhmn = cilhmn/sntr
      crrmn  = crrmn/sntr
      dbhmn  = dbhmn/sntr
      shlm   = shlm/tlai
      crcl   = scrtot
      cacl   = 1.d0 - exp(-scacl)
*     uumn   = (tlai + tbai)/tvol
      utot   = tlai + tbai
      tbai   = tbai/sntr
      tmass  = tmass
      tlty   = tlty*pi/2.0d0
*     dcmn   = dcmn/tlai
*     acmn   = acmn/tlai
c
************** Vertical profile of leaf area (18.12.2012) ****************
c              step = dz m
*     jzmax = int(htrmax/dz) + 1
      jzmax = int(htrmax/dz)
      jzmin = max(int(hcbmin/dz)-1, 1)
      do jz = nzmax, jzmax+1, -1
        rlaiz(jz) = 0.d0
      enddo 
      sumlai = 0.d0
      ugsz = 0.d0
      ugvz = 0.d0
      ugez = 0.d0
      do jz = jzmax, jzmin, -1
        zj = jz*dz
        ugs = 0.d0
        ugv = 0.d0
        uge = 0.d0
        do jcl = 1, ncl
          rcrj   = rcr(jcl)
          htrj   = htr(jcl)
          hc1j   = hc1(jcl)
          hc2j   = hc2(jcl)
          if (lelli(jcl)) then
            zmin   = htrj - hc1j
          else 
            zmin   = htrj - hc1j - hc2j
          endif
c                                           ! leaf area projection
          thmj  = thm3(jcl)
          elnj  = eln3(jcl)
          cshj  = clmpsh(jcl)
          eej  = 1.d0 - exp(-elnj)
          ellmyj = abs(.059d0*eln3(jcl) *(thmj - 1.02d0) + .02d0)
          ellsij = abs(.0107d0 - .0216d0*eln3(jcl)*(thmj - .265d0))
          call ggg1(ths, thmj, eej, ellmyj, ellsij, ggs)
          gg3sj = ggs*cshj
          gg3s(jcl) = gg3sj
          call ggg1(thv, thmj, eej, ellmyj, ellsij, ggv)
          gg3vj = ggv*cshj
          gg3v(jcl) = gg3vj
          call ggg1(thseff, thmj, eej, ellmyj, ellsij, gge)
          gg3ej = gge*cshj
c    
          if (zj .gt. zmin .and. zj .lt. htrj) then
            if (lelli(jcl)) then
               cellj  = hc1j/2.d0
               z2 = zj - (zmin + htrj)/2.d0
               rz = rcrj*sqrt(cellj*cellj - z2*z2)/cellj
            else
               if (zj .lt. htrj-hc1j) then
                  rz = rcrj
               else
                  rz = rcrj*(htrj - zj)/hc1j
               endif
            endif
            uz = pi*rz*rz*uuu(jcl)*stdns(jcl)*0.1d0
            uuz(jz) = uuz(jz) + uz
            ugs = ugs + uz*gg3sj
            ugv = ugv + uz*gg3vj
            uge = uge + uz*gg3ej
          endif
        enddo
        sumlai = sumlai + uuz(jz)
        rlaiz(jz) = sumlai
        ugsz = ugsz + ugs
        ugvz = ugvz + ugv
        ugez = ugez + uge
        tsz(jz) = exp(-ugsz*clmpj/cths)
        tvz(jz) = exp(-ugvz*clmpj/cthv)
      enddo
      ggsu = ugsz
      ggvu = ugvz
      ggeu = ugez
      tsst = tsz(jzmin)
      tsvt = tvz(jzmin)
      do jz = 1, jzmin-1
        rlaiz(jz) = rlaiz(jzmin)
        tsz(jz) = tsst
        tvz(jz) = tsvt
      enddo
c
      plevel = 0.05d0
      p1 = plevel
      p2 = 1.d0 - plevel
      do jz = 1, jzmax
*       write (*, '(f4.1, 4f10.5)') jz/10.d0, uuz(jz), rlaiz(jz),
*    &  tsz(jz), tvz(jz)
        if (rlaiz(jz)/rlaiz(1) .ge. p2) zmn2 = jz*dz
        if (rlaiz(jz)/rlaiz(1) .ge. p1) zmn1 = jz*dz
      enddo
      zcctr = (zmn1 + zmn2)/2.d0
      cmean = (zmn1 - zmn2)/2.d0
*     write (*, '(a, 2f6.1)') 'zmn1, zmn2: ', zmn1, zmn2
c
c                               LAI_eff for diffuse fluxes (09.2002) 
c                               thseff = 40° = .698 rad
      tgthx = tgthe
      zzx = 0.d0
      cell = crhmn/2.d0
      aell = crrmn
      vzx = 4.d0/3.d0*pi*cell*aell**2
      ugx = ggeu
      call pcrown
     &  (tgthx, zzx, zcctr, cell, aell, ugx, 
     &   vzx, sproj, rmnz, azxu)
      sze = sproj
      b11e = bij(azxu, glmp)
      alec = sntr*sze*b11e
      efflai = cthe*alec*tlai/ggeu
      ggmean = efflai/tlai
      return
      end
*
************************************************************************
*
