      subroutine sixd
     &      (jpass, iwrs, oufile, rlambda, 
     &         thss, iaer, dcaer, dvis, dtaer55, sqjwl)
c
c     program sixd(input,output,tape5,tape6)
c**********************************************************************c
c                                                                      c
c     Diffuse component of the 6s,    A. Kuusk  11.10.1995, 27.03.2023 c
c                                                        andres@bgf.ee c
c       ********************************************************       c
c       *          second  simulation of satellite signal      *       c
c       *                in the solar spectrum                 *       c
c       *       ... (6s.. ....... (6s.1)....... (6s.1)..       *       c
c       *             .... (6s.1)........(6s.1)...             *       c
c       *                  ...... (6s.1)......                 *       c
c       *              version 3.1                             *       c
c       *                                                      *       c
c       *  this code predicts the satellite signal from 0.25   *       c
c       *  to 4.0 microns assuming cloudless atmosphere.       *       c
c       *  the main atmospheric effects (gaseous absorption    *       c
c       *  by water vapor,carbon dioxyde,oxygen and ozone;     *       c
c       *  scattering by molecules and aerosols) are taken     *       c
c       *  into account. non-uniform surfaces may be           *       c
c       *  considered,as well as bidirectional reflectances    *       c
c       *            as boundary conditions                    *       c
c       *                                                      *       c
c       *   the following input parameters are needed          *       c
c       *         geometrical conditions                       *       c
c       *         atmospheric model for gaseous components     *       c
c       *         aerosol model (type and concentration)       *       c
c       *         spectral condition                           *       c
c       *         ground reflectance (type and spectral var.)  *       c
c       *   at each step, you can either select some proposed  *       c
c       *  standard conditions (for example,spectral bands of  *       c
c       *  satellite for spectral conditions) or define your   *       c
c       *  own conditions(in the example,you have to define    *       c
c       *  the assumed spectral response).                     *       c
c       *                                                      *       c
c       *   more details are given at each data input step     *       c
c       *                                                      *       c
c       ********************************************************       c
c                                                                      c
c                                                                      c
c**********************************************************************c

c**********************************************************************c
c                                                                      c
c                                                                      c
c       ********************************************************       c
c       *             authors of this code are                 *       c
c       *                                                      *       c
c       *  (1) Deuze j.l.;(1) Herman m.;(1),(2) Morcrette j.j. *       c
c       *               (1) Tanre d.; (1),(3) Vermote e.       *       c
c       *                                                      *       c
c       *                       from                           *       c
c       *                                                      *       c
c       *     (1) laboratoire d' optique atmospherique         *       c
c       *         universite des sciences et techniques        *       c
c       *         de lille                                     *       c
c       *         u.e.r. de physique fondamentale              *       c
c       *         59655 villeneuve d' ascq cedex               *       c
c       *         france                                       *       c
c       *                                                      *       c
c       *     (2) e.c.m.w.f.                                   *       c
c       *                                                      *       c
c       *     (3) code 923 / gimms group                       *       c
c       *         gsfc/nasa                                    *       c
c       *         greenbelt, md 20771                          *       c
c       *         usa                                          *       c
c       *                                                      *       c
c       *                                                      *       c
c       *                                                      *       c
c       ********************************************************       c
c                                                                      c
c                                                                      c
c**********************************************************************c


c**********************************************************************c
c       ********************************************************       c
c       *                limits of validity                    *       c
c       *                                                      *       c
c       *   geometrical parameters    solar zenith angle and   *       c
c       *                             satellite zenith angle   *       c
c       *                             must be less than 60 and *       c
c       *                             50 degrees respectively. *       c
c       *                                                      *       c
c       *   atmospheric model         no limitations           *       c
c       *                                                      *       c
c       *   aerosol model             the visibility must be   *       c
c       *                             better than 5.0km        *       c
c       *                             for smaller values       *       c
c       *                             calculations might be    *       c
c       *                             no more valid.           *       c
c       *                                                      *       c
c       *   spectral conditions       the gaseous transmittance*       c
c       *                             and the scattering func  *       c
c       *                             tions are valid from 0.25*       c
c       *                             to 4.0 micron. but the   *       c
c       *                             treatment of interaction *       c
c       *                             between absorption and   *       c
c       *                             scattering is correct for*       c
c       *                             not too large absorption *       c
c       *                             if you want to compute   *       c
c       *                             signal within absorption *       c
c       *                             bands,this interaction   *       c
c       *                             ought to be reconsidered *       c
c       *                                                      *       c
c       ********************************************************       c
c**********************************************************************c

c****************************************************************************c
c  For considering brdf, we have to compute the downward radiance in the     c
c  whole hemisphere. To perform such computions, we selected the successive  c
c  orders of scattering method. That method requires numerical integration   c
c  over angles and optical depth. The integration method is the gauss method,c
c  mu is the number of angles nmu+1, nmu is settled to 24. The accuracy of   c
c  the computations is obviously depending on the nmu value. This value      c
c  can be easily changed as a parameter as well as the nt value which        c
c  is the number of layers for performing the vertical integration. The      c
c  downward radiance is computed for nmu values of the zenith angle and np   c
c  values of the azimuth angle. The integration of the product of the        c
c  radiance by the brdf is so performed over the nmu*np values. np is settledc
c  to 13, that value can be also changed. mu2 is equal to 2 times nmu.       c
c  xlmus is the downward radiance, xf the downward irradiance, rm and gb     c
c  the angles and the weights for the gauss integration over the zenith, rp  c
c  and bp respectively for the azimuth integration.                          c
c****************************************************************************c

      parameter(nt_p=13,mu_p=13,mu2_p=24,np_p=13)
*     parameter (nbnds = 2002)
      dimension anglem(mu2_p),weightm(mu2_p),
     s   rm(-mu_p:mu_p),gb(-mu_p:mu_p),rp(np_p),gp(np_p)
      dimension angmu(10),angphi(13), 
     s    xlm1(-mu_p:mu_p,np_p),
     s    xlm2(-mu_p:mu_p,np_p)
c
      double precision rlambda, dvis, dtaer55, thss
      double precision sqjwl
      double precision dcaer
      dimension dcaer(4)
c
      character*12 oufile
      real anglem,weightm,rm,gb
      real rp,gp,angmu,angphi
      real xlm1,xlm2
      real c,wldisc,rocl
      real xacc,wlinf,wlsup,delta
      real sigma,z,p,t,roatm,dtdir
      real dtdif,utdir,utdif,sphal,wldis,trayl,traypl,pi,pi2,step
      real asol,phi0,avis,phiv
      real phi,phirad,xmus,xmuv,xmup,xmud,taer55
      real taer,v,taerp55
      real wl,wlmoy,tamoy,tamoyp,pizmoy,pizmp,trmoy
      real adtwava,dtozon,dtdica,dtoxyg
      real dtniox,dtmeth,dtmoca,autwava,utozon,utdica,utoxyg,utniox
      real utmeth,utmoca,attwava,ttozon,ttdica,ttoxyg,ttniox
      real ttmeth,ttmoca,dtwava,utwava,ttwava,romix,rorayl
      real roaero,phaa,phar,tsca,tray,trayp,taerp,dtott,utott
      real astot,asray,asaer,utotr,utota,dtotr,dtota
      real tdir,tdif
      integer nt,mu,mu2,np,k,iwr,mum1,idatmp
      integer j,l,idatm,iaer,n
      integer iinf,i,igroun
c***********************************************************************
c                             return to 6s
c***********************************************************************
      dimension c(4),wldisc(10)
      dimension rocl(1501)

      logical ier
      common/sixs_ier/iwr,ier
c***********************************************************************
c     for considering pixel and sensor  altitude
c***********************************************************************
      real palt,ftray
      common /sixs_test/xacc
c***********************************************************************
c                             return to 6s
c***********************************************************************
      common /sixs_ffu/wlinf,wlsup
      common /sixs_del/ delta,sigma
      common /sixs_atm/z(34),p(34),t(34)
      common /six_disc/ roatm(3,10),dtdir(3,10),dtdif(3,10),
     s utdir(3,10),utdif(3,10),sphal(3,10),wldis(10),trayl(10),
     s traypl(10)


c****************************************************************************c
c   angmu and angphi are the angles were the brdf is measured. these values  c
c   can be changed as soon as they are well distributed over the whole space c
c   before the gauss integration, these values are interpolated to the gauss c
c   angles                                                                   c
c****************************************************************************c
      data angmu /85.0,80.0,70.0,60.0,50.0,40.0,30.0,20.0,10.0,0.00/
      data angphi/0.00,30.0,60.0,90.0,120.0,150.0,180.0,
     &          210.0,240.0,270.0,300.0,330.0,360.0/
*     double precision dr
*     data dr/1.745329251994330d-2/

c***********************************************************************
c                             return to 6s
c***********************************************************************
      data wldisc /0.400,0.488,0.515,0.550,0.633,
     &             0.694,0.860,1.536,2.250,3.750/

c***********************************************************************
c   Parameters  initialization
c***********************************************************************
      nt=nt_p
      mu=mu_p
      mu2=mu2_p
      np=np_p
      iwr = iwrs
      ier=.FALSE.
c***********************************************************************
c  preliminary computations for gauss integration
c***********************************************************************
      pi=acos(-1.)
      pi2=2*pi
      do k=1,13
         angphi(k)=angphi(k)*pi/180.
      enddo
      do k=1,10
         angmu(k)=cos(angmu(k)*pi/180.)
      enddo
      call gauss(-1.,1.,anglem,weightm,mu2)
      call gauss(0.,pi2,rp,gp,np)
      mum1=mu-1
      do 581 j=-mum1,-1
         k=mu+j
         rm(-j-mu)=anglem(k)
         gb(-j-mu)=weightm(k)
  581 continue
      do 582 j=1,mum1
         k=mum1+j
         rm(mu-j)=anglem(k)
         gb(mu-j)=weightm(k)
  582 continue
      gb(-mu)=0.
      gb(0)=0.
      gb(mu)=0.

c***********************************************************************
c                             return to 6s
c***********************************************************************
c constantes values
      sigma=0.056032
      delta=0.0279
      xacc=1.e-06
      step=0.0025
      do 1111 l=1,10
         wldis(l)=wldisc(l)
 1111 continue

c**********************************************************************c
c       igeom               geometrical conditions                     c
c               --------------------------------------                 c
c                                                                      c
c   you choose your own conditions; igeom=0                            c
c         0     enter solar zenith angle   (in degrees )               c
c                     solar azimuth angle        "                     c
c                     satellite zenith angle     "                     c
c                     satellite azimuth angle    "                     c
c                                                                      c
c**********************************************************************c
c     igeom=0.....
c
      asol = real(thss)
      avis = 0.
      phi0 = 0.
      phiv = 0.
c
      continue

c**********************************************************************c
c                                                                      c
c                                 / scattered direction                c
c                               /                                      c
c                             /                                        c
c                           / adif                                     c
c    incident   + + + + + + + + + + + + + + +                          c
c    direction                                                         c
c                                                                      c
c**********************************************************************c
      phi=abs(phiv-phi0)
      phirad=(phi0-phiv)*pi/180.
      if (phirad.lt.0.) phirad=phirad+2.*pi
      if (phirad.gt.(2.*pi)) phirad=phirad-2.*pi
      xmus=cos(asol)
      xmuv=cos(avis*pi/180.)
      xmup=cos(phirad)
      xmud=-xmus*xmuv-sqrt(1.-xmus*xmus)*sqrt(1.-xmuv*xmuv)*xmup
c test vermote bug
      if (xmud.gt.1.) xmud=1.
      if (xmud.lt.-1.) xmud=-1.

c**********************************************************************c
c       idatm      atmospheric model                                   c
c                 --------------------                                 c
c                                                                      c
c  you select one of the following standard atmosphere: idatm=0 to 6   c
c         0    no gaseous absorption                                   c
c**********************************************************************c
      idatm = 0
c     we have to define an atmosphere to compute rayleigh optical depth
      if(idatm.eq.0.or.idatm.eq.8)  call us62

c**********************************************************************c
c                                                                      c
c       iaer       aerosol model(type)                                 c
c                  --------------                                      c
c                                                                      c
c  you select one of the following standard aerosol models: iaer=0to3  c
c         0  no aerosols                                               c
c         1  continental model  )                                      c
c         2  maritime model     )  according to sra models             c
c         3  urban model        )                                      c
c                                                                      c
c  or you define your own model: iaer=4                                c
c                                                                      c
c         4 enter the volumic percentage of each component             c
c                 c(1) = volumic % of dust-like                        c
c                 c(2) = volumic % of water-soluble                    c
c                 c(3) = volumic % of oceanic                          c
c                 c(4) = volumic % of soot                             c
c                   between 0 to 1                                     c
c                                                                      c
c**********************************************************************c

      if (iaer .ne. 4) goto 8
      do n = 1, 4
         c(n) = real(dcaer(n))
      enddo      
    8 continue

      goto (43, 40, 41, 42, 43), iaer+1
 
   40 c(1) = 0.70
      c(2) = 0.29
      c(3) = 0.00
      c(4) = 0.01
      goto 43
   41 c(1) = 0.00
      c(2) = 0.05
      c(3) = 0.95
      c(4) = 0.00
      goto 43
   42 c(1) = 0.17
      c(2) = 0.61
      c(3) = 0.00
      c(4) = 0.22
   43 call aeroso(iaer, c, xmud)

 

c**********************************************************************c
c              aerosol model (concentration)                           c
c              ----------------------------                            c
c                                                                      c
c                                                                      c
c   you have an estimate of the meteorological parameter: the visibi   c
c   lity v,                                                            c
c            enter directly the value of v in km(the aerosol optical   c
c            depth will be computed from a standard aerosol profile)   c
c                                                                      c
c   or you have an estimate of aerosol optical depth ,enter v=0 for    c
c            the visibility and enter the aerosol optical depth at 550 c
c                                                                      c
c   warning:  if iaer=0, enter v=-1                                    c
c**********************************************************************c
      taer55=0.
      taerp55=0.
      taer=0.
      ftray = 1.
      if (iaer .eq. 0) then
         v = -1
      else
         v = real(dvis)
      endif
c
      if (abs(v) .lt. 0.00001) then
        taer55 = real(dtaer55)
        taerp55 = taer55
        v   = exp(-log(taer55/2.7628)/0.79902)
      else if (v .gt. 0.0) then
        call oda550(iaer, v, taer55)
      endif
      continue
 
 
         palt=0.
       idatmp=0

c**********************************************************************c
c      iwave input of the spectral conditions                          c
c            --------------------------------                          c
c                                                                      c
c  you choose to define your own spectral conditions: iwave=-1,0 or 1  c
c                   (three user s conditions )                         c
c        -1  enter wl (monochr. cond,  gaseous absorption is included) c
c                                                                      c
c  note: wl has to be in micrometer                                    c
c**********************************************************************c
      continue
c
c
*     print *, 'sixs: iknot = ', ii1
*     do jwloc = 1,  nbnds
*     if (mod(jwloc, 20) .eq. 0) print *, 'sixs:    jwl = ', jwloc
      wloc = real(rlambda/1.d+3)
      wlinf=wloc
      wlsup=wloc
      iinf=int((wlinf-.245)/0.0025+0.5)

c**********************************************************************c
c Here, we first compute an equivalent wavelenght which is the input   c
c value for monochromatic conditions or the integrated value for a     c
c filter function (call equivwl) then, the atmospheric properties are  c
c computed for that wavelength (call discom then call specinterp)      c
c molecular optical thickness is computed too (call odrayl). Lastly    c
c the successive order of scattering code is called three times.       c
c First for a sun at thetas with the scattering properties of aerosols c
c and molecules, second with a pure molecular atmosphere, then with thec
c actual atmosphere for a sun at thetav. The iso code allows us to     c
c compute the scattering transmissions and the spherical albedo. All   c
c these computations are performed for checking the accuracy of the    c
c analytical expressions and in addition for computing the averaged    c
c directional reflectances                                             c
c**********************************************************************c
         wlmoy=wloc
      call discom (idatmp,iaer,xmus,xmuv,phi
     &            ,taer55,taerp55,palt,
     &           phirad,nt,mu,np,rm,gb,rp
     &         ,ftray,xlm1,xlm2)
      if(iaer.ne.0) then
         call specinterp(wlmoy,taer55,taerp55,
     &     tamoy,tamoyp,pizmoy,pizmp)
      endif
      call odrayl(wlmoy,
     &                   trmoy)
      if (idatmp.eq.4) then
          tamoyp=tamoy
      endif
      if (idatmp.eq.0) then
         tamoyp=0.
      endif

c**********************************************************************c
c     uniform surface with lambertian conditions                       c
c**********************************************************************c
             igroun = 1
      if(igroun.eq.1) call vegeta(rocl)
      l = iinf


c**********************************************************************c
c                     print of initial conditions                      c
c                                                                      c
c**********************************************************************c
      if (jpass .eq. 1) then
c       geometrical conditions
c       atmospheric model
c       aerosols model (type)
c
        open (iwr, file = oufile, status = 'old')
          do j = 1, 6000
             read (iwr, '(1x)', end = 65)
          enddo
65        continue
          backspace iwr
           if (iaer.lt.4) then
              goto(230,231,232,233),iaer+1
           else
              if (iaer.ge.5) goto(234),iaer-4
              write(iwr, 134)(c(i),i=1,4)
              goto 249
           endif
  234      write(iwr, 131)' desertic'
           goto 249
  233      write(iwr, 131)' urban'
           goto 249
  232      write(iwr, 131)'maritime'
           goto 249
  231      write(iwr, 131)'continental'
           goto 249
  230      write(iwr, 1301)
 1301 format('#',12x,' aerosols type identity :',
     &                 ' no aerosols computed   ',t79,'*')
  131 format('#',12x,' aerosols type identity :',1x,
     &                 a11,' aerosols model',t79,'*')
  134 format('#',12x,' aerosols type identity :',
     &               ' user defined aerosols model ',t79,'*',/,
     &  '#',36x,f6.3,' % of dust-like',t79,'*',/,
     &  '#',36x,f6.3,' % of water-soluble',t79,'*',/,
     &  '#',36x,f6.3,' % of oceanic',t79,'*',/,
     &  '#',36x,f6.3,' % of soot',t79,'*')
  249    continue

c     aerosol model (concentration)
         if(iaer.eq.0) write(iwr, 1401)
         if(iaer.eq.0) goto 1112
         if(abs(v).le.xacc) write(iwr, 140)taer55
         if(abs(v).gt.xacc) write(iwr, 141)v,taer55
  140 format('#',5x,' optical condition identity :',t79,'*',/,
     &       '#',5x,' user def. opt. thick. at 550nm', f7.4,
     &       t79,'*',/,'*',t79,'*')
  141 format('#',24x,' optical condition identity :',t79,'*',/,
     &       '#',14x,' visibility ',f6.2,' km ',
     &                 ' opt. thick. 550nm ',f7.4,t79,'*')
c
c     spectral condition
 1112       continue


c pressure at ground level (174) 
         write(iwr, 174)p(1)
  174 format('#',22x,' ground pressure [mb] ', f7.2,1x,t79,'*')
        close (iwr)
      endif

c**********************************************************************c
c                                                                      c
c                                                                      c
c                     start of computations                            c
c                                                                      c
c                                                                      c
c                                                                      c
c**********************************************************************c

c                                    ! Loop for wavelength
c
      l = iinf
         wl=.2475+(l-1)*step
c
         call abstra(idatm,
     &      adtwava,dtozon,dtdica,dtoxyg,dtniox,dtmeth,dtmoca,
     &      autwava,utozon,utdica,utoxyg,utniox,utmeth,utmoca,
     &      attwava,ttozon,ttdica,ttoxyg,ttniox,ttmeth,ttmoca )
c
         call abstra(idatm,
     &      dtwava,dtozon,dtdica,dtoxyg,dtniox,dtmeth,dtmoca,
     &      utwava,utozon,utdica,utoxyg,utniox,utmeth,utmoca,
     &      ttwava,ttozon,ttdica,ttoxyg,ttniox,ttmeth,ttmoca )

c
         call interp (iaer,idatmp,wl,taer55,taerp55,xmud,
     &             romix,rorayl,roaero,phaa,phar,tsca,
     &             tray,trayp,taer,taerp,dtott,utott,
     &             astot,asray,asaer,
     &             utotr,utota,dtotr,dtota)
c
c     output at the ground level.
c
         tdir=exp(-(tray+taer)/xmus)
         tdif=dtott-tdir
*        jwl = nint((wloc*1000. - (400. - real(wlstep)))/real(wlstep))
*        sqarr(jwl) = dble(tdir/(tdir + tdif))
         sqjwl = dble(tdir/(tdir + tdif))
c
*     enddo
      return

c**********************************************************************c
c                   output editing formats                             c
c**********************************************************************c

 1401 format('*',t79,'*')
      end
*
************************************************************************
*
        subroutine abstra (idatm,
     a      dtwava,dtozon,dtdica,dtoxyg,dtniox,dtmeth,dtmoca,
     a      utwava,utozon,utdica,utoxyg,utniox,utmeth,utmoca,
     a      ttwava,ttozon,ttdica,ttoxyg,ttniox,ttmeth,ttmoca )
 
c     transmittance calculation for ozone, water vapor,
c     carbon dioxyde and oxygen.
 
c     downward absorption water vapor dtwava
c     downward absorption ozone       dtozon
c     downward absorption carbon diox dtdica
c     downward absorption oxygen      dtoxyg
c     downward absorption nitrous oxi dtniox
c     downward absorption methane     dtmeth
c     downward absorption carbon mono dtmoca
c     upward absorption water vapor   utwava
c     upward absorption ozone         utozon
c     upward absorption carbon diox   utdica
c     upward absorption oxygen        utoxyg
c     upward   absorption nitrous oxi utniox
c     upward   absorption methane     utmeth
c     upward   absorption carbon mono utmoca
c     total(on the two paths ) absorption water vapor ttwava
c     total(on the two paths ) absorption ozone       ttozon
c     total(on the two paths ) absorption carbon diox ttdica
c     total(on the two paths ) absorption oxygen      ttoxyg
c     total    absorption nitrous oxi ttniox
c     total    absorption methane     ttmeth
c     total    absorption carbon mono ttmoca
 
      integer idatm
      real dtoxyg,dtniox,dtmeth,dtmoca,dtwava,dtozon,dtdica
      real utniox,utdica,utoxyg,utwava,utozon
      real utmeth,utmoca,ttwava,ttozon,ttdica,ttoxyg,ttniox,ttmeth
      real ttmoca
C
      dtwava=1.
      utwava=1.
      ttwava=1.
      dtozon=1.
      utozon=1.
      ttozon=1.
      dtdica=1.
      utdica=1.
      ttdica=1.
      dtoxyg=1.
      utoxyg=1.
      ttoxyg=1.
      dtniox=1.
      utniox=1.
      ttniox=1.
      dtmeth=1.
      utmeth=1.
      ttmeth=1.
      dtmoca=1.
      utmoca=1.
      ttmoca=1.
 
      if (idatm.eq.0) return
      call print_error(
     s 'Abstr0: no absorption in this subroutine')
      return
      end
*
************************************************************************
*
      subroutine aeroso (iaer,co,xmud)
      double precision cij(4),vi(4),nis,sumni,ni(4),vi2(1),ni2(1)
      real co(4),dd(4,10),ci(4),ex(4,10),sc(4,10),asy(4,10)
      real pha(4,10,83)
      real ex2(1,10),sc2(1,10),asy2(1,10)
      real xmud,ext,ome,gasym,phase,ph,phasel,cgaus
      real coef,sigm,sca
      integer i,j,k,l,j1,j2,iaer
c     sra basic components for aerosol model, extinction coefficients
c     are in km-1.
 
c     dust-like = 1
c     water-soluble = 2
c     oceanique = 3
c     soot = 4
 
      data vi /113.983516d0,113.983516d-06,5.1444150196d0,
     a          59.77353425d-06/
      data ni /54.734,1.86855d+06,276.05,1.80582d+06/
      data vi2 /6497.9973/     
      data ni2 /2.4477d-02/

 
c     i: 1=continental 2=maritime 3=urban  4=user
c     j: 1=dust-like 2=water-soluble 3=oceanic 4=soot
 
      data ((ex(i,j),sc(i,j),j=1,10),i=1,1) /
     a 0.1796674e-01,0.1126647e-01,0.1815135e-01,0.1168918e-01,
     a 0.1820247e-01,0.1180978e-01,0.1827016e-01,0.1196792e-01,
     a 0.1842182e-01,0.1232056e-01,0.1853081e-01,0.1256952e-01,
     a 0.1881427e-01,0.1319347e-01,0.1974608e-01,0.1520712e-01,
     a 0.1910712e-01,0.1531952e-01,0.1876025e-01,0.1546761e-01/
      data ((ex(i,j),sc(i,j),j=1,10),i=2,2) /
     a 0.7653460e-06,0.7377123e-06,0.6158538e-06,0.5939413e-06,
     a 0.5793444e-06,0.5587120e-06,0.5351736e-06,0.5125148e-06,
     a 0.4480091e-06,0.4289210e-06,0.3971033e-06,0.3772760e-06,
     a 0.2900993e-06,0.2648252e-06,0.1161433e-06,0.9331806e-07,
     a 0.3975192e-07,0.3345499e-07,0.1338443e-07,0.1201109e-07/
      data ((ex(i,j),sc(i,j),j=1,10),i=3,3) /
     a 0.3499458e-02,0.3499455e-02,0.3574996e-02,0.3574993e-02,
     a 0.3596592e-02,0.3596591e-02,0.3622467e-02,0.3622465e-02,
     a 0.3676341e-02,0.3676338e-02,0.3708866e-02,0.3708858e-02,
     a 0.3770822e-02,0.3770696e-02,0.3692255e-02,0.3677038e-02,
     a 0.3267943e-02,0.3233194e-02,0.2801670e-02,0.2728013e-02/
      data ((ex(i,j),sc(i,j),j=1,10),i=4,4) /
     a 0.8609083e-06,0.2299196e-06,0.6590103e-06,0.1519321e-06,
     a 0.6145787e-06,0.1350890e-06,0.5537643e-06,0.1155423e-06,
     a 0.4503008e-06,0.8200095e-07,0.3966041e-06,0.6469735e-07,
     a 0.2965532e-06,0.3610638e-07,0.1493927e-06,0.6227224e-08,
     a 0.1017134e-06,0.1779378e-08,0.6065031e-07,0.3050002e-09/
 
       data ((ex2(i,j),sc2(i,j),j=1,10),i=1,1) /
     A 0.4383631E+02,0.4028625E+02,0.4212415E+02,0.3904473E+02,        
     A 0.4157425E+02,0.3861470E+02,0.4085399E+02,0.3803645E+02,        
     A 0.3914040E+02,0.3661054E+02,0.3789763E+02,0.3554456E+02,        
     A 0.3467506E+02,0.3269951E+02,0.2459000E+02,0.2341019E+02,        
     A 0.1796726E+02,0.1715375E+02,0.1057569E+02,0.1009731E+02/        
 
      data ((asy(i,j),j=1,10),i=1,4) /
     a 0.896,0.885,0.880,0.877,0.867,0.860,0.845,0.836,0.905,0.871,
     a 0.642,0.633,0.631,0.628,0.621,0.616,0.610,0.572,0.562,0.495,
     a 0.795,0.790,0.788,0.781,0.783,0.782,0.778,0.783,0.797,0.750,
     a 0.397,0.359,0.348,0.337,0.311,0.294,0.253,0.154,0.103,0.055/
 
      data ((asy2(i,j),j=1,10),i=1,1)/
     A 0.718,0.712,0.710,0.708,0.704,0.702,0.696,0.680,0.668,0.649/    
 
 
      common /sixs_aer/ ext(10),ome(10),gasym(10),phase(10)
      common /sixs_aerbas/ ph(10,83)
      common /sixs_sos/phasel(10,83),cgaus(83)
c
c     optical properties of aerosol model computed from sra basic comp
 
      do l=1,10
        ext(l)=0.
        if(l.eq.4.and.iaer.eq.0) ext(l)=1.
        ome(l)=0.
        gasym(l)=0.
        phase(l)=0.
        do k=1,83
          phasel(l,k)=0.
        enddo
      enddo
 
      if(iaer.eq.0) return
 
      do 2 j=1,4
      ci(j)=co(j)
    2 continue
 
c
c
      if (iaer.ge.5) then
        do k=1,10
        asy(1,k)=asy2(iaer-4,k)
        ex(1,k)=ex2(iaer-4,k)
        sc(1,k)=sc2(iaer-4,k)
        enddo
        ci(1)=1.0
        ci(2)=0.0
        ci(3)=0.0
        ci(4)=0.0
        ni(1)=ni2(1)
        vi(1)=vi2(1)
      endif
c
 
      do 7 k=1,82
      if((xmud.ge.cgaus(k)).and.(xmud.lt.cgaus(k+1))) go to 8
    7 continue
      return
    8 j1=k
      j2=j1+1
      coef=-(xmud-cgaus(j1))/(cgaus(j2)-cgaus(j1))
c calling each sra components
c  -dust
         call dust
         do l=1,10
         dd(1,l)=ph(l,j1)+coef*(ph(l,j1)-ph(l,j2))
         do k=1,83
         pha(1,l,k)=ph(l,k)
         enddo
         enddo
c  -water soluble
         call wate
         do l=1,10
         dd(2,l)=ph(l,j1)+coef*(ph(l,j1)-ph(l,j2))
         do k=1,83
         pha(2,l,k)=ph(l,k)
         enddo
         enddo
c  -oceanic type
         call ocea
         do l=1,10
         dd(3,l)=ph(l,j1)+coef*(ph(l,j1)-ph(l,j2))
         do  k=1,83
         pha(3,l,k)=ph(l,k)
         enddo
         enddo
c  - soot
         call soot
         do l=1,10
         dd(4,l)=ph(l,j1)+coef*(ph(l,j1)-ph(l,j2))
         do k=1,83
         pha(4,l,k)=ph(l,k)
         enddo
         enddo
 
c     summ of the ci/vi calculation
 
      sumni=0.d0
      sigm=0.
      do i=1,4
        sigm=sigm+real(dble(ci(i))/vi(i))
      enddo
 
c     cij coefficients calculation
 
      do j=1,4
        cij(j)=dble(ci(j))/dble(sigm)/vi(j)
        sumni=sumni+cij(j)/ni(j)
      enddo
      nis=1.d+00/sumni
 
 
c     mixing parameters calculation
 
      do 5 l=1,10
      sca=0.
      do 6 j=1,4
      ext(l)=real(dble(ex(j,l))*cij(j))+ext(l)
      sca=real(dble(sc(j,l))*cij(j))+sca
      gasym(l)=real(dble(sc(j,l))*cij(j))*asy(j,l)+gasym(l)
      phase(l)=real(dble(sc(j,l))*cij(j)*dble(dd(j,l)))+phase(l)
      do 77 k=1,83
      phasel(l,k)=
     & real(dble(sc(j,l))*cij(j)*dble(pha(j,l,k)))+phasel(l,k)
   77 continue
    6 continue
      ome(l)=sca/ext(l)
      ext(l)=real(dble(ext(l))*nis)
      gasym(l)=gasym(l)/sca
      phase(l)=phase(l)/sca
      do 78 k=1,83
      phasel(l,k)=phasel(l,k)/sca
   78 continue
    5 continue
      return
      end
      block data aeroso_data
      common /sixs_sos/phasel(10,83),cgaus(83)
      real phasel,cgaus
      data cgaus/
     a-1.0000,-0.9996,-0.9976,-0.9942,-0.9893,-0.9828,-0.9749,-0.9655,
     a-0.9546,-0.9422,-0.9285,-0.9133,-0.8967,-0.8787,-0.8594,-0.8388,
     a-0.8170,-0.7938,-0.7695,-0.7440,-0.7174,-0.6896,-0.6609,-0.6311,
     a-0.6003,-0.5687,-0.5361,-0.5028,-0.4687,-0.4339,-0.3984,-0.3623,
     a-0.3257,-0.2885,-0.2510,-0.2130,-0.1747,-0.1362,-0.0974,-0.0585,
     a-0.0195, 0.0000, 0.0195, 0.0585, 0.0974, 0.1362, 0.1747, 0.2130,
     a 0.2510, 0.2885, 0.3257, 0.3623, 0.3984, 0.4339, 0.4687, 0.5028,
     a 0.5361, 0.5687, 0.6003, 0.6311, 0.6609, 0.6896, 0.7174, 0.7440,
     a 0.7695, 0.7938, 0.8170, 0.8388, 0.8594, 0.8787, 0.8967, 0.9133,
     a 0.9285, 0.9422, 0.9546, 0.9655, 0.9749, 0.9828, 0.9893, 0.9942,
     a 0.9976, 0.9996, 1.0000/
      end
*
************************************************************************
*
      subroutine atmref (iaer,tamoy,trmoy,pizmoy,tamoyp,trmoyp,palt,
     s               phi,xmus,xmuv,
     s               phirad,nt,mu,np,rm,gb,rp,
     a                   rorayl,roaero,romix,xlm1,xlm2)
      integer mu,np
      real rm(-mu:mu),rp(np),gb(-mu:mu),xlm1(-mu:mu,np)
      real xlm2(-mu:mu,np)
      real tamoy,trmoy,pizmoy,tamoyp,trmoyp,palt
      real phi,xmus,xmuv,phirad
      real rorayl,roaero,romix,tamol,tamolp
      integer iaer,nt
 
c     atmospheric reflectances
c
      rorayl=0.
      roaero=0.
c
c     rayleigh reflectance 3 cases (satellite,plane,ground)
      if(palt.lt.900..and.palt.gt.0.0)then
        rm(-mu)=-xmuv
        rm(mu)=xmuv
        rm(0)=-xmus
        tamol=0.
        tamolp=0.
      call os(tamol,trmoy,pizmoy,tamolp,trmoyp,palt,
     s               phirad,nt,mu,np,rm,gb,rp,
     s                     xlm1)
        rorayl=xlm1(mu,1)/xmus
        else
        if (palt.le.0.0) then
           rorayl=0.
           else
           call chand(phi,xmuv,xmus,trmoy,rorayl)
           endif
        endif
c
      if (iaer.eq.0) then
         romix=rorayl
         return
         endif
c
c     rayleigh+aerosol=romix,aerosol=roaero reflectance computed
c     using sucessive order of scattering method
c     3 cases: satellite,plane,ground
      if(palt.gt.0.0) then
        rm(-mu)=-xmuv
        rm(mu)=xmuv
        rm(0)=-xmus
c      write(6,*) "input OS",tamoy,trmoy,pizmoy,tamoyp,trmoyp,palt,
c    s phirad
        call os(tamoy,trmoy,pizmoy,tamoyp,trmoyp,palt,
     s               phirad,nt,mu,np,rm,gb,rp,
     s                     xlm2)
        romix=(xlm2(mu,1)/xmus)
        tamol=0.
        tamolp=0.
        call os(tamoy,tamol,pizmoy,tamoyp,tamolp,palt,
     s               phirad,nt,mu,np,rm,gb,rp,
     s                     xlm2)
        roaero=(xlm2(mu,1)/xmus)
        else
        roaero=0.
        romix=0.
        endif
c     write(6,*) " OS: ",rorayl,roaero,romix
      return
      end
*
************************************************************************
*
      subroutine chand (xphi,xmuv,xmus,xtau
     s                    ,xrray)
c input parameters: xphi,xmus,xmuv,xtau
c xphi: azimuthal difference between sun and observation (xphi=0,
c in backscattering) and expressed in degree (0.:360.)
c xmus: cosine of the sun zenith angle
c xmuv: cosine of the observation zenith angle
c xtau: molecular optical depth
c output parameter: xrray : molecular reflectance (0.:1.)
c constant : xdep: depolarization factor (0.0279)
      real xdep,pl(10)
      real fs0,fs1,fs2
      real as0(10),as1(2),as2(2)
        real xphi,xmus,fac,xmuv,xtau,xrray,pi,phios,xcosf1,xcosf2
        real xcosf3,xbeta2,xfd,xph1,xph2,xph3,xitm, xp1, xp2, xp3
        real cfonc1,cfonc2,cfonc3,xlntau,xitot1,xitot2,xitot3
        integer i
      data (as0(i),i=1,10) /.33243832,-6.777104e-02,.16285370
     s  ,1.577425e-03,-.30924818,-1.240906e-02,-.10324388
     s  ,3.241678e-02,.11493334,-3.503695e-02/
      data (as1(i),i=1,2) /.19666292, -5.439061e-02/
      data (as2(i),i=1,2) /.14545937,-2.910845e-02/
      pi=3.1415927
      fac=pi/180.
      phios=180.-xphi
      xcosf1=1.
      xcosf2=cos(phios*fac)
      xcosf3=cos(2*phios*fac)
      xbeta2=0.5
      xdep=0.0279
      xfd=xdep/(2-xdep)
      xfd=(1-xfd)/(1+2*xfd)
      xph1=1+(3*xmus*xmus-1)*(3*xmuv*xmuv-1)*xfd/8.
      xph2=-xmus*xmuv*sqrt(1-xmus*xmus)*sqrt(1-xmuv*xmuv)
      xph2=xph2*xfd*xbeta2*1.5
      xph3=(1-xmus*xmus)*(1-xmuv*xmuv)
      xph3=xph3*xfd*xbeta2*0.375
      xitm=(1-exp(-xtau*(1/xmus+1/xmuv)))*xmus/(4*(xmus+xmuv))
      xp1=xph1*xitm
      xp2=xph2*xitm
      xp3=xph3*xitm
      xitm=(1-exp(-xtau/xmus))*(1-exp(-xtau/xmuv))
      cfonc1=xph1*xitm
      cfonc2=xph2*xitm
      cfonc3=xph3*xitm
      xlntau=log(xtau)
      pl(1)=1.
      pl(2)=xlntau
      pl(3)=xmus+xmuv
      pl(4)=xlntau*pl(3)
      pl(5)=xmus*xmuv
      pl(6)=xlntau*pl(5)
      pl(7)=xmus*xmus+xmuv*xmuv
      pl(8)=xlntau*pl(7)
      pl(9)=xmus*xmus*xmuv*xmuv
      pl(10)=xlntau*pl(9)
      fs0=0.
      do i=1,10
      fs0=fs0+pl(i)*as0(i)
      enddo
      fs1=pl(1)*as1(1)+pl(2)*as1(2)
      fs2=pl(1)*as2(1)+pl(2)*as2(2)
      xitot1=xp1+cfonc1*fs0*xmus
      xitot2=xp2+cfonc2*fs1*xmus
      xitot3=xp3+cfonc3*fs2*xmus
      xrray=xitot1*xcosf1
      xrray=xrray+xitot2*xcosf2*2
      xrray=xrray+xitot3*xcosf3*2
      xrray=xrray/xmus
      return
      end
*
************************************************************************
*
      subroutine csalbr(xtau,xalb)
      real xtau,xalb,fintexp3
      xalb=(3*xtau-fintexp3(xtau)*(4+2*xtau)+2*exp(-xtau))
      xalb=xalb/(4.+3*xtau)
      return
      end
      real function fintexp3(xtau)
      real xx,xtau,fintexp1
      xx=(exp(-xtau)*(1.-xtau)+xtau*xtau*fintexp1(xtau))/2.
      fintexp3=xx
      return
      end
      real function fintexp1(xtau)
c accuracy 2e-07... for 0<xtau<1
      real xx,a(0:5),xtau,xftau
      integer i
      data (a(i),i=0,5) /-.57721566,0.99999193,-0.24991055,
     c                  0.05519968,-0.00976004,0.00107857/
      xx=a(0)
      xftau=1.
      do i=1,5
      xftau=xftau*xtau
      xx=xx+a(i)*xftau
      enddo
      fintexp1=xx-log(xtau)
      return
      end
*
************************************************************************
*
      subroutine discom (idatmp,iaer,xmus,xmuv,phi,
     a                   taer55,taerp55,palt,
     a                 phirad,nt,mu,np,rm,gb,rp,
     a                   ftray,xlm1,xlm2)
      integer mu,np
      real rm(-mu:mu),rp(np),gb(-mu:mu)
      real ftray,xlm1(-mu:mu,np),xlm2(-mu:mu,np)
      real xmus,xmuv,phi
      real taer55,taerp55,palt,phirad,ext,ome,gasym,phase,roatm
      real dtdir,dtdif,utdir,utdif,sphal,wldis,trayl,traypl
      real wlinf,wlsup,phasel,cgaus,pha,betal,wl,tray,trayp,taer
      real taerp,piza,tamoy,tamoyp,pizmoy,rorayl
      real roaero,romix,ddirtt,ddiftt,udirtt,udiftt,sphlbt,ddirtr
      real ddiftr,udirtr,udiftr,sphlbr,ddirta,ddifta,udirta,udifta
      real sphlba,coeff
      integer idatmp,iaer,nt,l,k
      common /sixs_aer/ext(10),ome(10),gasym(10),phase(10)
      common /six_disc/ roatm(3,10),dtdir(3,10),dtdif(3,10),
     a utdir(3,10),utdif(3,10),sphal(3,10),wldis(10),trayl(10),
     a traypl(10)
      common /sixs_ffu/wlinf,wlsup
      common /sixs_sos/phasel(10,83),cgaus(83)
      common /sixs_trunc/pha(83),betal(0:80)

c     computation of all scattering parameters at wavelength
c     discrete values,so we
c     can interpolate at any wavelength
 
      do 50 l=1,10
      wl=wldis(l)
      if ((wlsup.lt.wldis(1)).and.(l.le.2)) goto 30
      if (wlinf.gt.wldis(10).and.(l.ge.9)) goto 30
      if (l.lt.10) then
        if ((wldis(l).lt.wlinf).and.
     a     (wldis(l+1).lt.wlinf))
     a     goto 50
      endif
      if (l.gt.1) then
         if ((wldis(l) .gt. wlsup) .and. (wldis(l-1).gt. wlsup))
     a     goto 50
      endif
 
c     computation of rayleigh optical depth at wl
 
 30   call odrayl(wl,
     a           tray)
 
c plane case discussed here above
 
      if (idatmp.eq.0.or.idatmp.eq.4) then
        if (idatmp.eq.4) trayp=tray
        if (idatmp.eq.0) trayp=0.
        else
          trayp=tray*ftray
      endif
      trayl(l)=tray
      traypl(l)=trayp
 
c     computation of aerosol optical properties at wl
 
      taer=taer55*ext(l)/ext(4)
      taerp=taerp55*ext(l)/ext(4)
      piza=ome(l)
c
c     computation of atmospheric reflectances
c               rorayl is rayleigh ref
c               roaero is aerosol ref
c     call plegen to decompose aerosol phase function in Betal
      if (iaer.ne.0) then
      do k=1,83
      pha(k)=phasel(l,k)
      enddo
      call trunca(coeff)
      endif
      tamoy=taer*(1.-piza*coeff)
      tamoyp=taerp*(1.-piza*coeff)
      pizmoy=piza*(1.-coeff)/(1.-piza*coeff)
c     write(6,*) 'tray,trayp,tamoy,tamoyp,pizmoy,piza,taer,taerp',
c    S            tray,trayp,tamoy,tamoyp,pizmoy,piza,taer,taerp
c     do i=0,80
c     write(6,'(A5,I2.2,1X,E13.7)') 'betal',i,betal(i)
c     enddo
c
      call atmref(iaer,tamoy,tray,pizmoy,tamoyp,trayp,palt,
     a               phi,xmus,xmuv,
     s               phirad,nt,mu,np,rm,gb,rp,
     a                   rorayl,roaero,romix,xlm1,xlm2)
c     write(6,*) 'wl,refrayl,refaero,refmix',wl,rorayl,roaero,romix
c     computation of scattering transmitances (direct and diffuse)
c     first time for rayleigh ,next total (rayleigh+aerosols)
      call scatra (tamoy,tamoyp,tray,trayp,pizmoy,
     a      palt,nt,mu,rm,gb,xmus,xmuv,
     a             ddirtt,ddiftt,udirtt,udiftt,sphlbt,
     a             ddirtr,ddiftr,udirtr,udiftr,sphlbr,
     a             ddirta,ddifta,udirta,udifta,sphlba)
      roatm(1,l)=rorayl
      roatm(2,l)=romix
      roatm(3,l)=roaero
      dtdir(1,l)=ddirtr
      dtdif(1,l)=ddiftr
      dtdir(2,l)=ddirtt
      dtdif(2,l)=ddiftt
      dtdir(3,l)=ddirta
      dtdif(3,l)=ddifta
      utdir(1,l)=udirtr
      utdif(1,l)=udiftr
      utdir(2,l)=udirtt
      utdif(2,l)=udiftt
      utdir(3,l)=udirta
      utdif(3,l)=udifta
      sphal(1,l)=sphlbr
      sphal(2,l)=sphlbt
      sphal(3,l)=sphlba
   50 continue
      return
      end
*
************************************************************************
*
      subroutine discre(ta,ha,tr,hr,it,nt,yy,dd,ppp2,ppp1,
     s     zx)
      real ta,ha,tr,hr,yy,dd,ppp2,ppp1,zx,dt,ti,y1,y2,y3,x2
      real xd,delta,ecart
      double precision xx
      integer it,nt
      if (ha.ge.7.) then
          call print_error
     s    ('check aerosol measurements or plane altitude')
          return
          endif
      if (it.eq.0) then
         dt=1.e-17
         else
         dt=2.*(ta+tr-yy)/(nt-it+1.)
      endif
  99  dt=dt/2.
      ti=yy+dt
      y1=ppp2
      y3=ppp1
  706 y2=(y1+y3)*0.5
      xx=-dble(y2)/dble(ha)
      if (xx.lt.-18.d0) then
         x2=tr*exp(-y2/hr)
         else
         x2=real(dble(ta)*dexp(xx)+dble(tr*exp(-y2/hr)))
         endif
      xd = ti-x2
      if (abs(xd) .lt. 0.00001) then
        go to 705
      else if (xd .lt. 0.) then
        y3=y2
      else
        y1=y2
      endif
      go to 706
  705 continue
      zx=y2
      delta=1./(1.+ta*hr/tr/ha*exp((zx-ppp1)*(1./hr-1./ha)))
      ecart=0
      if(dd.ne.0) ecart=abs((dd-delta)/dd)
      if((ecart.gt.0.95).and.(it.ne.0)) go to 99
      return
      end
*
************************************************************************
*
      subroutine   dust
      common /sixs_aerbas/ ph(10,83)
      real phr(10,83),ph
      integer i,j
c
c    model: dust-like
c
            DATA ((PHR(I,J),J=1,83),I=01,01) /
     *0.2021E+00,0.2079E+00,0.2462E+00,0.2310E+00,0.2069E+00,0.1883E+00,
     *0.1750E+00,0.1624E+00,0.1458E+00,0.1241E+00,0.1013E+00,0.8379E-01,
     *0.7097E-01,0.6207E-01,0.5595E-01,0.5174E-01,0.4879E-01,0.4675E-01,
     *0.4531E-01,0.4435E-01,0.4373E-01,0.4337E-01,0.4324E-01,0.4330E-01,
     *0.4353E-01,0.4392E-01,0.4449E-01,0.4522E-01,0.4612E-01,0.4721E-01,
     *0.4850E-01,0.5001E-01,0.5177E-01,0.5381E-01,0.5616E-01,0.5885E-01,
     *0.6191E-01,0.6540E-01,0.6936E-01,0.7383E-01,0.7889E-01,0.8168E-01,
     *0.8459E-01,0.9096E-01,0.9808E-01,0.1060E+00,0.1148E+00,0.1246E+00,
     *0.1355E+00,0.1474E+00,0.1605E+00,0.1750E+00,0.1910E+00,0.2088E+00,
     *0.2284E+00,0.2501E+00,0.2739E+00,0.3000E+00,0.3284E+00,0.3594E+00,
     *0.3935E+00,0.4308E+00,0.4718E+00,0.5172E+00,0.5670E+00,0.6222E+00,
     *0.6840E+00,0.7528E+00,0.8308E+00,0.9217E+00,0.1029E+01,0.1159E+01,
     *0.1327E+01,0.1553E+01,0.1878E+01,0.2386E+01,0.3253E+01,0.4937E+01,
     *0.8737E+01,0.1952E+02,0.6427E+02,0.4929E+03,0.5169E+05/
            DATA ((PHR(I,J),J=1,83),I=02,02) /
     *0.2467E+00,0.2483E+00,0.2871E+00,0.2722E+00,0.2454E+00,0.2231E+00,
     *0.2060E+00,0.1900E+00,0.1704E+00,0.1452E+00,0.1186E+00,0.9754E-01,
     *0.8182E-01,0.7067E-01,0.6284E-01,0.5734E-01,0.5345E-01,0.5070E-01,
     *0.4875E-01,0.4741E-01,0.4651E-01,0.4596E-01,0.4570E-01,0.4569E-01,
     *0.4589E-01,0.4631E-01,0.4693E-01,0.4776E-01,0.4879E-01,0.5005E-01,
     *0.5153E-01,0.5328E-01,0.5532E-01,0.5768E-01,0.6040E-01,0.6350E-01,
     *0.6704E-01,0.7104E-01,0.7559E-01,0.8071E-01,0.8648E-01,0.8967E-01,
     *0.9298E-01,0.1002E+00,0.1083E+00,0.1173E+00,0.1273E+00,0.1384E+00,
     *0.1507E+00,0.1641E+00,0.1790E+00,0.1954E+00,0.2134E+00,0.2335E+00,
     *0.2557E+00,0.2801E+00,0.3070E+00,0.3366E+00,0.3687E+00,0.4039E+00,
     *0.4427E+00,0.4850E+00,0.5316E+00,0.5834E+00,0.6402E+00,0.7032E+00,
     *0.7738E+00,0.8527E+00,0.9422E+00,0.1047E+01,0.1171E+01,0.1321E+01,
     *0.1516E+01,0.1780E+01,0.2160E+01,0.2753E+01,0.3768E+01,0.5728E+01,
     *0.1011E+02,0.2231E+02,0.7109E+02,0.5001E+03,0.3548E+05/
            DATA ((PHR(I,J),J=1,83),I=03,03) /
     *0.2599E+00,0.2602E+00,0.2986E+00,0.2838E+00,0.2563E+00,0.2330E+00,
     *0.2148E+00,0.1978E+00,0.1774E+00,0.1513E+00,0.1237E+00,0.1017E+00,
     *0.8513E-01,0.7333E-01,0.6499E-01,0.5912E-01,0.5494E-01,0.5198E-01,
     *0.4986E-01,0.4840E-01,0.4742E-01,0.4681E-01,0.4651E-01,0.4647E-01,
     *0.4667E-01,0.4708E-01,0.4772E-01,0.4858E-01,0.4965E-01,0.5094E-01,
     *0.5249E-01,0.5430E-01,0.5642E-01,0.5887E-01,0.6169E-01,0.6491E-01,
     *0.6858E-01,0.7273E-01,0.7744E-01,0.8274E-01,0.8872E-01,0.9201E-01,
     *0.9544E-01,0.1029E+00,0.1113E+00,0.1206E+00,0.1309E+00,0.1424E+00,
     *0.1550E+00,0.1689E+00,0.1842E+00,0.2011E+00,0.2198E+00,0.2404E+00,
     *0.2633E+00,0.2886E+00,0.3163E+00,0.3468E+00,0.3800E+00,0.4164E+00,
     *0.4565E+00,0.5002E+00,0.5485E+00,0.6020E+00,0.6608E+00,0.7261E+00,
     *0.7993E+00,0.8810E+00,0.9739E+00,0.1083E+01,0.1211E+01,0.1368E+01,
     *0.1571E+01,0.1846E+01,0.2242E+01,0.2860E+01,0.3918E+01,0.5956E+01,
     *0.1050E+02,0.2307E+02,0.7281E+02,0.4999E+03,0.3196E+05/
            DATA ((PHR(I,J),J=1,83),I=04,04) /
     *0.2765E+00,0.2752E+00,0.3129E+00,0.2981E+00,0.2697E+00,0.2452E+00,
     *0.2256E+00,0.2075E+00,0.1862E+00,0.1589E+00,0.1301E+00,0.1069E+00,
     *0.8939E-01,0.7677E-01,0.6780E-01,0.6145E-01,0.5690E-01,0.5366E-01,
     *0.5134E-01,0.4973E-01,0.4862E-01,0.4794E-01,0.4758E-01,0.4751E-01,
     *0.4769E-01,0.4811E-01,0.4877E-01,0.4965E-01,0.5076E-01,0.5212E-01,
     *0.5373E-01,0.5563E-01,0.5784E-01,0.6041E-01,0.6336E-01,0.6672E-01,
     *0.7055E-01,0.7488E-01,0.7979E-01,0.8532E-01,0.9155E-01,0.9497E-01,
     *0.9854E-01,0.1063E+00,0.1150E+00,0.1247E+00,0.1354E+00,0.1473E+00,
     *0.1604E+00,0.1748E+00,0.1907E+00,0.2083E+00,0.2276E+00,0.2491E+00,
     *0.2729E+00,0.2990E+00,0.3279E+00,0.3596E+00,0.3941E+00,0.4319E+00,
     *0.4735E+00,0.5191E+00,0.5693E+00,0.6251E+00,0.6864E+00,0.7545E+00,
     *0.8309E+00,0.9163E+00,0.1013E+01,0.1127E+01,0.1262E+01,0.1426E+01,
     *0.1640E+01,0.1928E+01,0.2345E+01,0.2995E+01,0.4106E+01,0.6242E+01,
     *0.1098E+02,0.2400E+02,0.7481E+02,0.4984E+03,0.2810E+05/
            DATA ((PHR(I,J),J=1,83),I=05,05) /
     *0.3140E+00,0.3090E+00,0.3440E+00,0.3291E+00,0.2988E+00,0.2716E+00,
     *0.2491E+00,0.2285E+00,0.2053E+00,0.1759E+00,0.1447E+00,0.1190E+00,
     *0.9926E-01,0.8484E-01,0.7446E-01,0.6700E-01,0.6162E-01,0.5774E-01,
     *0.5493E-01,0.5295E-01,0.5158E-01,0.5070E-01,0.5021E-01,0.5005E-01,
     *0.5019E-01,0.5060E-01,0.5129E-01,0.5224E-01,0.5344E-01,0.5492E-01,
     *0.5668E-01,0.5876E-01,0.6118E-01,0.6400E-01,0.6723E-01,0.7091E-01,
     *0.7509E-01,0.7981E-01,0.8516E-01,0.9117E-01,0.9793E-01,0.1016E+00,
     *0.1055E+00,0.1140E+00,0.1234E+00,0.1338E+00,0.1454E+00,0.1582E+00,
     *0.1724E+00,0.1879E+00,0.2051E+00,0.2241E+00,0.2449E+00,0.2681E+00,
     *0.2937E+00,0.3220E+00,0.3531E+00,0.3873E+00,0.4247E+00,0.4656E+00,
     *0.5108E+00,0.5603E+00,0.6149E+00,0.6756E+00,0.7425E+00,0.8168E+00,
     *0.9003E+00,0.9939E+00,0.1101E+01,0.1226E+01,0.1374E+01,0.1557E+01,
     *0.1793E+01,0.2114E+01,0.2577E+01,0.3299E+01,0.4529E+01,0.6879E+01,
     *0.1204E+02,0.2596E+02,0.7866E+02,0.4906E+03,0.2124E+05/
            DATA ((PHR(I,J),J=1,83),I=06,06) /
     *0.3397E+00,0.3323E+00,0.3646E+00,0.3493E+00,0.3179E+00,0.2889E+00,
     *0.2644E+00,0.2424E+00,0.2181E+00,0.1874E+00,0.1547E+00,0.1274E+00,
     *0.1062E+00,0.9063E-01,0.7928E-01,0.7107E-01,0.6509E-01,0.6076E-01,
     *0.5761E-01,0.5537E-01,0.5380E-01,0.5278E-01,0.5218E-01,0.5196E-01,
     *0.5206E-01,0.5246E-01,0.5317E-01,0.5415E-01,0.5542E-01,0.5697E-01,
     *0.5883E-01,0.6103E-01,0.6359E-01,0.6657E-01,0.6998E-01,0.7387E-01,
     *0.7829E-01,0.8327E-01,0.8891E-01,0.9524E-01,0.1024E+00,0.1063E+00,
     *0.1103E+00,0.1192E+00,0.1291E+00,0.1400E+00,0.1522E+00,0.1656E+00,
     *0.1805E+00,0.1968E+00,0.2148E+00,0.2346E+00,0.2565E+00,0.2807E+00,
     *0.3076E+00,0.3372E+00,0.3699E+00,0.4058E+00,0.4451E+00,0.4881E+00,
     *0.5357E+00,0.5878E+00,0.6454E+00,0.7094E+00,0.7800E+00,0.8586E+00,
     *0.9471E+00,0.1046E+01,0.1160E+01,0.1293E+01,0.1451E+01,0.1646E+01,
     *0.1899E+01,0.2242E+01,0.2738E+01,0.3509E+01,0.4820E+01,0.7310E+01,
     *0.1274E+02,0.2720E+02,0.8080E+02,0.4822E+03,0.1763E+05/
            DATA ((PHR(I,J),J=1,83),I=07,07) /
     *0.3665E+00,0.3585E+00,0.3853E+00,0.3705E+00,0.3386E+00,0.3093E+00,
     *0.2869E+00,0.2705E+00,0.2507E+00,0.2187E+00,0.1832E+00,0.1512E+00,
     *0.1258E+00,0.1065E+00,0.9217E-01,0.8162E-01,0.7386E-01,0.6812E-01,
     *0.6393E-01,0.6088E-01,0.5870E-01,0.5723E-01,0.5631E-01,0.5585E-01,
     *0.5579E-01,0.5612E-01,0.5681E-01,0.5783E-01,0.5918E-01,0.6088E-01,
     *0.6291E-01,0.6532E-01,0.6815E-01,0.7143E-01,0.7521E-01,0.7951E-01,
     *0.8439E-01,0.8988E-01,0.9607E-01,0.1030E+00,0.1108E+00,0.1151E+00,
     *0.1196E+00,0.1293E+00,0.1400E+00,0.1520E+00,0.1652E+00,0.1799E+00,
     *0.1961E+00,0.2140E+00,0.2338E+00,0.2557E+00,0.2799E+00,0.3069E+00,
     *0.3367E+00,0.3696E+00,0.4060E+00,0.4461E+00,0.4901E+00,0.5388E+00,
     *0.5927E+00,0.6520E+00,0.7180E+00,0.7913E+00,0.8725E+00,0.9634E+00,
     *0.1066E+01,0.1181E+01,0.1314E+01,0.1469E+01,0.1655E+01,0.1885E+01,
     *0.2183E+01,0.2586E+01,0.3166E+01,0.4061E+01,0.5568E+01,0.8386E+01,
     *0.1440E+02,0.2992E+02,0.8452E+02,0.4537E+03,0.1132E+05/
            DATA ((PHR(I,J),J=1,83),I=08,08) /
     *0.2248E+00,0.2041E+00,0.2013E+00,0.2015E+00,0.2038E+00,0.2142E+00,
     *0.2218E+00,0.2177E+00,0.2078E+00,0.1973E+00,0.1876E+00,0.1779E+00,
     *0.1666E+00,0.1530E+00,0.1377E+00,0.1221E+00,0.1078E+00,0.9531E-01,
     *0.8504E-01,0.7686E-01,0.7052E-01,0.6573E-01,0.6219E-01,0.5966E-01,
     *0.5794E-01,0.5689E-01,0.5645E-01,0.5656E-01,0.5718E-01,0.5825E-01,
     *0.5974E-01,0.6159E-01,0.6382E-01,0.6647E-01,0.6955E-01,0.7314E-01,
     *0.7723E-01,0.8187E-01,0.8711E-01,0.9302E-01,0.9976E-01,0.1035E+00,
     *0.1075E+00,0.1163E+00,0.1263E+00,0.1377E+00,0.1507E+00,0.1653E+00,
     *0.1819E+00,0.2008E+00,0.2222E+00,0.2467E+00,0.2745E+00,0.3060E+00,
     *0.3418E+00,0.3822E+00,0.4279E+00,0.4800E+00,0.5391E+00,0.6066E+00,
     *0.6838E+00,0.7715E+00,0.8718E+00,0.9864E+00,0.1117E+01,0.1268E+01,
     *0.1442E+01,0.1643E+01,0.1880E+01,0.2160E+01,0.2496E+01,0.2906E+01,
     *0.3423E+01,0.4095E+01,0.5014E+01,0.6356E+01,0.8465E+01,0.1211E+02,
     *0.1924E+02,0.3569E+02,0.8510E+02,0.3357E+03,0.3290E+04/
            DATA ((PHR(I,J),J=1,83),I=09,09) /
     *0.8649E-01,0.6705E-01,0.5195E-01,0.7001E-01,0.7008E-01,0.6002E-01,
     *0.5176E-01,0.4616E-01,0.4241E-01,0.3977E-01,0.3795E-01,0.3668E-01,
     *0.3583E-01,0.3535E-01,0.3514E-01,0.3524E-01,0.3565E-01,0.3638E-01,
     *0.3751E-01,0.3892E-01,0.4055E-01,0.4217E-01,0.4354E-01,0.4447E-01,
     *0.4473E-01,0.4432E-01,0.4334E-01,0.4196E-01,0.4043E-01,0.3895E-01,
     *0.3767E-01,0.3668E-01,0.3599E-01,0.3567E-01,0.3568E-01,0.3603E-01,
     *0.3675E-01,0.3782E-01,0.3929E-01,0.4119E-01,0.4354E-01,0.4489E-01,
     *0.4638E-01,0.4977E-01,0.5377E-01,0.5848E-01,0.6402E-01,0.7052E-01,
     *0.7819E-01,0.8720E-01,0.9780E-01,0.1103E+00,0.1250E+00,0.1423E+00,
     *0.1629E+00,0.1872E+00,0.2164E+00,0.2514E+00,0.2934E+00,0.3442E+00,
     *0.4055E+00,0.4799E+00,0.5709E+00,0.6824E+00,0.8200E+00,0.9912E+00,
     *0.1205E+01,0.1474E+01,0.1814E+01,0.2247E+01,0.2801E+01,0.3520E+01,
     *0.4460E+01,0.5710E+01,0.7406E+01,0.9765E+01,0.1318E+02,0.1847E+02,
     *0.2749E+02,0.4547E+02,0.9155E+02,0.2798E+03,0.1582E+04/
            DATA ((PHR(I,J),J=1,83),I=10,10) /
     *0.9344E-01,0.8261E-01,0.6680E-01,0.7550E-01,0.8962E-01,0.9095E-01,
     *0.8469E-01,0.7755E-01,0.7170E-01,0.6726E-01,0.6401E-01,0.6173E-01,
     *0.6034E-01,0.5974E-01,0.5979E-01,0.6028E-01,0.6096E-01,0.6155E-01,
     *0.6179E-01,0.6151E-01,0.6067E-01,0.5928E-01,0.5752E-01,0.5554E-01,
     *0.5354E-01,0.5165E-01,0.4997E-01,0.4858E-01,0.4752E-01,0.4683E-01,
     *0.4651E-01,0.4657E-01,0.4701E-01,0.4781E-01,0.4897E-01,0.5053E-01,
     *0.5250E-01,0.5493E-01,0.5787E-01,0.6137E-01,0.6550E-01,0.6782E-01,
     *0.7033E-01,0.7593E-01,0.8242E-01,0.8992E-01,0.9860E-01,0.1087E+00,
     *0.1203E+00,0.1339E+00,0.1497E+00,0.1682E+00,0.1896E+00,0.2147E+00,
     *0.2441E+00,0.2786E+00,0.3193E+00,0.3675E+00,0.4248E+00,0.4931E+00,
     *0.5747E+00,0.6726E+00,0.7902E+00,0.9324E+00,0.1105E+01,0.1316E+01,
     *0.1575E+01,0.1895E+01,0.2292E+01,0.2787E+01,0.3407E+01,0.4192E+01,
     *0.5195E+01,0.6498E+01,0.8221E+01,0.1057E+02,0.1389E+02,0.1886E+02,
     *0.2699E+02,0.4205E+02,0.7598E+02,0.1847E+03,0.5926E+03/
      do i=1,10
        do j=1,83
          ph(i,j)=phr(i,j)
        enddo
      enddo
      return
      end
*
************************************************************************
*
      subroutine gauss(x1,x2,x,w,n)
      integer n
      real x1,x2,x(n),w(n)
      double precision eps,xm,xl,z,p1,p2,p3,pp,z1
      integer m,i,j
      parameter (eps=3.d-14)
      m=(n+1)/2
      xm=0.5d0*(dble(x2)+dble(x1))
      xl=0.5d0*(dble(x2)-dble(x1))
      do 12 i=1,m
        z=cos(3.141592654d0*(i-.25d0)/(n+.5d0))
1       continue
          p1=1.d0
          p2=0.d0
          do 11 j=1,n
            p3=p2
            p2=p1
            p1=((2.d0*j-1.d0)*z*p2-(j-1.d0)*p3)/j
11        continue
          pp=n*(z*p1-p2)/(z*z-1.d0)
          z1=z
          z=z1-p1/pp
        if(abs(z-z1).gt.eps)go to 1
        if (abs(z).lt.eps) z=0.d0
        x(i)=real(xm-xl*z)
        x(n+1-i)=real(xm+xl*z)
        w(i)=real(2.d0*xl/((1.d0-z*z)*pp*pp))
        w(n+1-i)=w(i)
12    continue
      return
      end
*
************************************************************************
*
      subroutine interp (iaer,idatmp,wl,taer55,taerp55,xmud,
     a                   romix,rorayl,roaero,phaa,phar,tsca,
     a                   tray,trayp,taer,taerp,dtott,utott,
     a                   astot,asray,asaer,
     a                   utotr,utota,dtotr,dtota)
 
      common /sixs_aer/ext(10),ome(10),gasym(10),phase(10)
      common /six_disc/ roatm(3,10),dtdir(3,10),dtdif(3,10),
     a utdir(3,10),utdif(3,10),sphal(3,10),wldis(10),trayl(10),
     a traypl(10)
      common /sixs_del/ delta,sigma
      Real wl,taer55,taerp55
      Real xmud,romix,rorayl,roaero,phaa,phar,tsca,tray
      Real trayp,taer,taerp,dtott,utott,astot,asray,asaer,utotr
      Real utota,dtotr,dtota,ext,ome,gasym,phase,roatm,dtdir
      Real dtdif,utdir,utdif,sphal,wldis,trayl,traypl,delta,sigma
      Real alphaa,betaa,alphar,betar,alphac,betac,coef,wlinf,d2
      Real drinf,drsup,dtinf,dtsup,dtotc,dainf,dasup,urinf,ursup
      Real utinf,utsup,utotc,uainf,uasup,arinf,arsup,atinf,atsup
      Real aainf,aasup
      Integer iaer,idatmp,linf,ll,lsup

 
c     that for the atmosphere :
c     the reflectances
c                     rayleigh                             = rorayl
c                     aerosols                             = roaero
c                     mixing                               = romix
c     the downward transmittances
c                     rayleigh                             = dtotr
c                     aerosols                             = dtota
c                     total                                = dtott
c     the upward transmittances
c                     rayleigh                             = utotr
c                     aerosols                             = utota
c                     total                                = utott
c     the spherical albedos
c                     rayleigh                             = asray
c                     aerosols                             = asaer
c                     total                                = astot
c     the optical thickness of total atmosphere
c                     rayleigh                             = tray
c                     aerosols                             = taer
c     the optical thickness of the atmosphere above the plane
c                     rayleigh                             = trayp
c                     aerosols                             = taerp
c     the tsca of the aerosols (god dammed it)
c                     total atmosphere                     = tsca
      
      linf=1
      do 81 ll=1,9
      if(wl.gt.wldis(ll).and.wl.le.wldis(ll+1)) linf=ll
   81 continue
      if(wl.gt.wldis(10)) linf=9
      lsup=linf+1
 
c     interpolation in function of wavelength for scattering
c     atmospheric functions from discrete values at wldis
 
      alphaa=0.
      betaa=0.
      alphar=0.
      betar=0.
      alphac=0.
      betac=0.
      phaa=0.
      roaero=0.
      dtota=1.
      utota=1.
      asaer=0.
      taer=0.
      taerp=0.
      coef=alog(wldis(lsup)/wldis(linf))
      wlinf=wldis(linf)
c
      if(iaer.eq.0) goto 1240
      alphaa=alog(phase(lsup)/phase(linf))/coef
      betaa=phase(linf)/(wlinf**alphaa)
      phaa=betaa*(wl**alphaa)
 1240 d2=2.+delta
      phar=(2.*(1.-delta)/d2)*.75*(1.+xmud*xmud)+3.*delta/d2
      if (idatmp.eq.0) then
         betar=0.
         betaa=0.
         betac=0.
         goto 1234
      endif
      alphar=alog(roatm(1,lsup)/roatm(1,linf))/ coef
      betar=roatm(1,linf)/(wlinf**alphar)
      alphac=alog(roatm(2,lsup)/roatm(2,linf))/coef
      betac=roatm(2,linf)/(wlinf**alphac)
      if(iaer.eq.0) goto 1234
      alphaa=alog(roatm(3,lsup)/roatm(3,linf))/coef
      betaa=roatm(3,linf)/(wlinf**alphaa)
      roaero=betaa*(wl**alphaa)
 1234 rorayl=betar*(wl**alphar)
      romix=betac*(wl**alphac)
c
      alphar=alog(trayl(lsup)/trayl(linf))/coef
      betar=trayl(linf)/(wlinf**alphar)
      tray=betar*(wl**alphar)
      if (idatmp.ne.0.) then
         alphar=alog(traypl(lsup)/traypl(linf))/coef
         betar=traypl(linf)/(wlinf**alphar)
         trayp=betar*(wl**alphar)
         else
         trayp=0.
         endif
c
      if(iaer.eq.0) goto 1235
      alphaa=alog(ext(lsup)*ome(lsup)/(ext(linf)*ome(linf)))/coef
      betaa=ext(linf)*ome(linf)/(wlinf**alphaa)
      tsca=taer55*betaa*(wl**alphaa)/ext(4)
      alphaa=alog(ext(lsup)/ext(linf))/coef
      betaa=ext(linf)/(wlinf**alphaa)
      taerp=taerp55*betaa*(wl**alphaa)/ext(4)
      taer=taer55*betaa*(wl**alphaa)/ext(4)
c
 1235 drinf=dtdif(1,linf)+dtdir(1,linf)
      drsup=dtdif(1,lsup)+dtdir(1,lsup)
      alphar=alog(drsup/drinf)/coef
      betar=drinf/(wlinf**alphar)
      dtotr=betar*(wl**alphar)
      dtinf=dtdif(2,linf)+dtdir(2,linf)
      dtsup=dtdif(2,lsup)+dtdir(2,lsup)
      alphac=alog((dtsup*drinf)/(dtinf*drsup))/coef
      betac=(dtinf/drinf)/(wlinf**alphac)
      dtotc=betac*(wl**alphac)
      dainf=dtdif(3,linf)+dtdir(3,linf)
      dasup=dtdif(3,lsup)+dtdir(3,lsup)
      if(iaer.eq.0) goto 1236
      alphaa=alog(dasup/dainf)/coef
      betaa=dainf/(wlinf**alphaa)
      dtota=betaa*(wl**alphaa)
 1236 dtott=dtotc*dtotr
      urinf=utdif(1,linf)+utdir(1,linf)
      ursup=utdif(1,lsup)+utdir(1,lsup)
      alphar=alog(ursup/urinf)/ coef
      betar=urinf/(wlinf**alphar)
      utotr=betar*(wl**alphar)
      utinf=utdif(2,linf)+utdir(2,linf)
      utsup=utdif(2,lsup)+utdir(2,lsup)
      alphac=alog((utsup*urinf)/(utinf*ursup))/ coef
      betac=(utinf/urinf)/(wlinf**alphac)
      utotc=betac*(wl**alphac)
      uainf=utdif(3,linf)+utdir(3,linf)
      uasup=utdif(3,lsup)+utdir(3,lsup)
      if(iaer.eq.0) goto 1237
      alphaa=alog(uasup/uainf)/ coef
      betaa=uainf/(wlinf**alphaa)
      utota=betaa*(wl**alphaa)
 1237 utott=utotc*utotr
      arinf=sphal(1,linf)
      arsup=sphal(1,lsup)
      alphar=alog(arsup/arinf)/ coef
      betar=arinf/(wlinf**alphar)
      asray=betar*(wl**alphar)
      atinf=sphal(2,linf)
      atsup=sphal(2,lsup)
      alphac=alog(atsup/atinf)/coef
      betac=atinf/(wlinf**alphac)
      astot=betac*(wl**alphac)
      aainf=sphal(3,linf)
      aasup=sphal(3,lsup)
      if(iaer.eq.0) goto 1239
      alphaa=alog(aasup/aainf)/coef
      betaa=aainf/(wlinf**alphaa)
      asaer=betaa*(wl**alphaa)
 1239 return
      end
*
************************************************************************
*
      subroutine iso(tamoy,trmoy,pizmoy,tamoyp,trmoyp,palt,
     s               nt,mu,rm,gb,
     s                     xf)
c  dimension for gauss integration
      integer mu
      real rm(-mu:mu),gb(-mu:mu)
c  dimension for os computation
      real  xf(-1:1)
c array for sos computation
      real xpl(-25:25),psl(-1:80,-25:25),bp(0:25,-25:25),
     s xdel(0:30),ydel(0:30),h(0:30)
      real i1(0:30,-25:25),i2(0:30,-25:25),i3(-25:25),
     s   in(-25:25),inm1(-25:25),inm2(-25:25)
      real altc(0:30)
      real ii1,ii2
      Real tamoy,trmoy,pizmoy
      Real tamoyp,trmoyp,palt
      Real delta,sigma,accu,ta,piz
      Real tr,trp,tap,hr,ha,zx,yy,dd,ppp2,ppp1,ca
      Real cr,ratio,taup,th,xt1,xt2,aaaa,ron,beta0,beta2
      Real tavion0,tavion1,tavion2,tavion,zi1,xpk,ypk,x,y,xpj
      Real z,xi1,xi2,bpjk,bpjmk,f,a,b,c,d,xx,a1,d1,g1
      Real y1
      Double precision xxx, yyy
      integer snt,nt,iplane,ntp,j,it,i,ig,k,index,iwr,m
      integer jj,l
      logical ier
      common/sixs_del/delta,sigma
      common/sixs_ier/iwr,ier
      snt=nt
      iplane=0
      accu=1.e-20
      ta=tamoy
      piz=pizmoy
      tr=trmoy
      do m=-1,1
        xf(m)=0.
      enddo
c
c     molecular ratio within the layer
c     computations are performed assuming a scale of 8km for
c     molecules and 2km for aerosols
c
c the optical thickness above plane are recomputed to give o.t above pla
      trp=trmoy-trmoyp
      tap=tamoy-tamoyp
c     print *, 'tamoy,trmoy,pizmoy,tap,trp,palt,nt'
c     print *,tamoy,trmoy,pizmoy,tap,trp,palt,nt
c if plane observations recompute scale height for aerosol knowing:
c the aerosol optical depth as measure from the plane       = tamoyp
c the rayleigh scale   height =       		= hr (8km)
c the rayleigh optical depth  at plane level       	= trmoyp
c the altitude of the plane       			= palt
c the rayleigh optical depth for total atmos      	= trmoy
c the aerosol  optical depth for total atmos      	= tamoy
c if not plane observations then ha is equal to 2.0km
c ntp local variable: if ntp=nt     no plane observation selected
c                        ntp=nt-1   plane observation selected
      hr=8.0
c     it's a mixing rayleigh+aerosol
      if(palt.le.900..and.palt.gt.0.0)then
      if (tap.gt.1.e-03) then
         ha=-palt/log(tap/ta)
         else
         ha=2.
         endif
      ntp=nt-1
      else
      ha=2.0
      ntp=nt
      endif
c
      ta=tamoy
      tr=trmoy
      piz=pizmoy
c
c compute mixing rayleigh, aerosol
c case 1: pure rayleigh
c case 2: pure aerosol
c case 3: mixing rayleigh-aerosol
c
      if(ta.le.accu) then
      do j=0,ntp
      h(j)=j*tr/ntp
      ydel(j)=1.0
      xdel(j)=0.0
      enddo
      endif
      if(tr.le.accu) then
      do j=0,ntp
      h(j)=j*ta/ntp
      ydel(j)=0.0
      xdel(j)=piz
      enddo
      endif
c
      if(tr.gt.accu.and.ta.gt.accu)then
      ydel(0)=1.0
      xdel(0)=0.0
      h(0)=0.
      altc(0)=300.
      zx=300.
      iplane=0
      do 14 it=0,ntp
      if (it.eq.0) then
         yy=0.
         dd=0.
         goto 111
      endif
      yy=h(it-1)
      dd=ydel(it-1)
 111  ppp2=300.0
      ppp1=0.0
      call discre(ta,ha,tr,hr,it,ntp,yy,dd,ppp2,ppp1,
     s    zx)
      if(ier)return
      xxx=-dble(zx)/dble(ha)
      if (xxx.lt.-18.d0) then
         ca=0.
         else
         yyy=exp(xxx)
         ca=real(dble(ta)*yyy)
         endif
      xxx=-dble(zx)/dble(hr)
      yyy=exp(xxx)
      cr=real(dble(tr)*yyy)
      h(it)=cr+ca
      altc(it)=zx
c     print *,it,cr,ca,h(it),zx
      cr=cr/hr
      ca=ca/ha
      ratio=cr/(cr+ca)
      xdel(it)=(1.e+00-ratio)*piz
      ydel(it)=ratio
  14  continue
      endif
c update plane layer if necessary
      if (ntp.eq.(nt-1)) then
c compute position of the plane layer
         taup=tap+trp
         iplane=-1
         do i=0,ntp
         if (taup.ge.h(i)) iplane=i
         enddo
c update the layer from the end to the position to update if necessary
         th=0.005
         xt1=abs(h(iplane)-taup)
         xt2=abs(h(iplane+1)-taup)
         if ((xt1.gt.th).and.(xt2.gt.th)) then
         do i=nt,iplane+1,-1
            xdel(i)=xdel(i-1)
            ydel(i)=ydel(i-1)
            h(i)=h(i-1)
            altc(i)=altc(i-1)
         enddo
         else
         nt=ntp
         if (xt2.lt.xt1) iplane=iplane+1
         endif
         h(iplane)=taup
         if ( tr.gt.accu.and.ta.gt.accu) then
         ca=ta*exp(-palt/ha)
         cr=tr*exp(-palt/hr)
         cr=cr/hr
         ca=ca/ha
         ratio=cr/(cr+ca)
         xdel(iplane)=(1.e+00-ratio)*piz
         ydel(iplane)=ratio
         altc(iplane)=palt
         endif
         if ( tr.gt.accu.and.ta.le.accu) then
         ydel(iplane)=1.
         xdel(iplane)=0.
         altc(iplane)=palt
         endif
         if ( tr.le.accu.and.ta.gt.accu) then
         ydel(iplane)=0.
         xdel(iplane)=1.*piz
         altc(iplane)=palt
         endif
      endif
c
c     print *,ha,hr,palt,tamoy,trmoy,tap,trp
c     do i=0,nt
c     print *,i,h(i),xdel(i),ydel(i),altc(i)
c     enddo
c
      aaaa=delta/(2-delta)
      ron=(1-aaaa)/(1+2*aaaa)
c
c     rayleigh phase function
c
      beta0=1.
      beta2=0.5*ron
c
c    primary scattering
c
      ig=1
      tavion0=0.
      tavion1=0.
      tavion2=0.
      tavion=0.
      do 16 j=-mu,mu
      i3(j)=0.
   16 continue
c
c     kernel computations
c
      call kernel(0,mu,rm,xpl,psl,bp)
      do 100 j=-mu,mu
      do 101 k=0,nt
      i2(k,j)=0.0000
  101 continue
  100 continue
c
c     vertical integration, primary upward radiation
c
 
      do k=1,mu
        i1(nt,k)=1.0
        zi1=i1(nt,k)
        yy=rm(k)
        do i=nt-1,0,-1
          i1(i,k)=exp(-(ta+tr-h(i))/yy)
        enddo
      enddo
c
c     vertical integration, primary downward radiation
c
      do k=-mu,-1
        do i=0,nt
          i1(i,k)=0.00
        enddo
      enddo
c
c     inm2 is inialized with scattering computed at n-2
c     i3 is inialized with primary scattering
c
      do 20 k=-mu,mu
      if (k .eq. 0) then
        goto 20
      else if (k .lt. 0) then
        index=nt
      else
        index=0
      endif
      inm1(k)=i1(index,k)
      inm2(k)=i1(index,k)
      i3(k)=i1(index,k)
   20 continue
      tavion=i1(iplane,mu)
      tavion2=i1(iplane,mu)
c
c     loop on successive order
c
  503 ig=ig+1
c     write(6,*) 'ig ',ig
c
c     successive orders
c
c     multiple scattering source function at every level within the laye
c
c
      do k=1,mu
        xpk=xpl(k)
        ypk=xpl(-k)
        do i=0,nt
          ii1=0.
          ii2=0.
          x=xdel(i)
          y=ydel(i)
          do j=1,mu
            xpj=xpl(j)
            z=gb(j)
            xi1=i1(i,j)
            xi2=i1(i,-j)
            bpjk=bp(j,k)*x+y*(beta0+beta2*xpj*xpk)
            bpjmk=bp(j,-k)*x+y*(beta0+beta2*xpj*ypk)
            ii2=ii2+z*(xi1*bpjk+xi2*bpjmk)
            ii1=ii1+z*(xi1*bpjmk+xi2*bpjk)
          enddo
          i2(i,k)=ii2
          i2(i,-k)=ii1
        enddo
      enddo
c
c     vertical integration, upward radiation
c
      do k=1,mu
        i1(nt,k)=0.0
        zi1=i1(nt,k)
        yy=rm(k)
        do i=nt-1,0,-1
          jj=i+1
          f=h(jj)-h(i)
          if (abs(f) .gt. accu) then
            a=(i2(jj,k)-i2(i,k))/f
            b=i2(i,k)-a*h(i)
            c=exp(-f/yy)
            d=1.e+00-c
            xx=h(i)-h(jj)*c
            zi1=c*zi1+(d*(b+a*yy)+a*xx)*0.5e+00
          endif
          i1(i,k)=zi1
        enddo
      enddo
c
c     vertical integration, downward radiation
c
      do k=-mu,-1
        i1(0,k)=0.
        zi1=i1(0,k)
        yy=rm(k)
        do i=1,nt
          jj=i-1
          f=h(i)-h(jj)
          if (abs(f) .gt. accu) then
            c=exp(f/yy)
            d=1.e+00-c
            a=(i2(i,k)-i2(jj,k))/f
            b=i2(i,k)-a*h(i)
            xx=h(i)-h(jj)*c
            zi1=c*zi1+(d*(b+a*yy)+a*xx)*0.5e+00
          endif
          i1(i,k)=zi1
        enddo
      enddo
c
c     in is the nieme scattering order
c
      do 30 k=-mu,mu
      if (k .eq. 0) then
        goto 30
      else if (k .lt.0) then
        index=nt
      else
        index=0
      endif
      in(k)=i1(index,k)
   30 continue
      tavion0=i1(iplane,mu)
c
c   convergence test (geometrical serie)
c
      if(ig.gt.2) then
      z=0.
      a1=tavion2
      d1=tavion1
      g1=tavion0
      if (a1.ge.accu.and.d1.ge.accu.and.tavion.ge.accu) then
         y=((g1/d1-d1/a1)/((1.-g1/d1)**2)*(g1/tavion))
         y=abs(y)
         z=amax1(y,z)
      endif
      do 99 l=-mu,mu
      if (l.eq.0) goto 99
      a1=inm2(l)
      d1=inm1(l)
      g1=in(l)
      if(a1.eq.0.) go to 99
      if(d1.eq.0.) go to 99
      if(i3(l).eq.0.) go to 99
      y=((g1/d1-d1/a1)/((1-g1/d1)**2)*(g1/i3(l)))
      y=abs(y)
      z=amax1(y,z)
  99  continue
      if(z.lt.0.0001) then
c
c     successful test (geometrical serie)
c
      do 606 l=-mu,mu
      if (l.eq.0) goto 606
      y1=1.
      d1=inm1(l)
      g1=in(l)
      if(d1.eq.0.0) go to 606
      y1=1-g1/d1
      g1=g1/y1
      i3(l)=i3(l)+g1
  606 continue
      d1=tavion1
      g1=tavion0
      y1=1.
      if (d1.ge.accu) then
      if (abs(g1-d1).ge.accu) then
         y1=1.-g1/d1
         g1=g1/y1
      endif
      tavion=tavion+g1
      endif
      go to 505
      endif
c
c     inm2 is the (n-2)ieme scattering order
c
      do 26 k=-mu,mu
      inm2(k)=inm1(k)
   26 continue
      tavion2=tavion1
      endif
c
c     inm1 is the (n-1)ieme scattering order
c
      do 27 k=-mu,mu
      inm1(k)=in(k)
   27 continue
      tavion1=tavion0
c
c     sum of the n-1 orders
c
      do 610 l=-mu,mu
      i3(l)=i3(l)+in(l)
  610 continue
      tavion=tavion+tavion0
c
c     stop if order n is less than 1% of the sum
c
      z=0.
      do 611 l=-mu,mu
      if(i3(l).ne.0)then
      y=abs(in(l)/i3(l))
      z=amax1(z,y)
      endif
  611 continue
      if(z.lt.0.00001) go to 505
c
c      stop if order n is greater than 20 in any case
c
      if ((ig-20) .lt. 0) goto 503
  505 continue
c
c
      xf(1)=xf(1)+i3(mu)
      xf(-1)=tavion
      do k=1,mu
      xf(0)=xf(0)+rm(k)*gb(k)*i3(-k)
      enddo
      nt=snt
      return
      end
*
************************************************************************
*
      subroutine kernel(is,mu,rm,xpl,psl,bp)
      integer mu
      real rm(-mu:mu)
      real psl(-1:80,-25:25),xpl(-25:25),bp(0:25,-25:25)
      real pha,betal
      integer is,ip1,j,i,k,ip,ig,l,lp,lm,ij
      double precision xdb,a,b,c,xx,rac3,x,bt,sbp
      common /sixs_trunc/pha(83),betal(0:80)
      ip1=80
      rac3=dsqrt(3.D+00)
      if(is.ne.0)go to 700
      do 25 j=0,mu
      c=dble(rm(j))
      psl(0,-j)=1.
      psl(0,j)=1.
      psl(1,j)=real(c)
      psl(1,-j)=-real(c)
      xdb=(3.d0*c*c-1.d0)*0.5d0
      if (abs(xdb).lt.1.d-30) xdb=0.0d0
      psl(2,-j)=real(xdb)
      psl(2,j)=real(xdb)
   25 continue
      psl(1,0)=rm(0)
      goto 501
c
  700 if(is.ne.1)go to 701
      do 26 j=0,mu
      c=dble(rm(j))
      x=1.d0-c*c
      psl(0,j)=0.
      psl(0,-j)=0.
      psl(1,-j)=real(sqrt(x*0.5d0))
      psl(1,j)=real(sqrt(x*0.5d0))
      psl(2,j)=real(c*dble(psl(1,j))*rac3)
      psl(2,-j)=-psl(2,j)
   26 continue
      psl(2,0)=-psl(2,0)
      goto 501
c
  701 a=1
      do 27 i=1,is
      x=i
      a=a*sqrt((i+is)/x)*0.5d0
 27   continue
      b=a*sqrt(is/(is+1.d0))*sqrt((is-1.d0)/(is+2.d0))
      do 28 j=0,mu
      c=dble(rm(j))
      xx=1.d0-c*c
      psl(is-1,j)=0.
      xdb=a*xx**(is*0.5d0)
      if (abs(xdb).lt.1.d-30) xdb=0.0d0
      psl(is,-j)=real(xdb)
      psl(is,j)=real(xdb)
   28 continue
  501 k=2
      ip=ip1
      if(is.gt.2)k=is
      if(k.eq.ip)goto 502
      ig=-1
      if(is.eq.1)ig=1
      do 30 l=k,ip-1
      lp=l+1
      lm=l-1
      a=(2*l+1.d0)/sqrt((l+is+1.d0)*(l-is+1.d0))
      b=sqrt(dble((l+is)*(l-is)))/(2.d0*l+1.d0)
      do 31 j=0,mu
      c=dble(rm(j))
      xdb=a*(c*dble(psl(l,j))-b*dble(psl(lm,j)))
      if (abs(xdb).lt.1.d-30) xdb=0.d0
      psl(lp,j)=real(xdb)
      if(j.eq.0) go to 31
      psl(lp,-j)=ig*psl(lp,j)
   31 continue
      ig=-ig
   30 continue
  502 continue
      do 1005 j=-mu,mu
      xpl(j)=psl(2,j)
 1005 continue
      ij=ip1
      do j=0,mu
        do k=-mu,mu
          sbp=0.d0
          if(is.gt.ij) goto 1
          do l=is,ij
            bt=dble(betal(l))
            sbp=sbp+dble(psl(l,j))*dble(psl(l,k))*bt
          enddo
 1        continue
          if (abs(sbp).lt.1.d-30) sbp=0.d0
          bp(j,k)=real(sbp)
        enddo
      enddo
      return
      end
*
************************************************************************
*
      subroutine   ocea
      common /sixs_aerbas/ ph(10,83)
      real phr(10,83),ph
      integer i,j
c
c    model: oceanic
c
            DATA ((PHR(I,J),J=1,83),I=01,01) /
     *0.7855E+00,0.6283E+00,0.5465E+00,0.4693E+00,0.4153E+00,0.3917E+00,
     *0.3657E+00,0.3378E+00,0.3161E+00,0.3025E+00,0.2972E+00,0.2990E+00,
     *0.3055E+00,0.3118E+00,0.3059E+00,0.2715E+00,0.2118E+00,0.1585E+00,
     *0.1230E+00,0.9913E-01,0.8327E-01,0.7292E-01,0.6585E-01,0.6171E-01,
     *0.5883E-01,0.5780E-01,0.5791E-01,0.5893E-01,0.6144E-01,0.6406E-01,
     *0.6717E-01,0.6966E-01,0.7130E-01,0.7291E-01,0.7434E-01,0.7626E-01,
     *0.7847E-01,0.8190E-01,0.8583E-01,0.9044E-01,0.9709E-01,0.1006E+00,
     *0.1045E+00,0.1128E+00,0.1239E+00,0.1360E+00,0.1497E+00,0.1667E+00,
     *0.1856E+00,0.2070E+00,0.2323E+00,0.2615E+00,0.2948E+00,0.3326E+00,
     *0.3772E+00,0.4263E+00,0.4840E+00,0.5492E+00,0.6242E+00,0.7103E+00,
     *0.8075E+00,0.9192E+00,0.1046E+01,0.1190E+01,0.1354E+01,0.1541E+01,
     *0.1756E+01,0.2002E+01,0.2277E+01,0.2603E+01,0.2976E+01,0.3416E+01,
     *0.3931E+01,0.4563E+01,0.5372E+01,0.6490E+01,0.8191E+01,0.1111E+02,
     *0.1692E+02,0.3097E+02,0.7524E+02,0.2992E+03,0.1697E+04/
            DATA ((PHR(I,J),J=1,83),I=02,02) /
     *0.7129E+00,0.5739E+00,0.5059E+00,0.4429E+00,0.4035E+00,0.3898E+00,
     *0.3678E+00,0.3416E+00,0.3195E+00,0.3042E+00,0.2975E+00,0.2961E+00,
     *0.2987E+00,0.2994E+00,0.2909E+00,0.2614E+00,0.2134E+00,0.1670E+00,
     *0.1336E+00,0.1100E+00,0.9363E-01,0.8252E-01,0.7480E-01,0.6967E-01,
     *0.6621E-01,0.6499E-01,0.6438E-01,0.6506E-01,0.6656E-01,0.6880E-01,
     *0.7108E-01,0.7332E-01,0.7497E-01,0.7681E-01,0.7860E-01,0.8093E-01,
     *0.8357E-01,0.8723E-01,0.9184E-01,0.9665E-01,0.1036E+00,0.1075E+00,
     *0.1112E+00,0.1200E+00,0.1316E+00,0.1436E+00,0.1580E+00,0.1748E+00,
     *0.1937E+00,0.2154E+00,0.2413E+00,0.2704E+00,0.3031E+00,0.3421E+00,
     *0.3856E+00,0.4356E+00,0.4928E+00,0.5586E+00,0.6333E+00,0.7196E+00,
     *0.8188E+00,0.9313E+00,0.1060E+01,0.1208E+01,0.1375E+01,0.1568E+01,
     *0.1791E+01,0.2047E+01,0.2340E+01,0.2679E+01,0.3075E+01,0.3547E+01,
     *0.4107E+01,0.4805E+01,0.5714E+01,0.6981E+01,0.8889E+01,0.1212E+02,
     *0.1839E+02,0.3283E+02,0.7515E+02,0.2626E+03,0.1134E+04/
            DATA ((PHR(I,J),J=1,83),I=03,03) /
     *0.6966E+00,0.5607E+00,0.4902E+00,0.4336E+00,0.3978E+00,0.3866E+00,
     *0.3674E+00,0.3412E+00,0.3187E+00,0.3039E+00,0.2960E+00,0.2945E+00,
     *0.2960E+00,0.2961E+00,0.2874E+00,0.2591E+00,0.2133E+00,0.1692E+00,
     *0.1362E+00,0.1129E+00,0.9630E-01,0.8484E-01,0.7707E-01,0.7190E-01,
     *0.6854E-01,0.6653E-01,0.6597E-01,0.6668E-01,0.6812E-01,0.7009E-01,
     *0.7216E-01,0.7425E-01,0.7580E-01,0.7758E-01,0.7959E-01,0.8174E-01,
     *0.8490E-01,0.8852E-01,0.9294E-01,0.9864E-01,0.1048E+00,0.1084E+00,
     *0.1128E+00,0.1220E+00,0.1325E+00,0.1453E+00,0.1596E+00,0.1762E+00,
     *0.1959E+00,0.2177E+00,0.2428E+00,0.2725E+00,0.3055E+00,0.3440E+00,
     *0.3882E+00,0.4382E+00,0.4953E+00,0.5613E+00,0.6365E+00,0.7225E+00,
     *0.8218E+00,0.9344E+00,0.1065E+01,0.1212E+01,0.1381E+01,0.1577E+01,
     *0.1801E+01,0.2059E+01,0.2360E+01,0.2701E+01,0.3107E+01,0.3586E+01,
     *0.4166E+01,0.4885E+01,0.5821E+01,0.7115E+01,0.9088E+01,0.1241E+02,
     *0.1877E+02,0.3323E+02,0.7480E+02,0.2523E+03,0.1018E+04/
            DATA ((PHR(I,J),J=1,83),I=04,04) /
     *0.6774E+00,0.5476E+00,0.4775E+00,0.4252E+00,0.3937E+00,0.3855E+00,
     *0.3684E+00,0.3432E+00,0.3209E+00,0.3059E+00,0.2974E+00,0.2950E+00,
     *0.2951E+00,0.2935E+00,0.2832E+00,0.2550E+00,0.2114E+00,0.1697E+00,
     *0.1380E+00,0.1153E+00,0.9882E-01,0.8737E-01,0.7952E-01,0.7423E-01,
     *0.7074E-01,0.6859E-01,0.6788E-01,0.6842E-01,0.6969E-01,0.7150E-01,
     *0.7349E-01,0.7557E-01,0.7720E-01,0.7911E-01,0.8125E-01,0.8356E-01,
     *0.8685E-01,0.9062E-01,0.9516E-01,0.1010E+00,0.1073E+00,0.1109E+00,
     *0.1154E+00,0.1247E+00,0.1352E+00,0.1482E+00,0.1626E+00,0.1793E+00,
     *0.1991E+00,0.2210E+00,0.2462E+00,0.2760E+00,0.3091E+00,0.3477E+00,
     *0.3920E+00,0.4422E+00,0.4994E+00,0.5656E+00,0.6410E+00,0.7275E+00,
     *0.8272E+00,0.9405E+00,0.1071E+01,0.1220E+01,0.1391E+01,0.1588E+01,
     *0.1815E+01,0.2077E+01,0.2382E+01,0.2731E+01,0.3145E+01,0.3636E+01,
     *0.4233E+01,0.4974E+01,0.5942E+01,0.7282E+01,0.9319E+01,0.1273E+02,
     *0.1919E+02,0.3364E+02,0.7414E+02,0.2397E+03,0.8914E+03/
            DATA ((PHR(I,J),J=1,83),I=05,05) /
     *0.6153E+00,0.5058E+00,0.4382E+00,0.3950E+00,0.3738E+00,0.3731E+00,
     *0.3585E+00,0.3354E+00,0.3139E+00,0.2983E+00,0.2892E+00,0.2849E+00,
     *0.2832E+00,0.2800E+00,0.2703E+00,0.2469E+00,0.2112E+00,0.1741E+00,
     *0.1442E+00,0.1219E+00,0.1054E+00,0.9356E-01,0.8531E-01,0.7966E-01,
     *0.7561E-01,0.7323E-01,0.7198E-01,0.7214E-01,0.7291E-01,0.7415E-01,
     *0.7601E-01,0.7747E-01,0.7901E-01,0.8091E-01,0.8293E-01,0.8564E-01,
     *0.8906E-01,0.9289E-01,0.9788E-01,0.1033E+00,0.1102E+00,0.1141E+00,
     *0.1181E+00,0.1275E+00,0.1385E+00,0.1511E+00,0.1660E+00,0.1823E+00,
     *0.2018E+00,0.2241E+00,0.2491E+00,0.2784E+00,0.3123E+00,0.3503E+00,
     *0.3942E+00,0.4451E+00,0.5020E+00,0.5684E+00,0.6448E+00,0.7319E+00,
     *0.8325E+00,0.9481E+00,0.1081E+01,0.1234E+01,0.1409E+01,0.1612E+01,
     *0.1846E+01,0.2118E+01,0.2440E+01,0.2809E+01,0.3249E+01,0.3773E+01,
     *0.4413E+01,0.5211E+01,0.6259E+01,0.7710E+01,0.9888E+01,0.1347E+02,
     *0.2009E+02,0.3435E+02,0.7217E+02,0.2130E+03,0.6728E+03/
            DATA ((PHR(I,J),J=1,83),I=06,06) /
     *0.5916E+00,0.4877E+00,0.4171E+00,0.3786E+00,0.3632E+00,0.3654E+00,
     *0.3546E+00,0.3335E+00,0.3124E+00,0.2967E+00,0.2869E+00,0.2822E+00,
     *0.2792E+00,0.2744E+00,0.2635E+00,0.2413E+00,0.2085E+00,0.1740E+00,
     *0.1459E+00,0.1244E+00,0.1084E+00,0.9682E-01,0.8822E-01,0.8243E-01,
     *0.7835E-01,0.7606E-01,0.7463E-01,0.7441E-01,0.7473E-01,0.7609E-01,
     *0.7739E-01,0.7905E-01,0.8078E-01,0.8256E-01,0.8474E-01,0.8745E-01,
     *0.9082E-01,0.9490E-01,0.9996E-01,0.1057E+00,0.1127E+00,0.1166E+00,
     *0.1207E+00,0.1301E+00,0.1412E+00,0.1539E+00,0.1686E+00,0.1858E+00,
     *0.2048E+00,0.2270E+00,0.2528E+00,0.2818E+00,0.3154E+00,0.3545E+00,
     *0.3980E+00,0.4487E+00,0.5067E+00,0.5728E+00,0.6491E+00,0.7374E+00,
     *0.8386E+00,0.9547E+00,0.1090E+01,0.1244E+01,0.1423E+01,0.1630E+01,
     *0.1870E+01,0.2149E+01,0.2477E+01,0.2862E+01,0.3316E+01,0.3862E+01,
     *0.4527E+01,0.5365E+01,0.6458E+01,0.7974E+01,0.1023E+02,0.1390E+02,
     *0.2058E+02,0.3459E+02,0.7042E+02,0.1961E+03,0.5608E+03/
            DATA ((PHR(I,J),J=1,83),I=07,07) /
     *0.5164E+00,0.4330E+00,0.3650E+00,0.3341E+00,0.3313E+00,0.3413E+00,
     *0.3356E+00,0.3182E+00,0.2998E+00,0.2844E+00,0.2744E+00,0.2677E+00,
     *0.2626E+00,0.2560E+00,0.2453E+00,0.2267E+00,0.2009E+00,0.1730E+00,
     *0.1485E+00,0.1291E+00,0.1141E+00,0.1028E+00,0.9425E-01,0.8828E-01,
     *0.8375E-01,0.8105E-01,0.7927E-01,0.7843E-01,0.7860E-01,0.7925E-01,
     *0.8010E-01,0.8165E-01,0.8331E-01,0.8499E-01,0.8754E-01,0.9034E-01,
     *0.9390E-01,0.9825E-01,0.1034E+00,0.1093E+00,0.1164E+00,0.1203E+00,
     *0.1246E+00,0.1342E+00,0.1452E+00,0.1582E+00,0.1728E+00,0.1896E+00,
     *0.2094E+00,0.2310E+00,0.2569E+00,0.2863E+00,0.3195E+00,0.3587E+00,
     *0.4030E+00,0.4534E+00,0.5122E+00,0.5794E+00,0.6565E+00,0.7463E+00,
     *0.8505E+00,0.9697E+00,0.1109E+01,0.1270E+01,0.1457E+01,0.1674E+01,
     *0.1929E+01,0.2226E+01,0.2578E+01,0.2997E+01,0.3495E+01,0.4096E+01,
     *0.4831E+01,0.5758E+01,0.6967E+01,0.8629E+01,0.1105E+02,0.1487E+02,
     *0.2152E+02,0.3465E+02,0.6548E+02,0.1595E+03,0.3700E+03/
            DATA ((PHR(I,J),J=1,83),I=08,08) /
     *0.3257E+00,0.2888E+00,0.2378E+00,0.2215E+00,0.2345E+00,0.2532E+00,
     *0.2578E+00,0.2504E+00,0.2390E+00,0.2282E+00,0.2194E+00,0.2123E+00,
     *0.2059E+00,0.1991E+00,0.1906E+00,0.1797E+00,0.1665E+00,0.1520E+00,
     *0.1379E+00,0.1254E+00,0.1147E+00,0.1061E+00,0.9917E-01,0.9373E-01,
     *0.8960E-01,0.8656E-01,0.8438E-01,0.8306E-01,0.8243E-01,0.8240E-01,
     *0.8294E-01,0.8394E-01,0.8543E-01,0.8740E-01,0.8990E-01,0.9302E-01,
     *0.9681E-01,0.1013E+00,0.1067E+00,0.1129E+00,0.1200E+00,0.1240E+00,
     *0.1283E+00,0.1379E+00,0.1490E+00,0.1618E+00,0.1764E+00,0.1932E+00,
     *0.2124E+00,0.2345E+00,0.2599E+00,0.2892E+00,0.3231E+00,0.3622E+00,
     *0.4072E+00,0.4593E+00,0.5195E+00,0.5895E+00,0.6711E+00,0.7664E+00,
     *0.8781E+00,0.1009E+01,0.1163E+01,0.1343E+01,0.1556E+01,0.1808E+01,
     *0.2107E+01,0.2464E+01,0.2891E+01,0.3405E+01,0.4025E+01,0.4779E+01,
     *0.5707E+01,0.6863E+01,0.8338E+01,0.1027E+02,0.1291E+02,0.1670E+02,
     *0.2248E+02,0.3211E+02,0.5001E+02,0.8772E+02,0.1334E+03/
            DATA ((PHR(I,J),J=1,83),I=09,09) /
     *0.2139E+00,0.1949E+00,0.1618E+00,0.1541E+00,0.1685E+00,0.1828E+00,
     *0.1856E+00,0.1800E+00,0.1718E+00,0.1642E+00,0.1581E+00,0.1534E+00,
     *0.1495E+00,0.1460E+00,0.1421E+00,0.1375E+00,0.1318E+00,0.1252E+00,
     *0.1178E+00,0.1105E+00,0.1036E+00,0.9754E-01,0.9237E-01,0.8811E-01,
     *0.8468E-01,0.8198E-01,0.7994E-01,0.7852E-01,0.7768E-01,0.7741E-01,
     *0.7767E-01,0.7843E-01,0.7969E-01,0.8144E-01,0.8373E-01,0.8662E-01,
     *0.9014E-01,0.9438E-01,0.9939E-01,0.1052E+00,0.1120E+00,0.1158E+00,
     *0.1198E+00,0.1289E+00,0.1394E+00,0.1514E+00,0.1653E+00,0.1813E+00,
     *0.1997E+00,0.2208E+00,0.2453E+00,0.2736E+00,0.3064E+00,0.3444E+00,
     *0.3886E+00,0.4400E+00,0.5000E+00,0.5703E+00,0.6528E+00,0.7502E+00,
     *0.8652E+00,0.1001E+01,0.1163E+01,0.1355E+01,0.1584E+01,0.1859E+01,
     *0.2188E+01,0.2586E+01,0.3067E+01,0.3649E+01,0.4358E+01,0.5222E+01,
     *0.6282E+01,0.7594E+01,0.9235E+01,0.1132E+02,0.1404E+02,0.1768E+02,
     *0.2278E+02,0.3033E+02,0.4233E+02,0.6237E+02,0.7953E+02/
            DATA ((PHR(I,J),J=1,83),I=10,10) /
     *0.2110E+00,0.2025E+00,0.1832E+00,0.1730E+00,0.1773E+00,0.1912E+00,
     *0.2055E+00,0.2138E+00,0.2152E+00,0.2113E+00,0.2040E+00,0.1946E+00,
     *0.1842E+00,0.1734E+00,0.1627E+00,0.1524E+00,0.1429E+00,0.1344E+00,
     *0.1268E+00,0.1203E+00,0.1149E+00,0.1104E+00,0.1068E+00,0.1040E+00,
     *0.1019E+00,0.1006E+00,0.9982E-01,0.9972E-01,0.1003E+00,0.1014E+00,
     *0.1031E+00,0.1054E+00,0.1084E+00,0.1119E+00,0.1162E+00,0.1212E+00,
     *0.1271E+00,0.1338E+00,0.1415E+00,0.1503E+00,0.1603E+00,0.1658E+00,
     *0.1717E+00,0.1847E+00,0.1995E+00,0.2163E+00,0.2354E+00,0.2571E+00,
     *0.2818E+00,0.3100E+00,0.3422E+00,0.3792E+00,0.4216E+00,0.4702E+00,
     *0.5261E+00,0.5903E+00,0.6644E+00,0.7500E+00,0.8493E+00,0.9645E+00,
     *0.1098E+01,0.1254E+01,0.1436E+01,0.1649E+01,0.1897E+01,0.2189E+01,
     *0.2531E+01,0.2934E+01,0.3408E+01,0.3968E+01,0.4630E+01,0.5415E+01,
     *0.6348E+01,0.7463E+01,0.8805E+01,0.1044E+02,0.1244E+02,0.1495E+02,
     *0.1816E+02,0.2237E+02,0.2799E+02,0.3517E+02,0.3934E+02/
      do i=1,10
        do j=1,83
          ph(i,j)=phr(i,j)
        enddo
      enddo
      return
      end
*
************************************************************************
*
      subroutine oda550 (iaer,v,
     a                   taer55)
 
      double precision bnz,bnz1
      common /sixs_atm/ z(34),p(34),t(34)
      common /sixs_del/ delta,sigma
      real an5(34),an23(34)
      Real v,taer55,z,p,t
      Real delta,sigma,dz,bn5,bn51,bn23,bn231,az
      Real az1,bz,bz1,ev
      Integer iaer,k
c    aerosol optical depth at wl=550nm
c     vertical repartition of aerosol density for v=23km
c                     ( in nbr of part/cm3 )
 
      data an23 /2.828e+03,1.244e+03,5.371e+02,2.256e+02,1.192e+02
     a,8.987e+01,6.337e+01,5.890e+01,6.069e+01,5.818e+01,5.675e+01
     a,5.317e+01,5.585e+01,5.156e+01,5.048e+01,4.744e+01,4.511e+01
     a,4.458e+01,4.314e+01,3.634e+01,2.667e+01,1.933e+01,1.455e+01
     a,1.113e+01,8.826e+00,7.429e+00,2.238e+00,5.890e-01,1.550e-01
     a,4.082e-02,1.078e-02,5.550e-05,1.969e-08,0.000e+00/
 
c     vertical repartition of aerosol density for v=5km
c                     ( in nbr of part/cm3 )
 
      data  an5 /1.378e+04,5.030e+03,1.844e+03,6.731e+02,2.453e+02
     a,8.987e+01,6.337e+01,5.890e+01,6.069e+01,5.818e+01,5.675e+01
     a,5.317e+01,5.585e+01,5.156e+01,5.048e+01,4.744e+01,4.511e+01
     a,4.458e+01,4.314e+01,3.634e+01,2.667e+01,1.933e+01,1.455e+01
     a,1.113e+01,8.826e+00,7.429e+00,2.238e+00,5.890e-01,1.550e-01
     a,4.082e-02,1.078e-02,5.550e-05,1.969e-08,0.000e+00/
 
 
      taer55=0.
 
      if(abs(v).le.0.) return
      if(iaer.eq.0) return
 
      do 1 k=1,32
      dz=z(k+1)-z(k)
      bn5=an5(k)
      bn51=an5(k+1)
      bn23=an23(k)
      bn231=an23(k+1)
      az=(115./18.)*(bn5-bn23)
      az1=(115./18.)*(bn51-bn231)
      bz=(5.*bn5/18.)-(23.*bn23/18.)
      bz1=(5.*bn51/18.)-(23.*bn231/18.)
      bnz=dble(az)/dble(v)-dble(bz)
      bnz1=dble(az1)/dble(v)-dble(bz1)
      ev=real(dble(dz)*exp((dlog(bnz)+dlog(bnz1))*.5d0))
      taer55=taer55+ev*sigma*1.0e-03
    1 continue
      return
      end
*
************************************************************************
*
      subroutine odrayl ( wl,
     a                   tray)
      double precision pi,a1,a2,a3,a4,awl,an,ak,a,ns
      real wl,tray,z,p,t,delta,sigma,dppt,sr
      integer k
c     molecular optical depth
 
      common /sixs_atm/ z(34),p(34),t(34)
      common /sixs_del/ delta,sigma
      data pi /3.1415926d0/
      ak=1.0d0/dble(wl)
      awl=dble(wl)
c     air refraction index edlen 1966 / metrologia,2,71-80  putting pw=0
      a1=130.d0-ak*ak
      a2=38.9d0-ak*ak
      a3=2406030.d0/a1
      a4=15997.d0/a2
      an=(8342.13d0+a3+a4)*1.0d-08
      an=an+1.d+00
      a=(24.d0*pi**3)*((an*an-1.d0)**2)*(6.d0+3.d0*dble(delta))/
     s        (6.d0-7.d0*dble(delta))/((an*an+2.d0)**2)
      tray=0.
      do k=1,33
      ns=2.54743d+19
      dppt=(288.15/1013.25)*(p(k)/t(k)+p(k+1)/t(k+1))/2.
      sr=real((a*dble(dppt)/(awl**4)/ns*1.d+16)*1.d+05)
      tray=tray+(z(k+1)-z(k))*sr
      enddo
      return
      end
*
************************************************************************
*
      subroutine os (tamoy,trmoy,pizmoy,tamoyp,trmoyp,palt,
     s               phirad,nt,mu,np,rm,gb,rp,
     s                     xl)
c  dimension for gauss integration
      integer mu,np
      real rm(-mu:mu),gb(-mu:mu),rp(np)
c  dimension for os computation
      real  xl(-mu:mu,np)
c array for sos computation
      real xpl(-25:25),psl(-1:80,-25:25),bp(0:25,-25:25),
     s xdel(0:30),ydel(0:30),ch(0:30),h(0:30)
      real i1(0:30,-25:25),i2(0:30,-25:25),i3(-25:25),
     s   i4(-25:25),in(-25:25),inm1(-25:25),inm2(-25:25)
      real altc(0:30)
      Real tamoy,trmoy,pizmoy
      Real tamoyp,trmoyp,palt,phirad
      Real delta,sigma,hr,ta,tr,trp
      Real tap,piz,accu,ha,xmus,zx,yy,dd,ppp2,ppp1,ca,cr,ratio
      Real taup,th,xt1,xt2,pi,phi,aaaa,ron
      Real beta0,beta2,roavion0,roavion1,roavion2,roavion,spl,sa1
      Real sa2,c,zi1,f,d,xpk,y
      Real a1,d1,g1,y1,delta0s
      integer snt
      integer nt,iwr,iplane,mum1,ntp,j,it,i,l,m,iborm
      integer is,ig,k,jj,index
      logical ier
      common/sixs_del/delta,sigma
      common/sixs_ier/iwr,ier
      double precision xx,xdb,bpjk,bpjmk,z,xi1,xi2,x,xpj,ypk,a,b,ii1,ii2
c the optical thickness above plane are recomputed to give o.t above pla
c     write(6,*) 'tamoy,trmoy,tamoyp,trmoyp,palt,pizmoy'
c     write(6,*) tamoy,trmoy,tamoyp,trmoyp,palt,pizmoy
c     write(6,*) 'betal 0:80'
c     do i=0,80
c     write(6,*) i,betal(i)
c     enddo
c     write(6,*) 'phase function 83 terms'
c     do i=1,83
c     write(6,*) pha(i)
c     enddo
      snt=nt
      hr=8.0
      ta=tamoy
      tr=trmoy
      trp=trmoy-trmoyp
      tap=tamoy-tamoyp
      piz=pizmoy
c     print *, 'ta,tr,piz,tap,trp,palt,nt'
c     print *,ta,tr,piz,tap,trp,palt,nt
      iplane=0
      accu=1.e-20
      mum1=mu-1
c if plane observations recompute scale height for aerosol knowing:
c the aerosol optical depth as measure from the plane       = tamoyp
c the rayleigh scale   height =       		= hr (8km)
c the rayleigh optical depth  at plane level       	= trmoyp
c the altitude of the plane       			= palt
c the rayleigh optical depth for total atmos      	= trmoy
c the aerosol  optical depth for total atmos      	= tamoy
c if not plane observations then ha is equal to 2.0km
c ntp local variable: if ntp=nt     no plane observation selected
c                        ntp=nt-1   plane observation selected
c     it's a mixing rayleigh+aerosol
      if(palt.le.900..and.palt.gt.0.0) then
      if (tap.gt.1.e-03) then
         ha=-palt/log(tap/ta)
         else
         ha=2.
         endif
      ntp=nt-1
      else
      ha=2.0
      ntp=nt
      endif
c
      xmus=-rm(0)
c
c compute mixing rayleigh, aerosol
c case 1: pure rayleigh
c case 2: pure aerosol
c case 3: mixing rayleigh-aerosol
c
      if(ta.le.accu) then
      do j=0,ntp
      h(j)=j*tr/ntp
      ch(j)=exp(-h(j)/xmus)/2.
      ydel(j)=1.0
      xdel(j)=0.0
      if (j.eq.0) then
         altc(j)=300.
         else
         altc(j)=-log(h(j)/tr)*hr
         endif
      enddo
      endif
      if(tr.le.accu) then
      do j=0,ntp
      h(j)=j*ta/ntp
      ch(j)=exp(-h(j)/xmus)/2.
      ydel(j)=0.0
      xdel(j)=piz
      if (j.eq.0) then
         altc(j)=300.
         else
         altc(j)=-log(h(j)/ta)*ha
         endif
      enddo
      endif
c
      if(tr.gt.accu.and.ta.gt.accu)then
      ydel(0)=1.0
      xdel(0)=0.0
      h(0)=0.
      ch(0)=0.5
      altc(0)=300.
      zx=300.
      iplane=0
      do 14 it=0,ntp
      if (it.eq.0) then
         yy=0.
         dd=0.
         goto 111
      endif
      yy=h(it-1)
      dd=ydel(it-1)
 111  ppp2=300.0
      ppp1=0.0
      call discre(ta,ha,tr,hr,it,ntp,yy,dd,ppp2,ppp1,
     s    zx)
      if(ier)return
      xx=-dble(zx)/dble(ha)
      if (xx.le.-20.d0) then
         ca=0.
         else
         ca=real(dble(ta)*dexp(xx))
         endif
      xx=-dble(zx)/dble(hr)
      cr=real(dble(tr)*dexp(xx))
      h(it)=cr+ca
      altc(it)=zx
      ch(it)=exp(-h(it)/xmus)/2.
      cr=cr/hr
      ca=ca/ha
      ratio=cr/(cr+ca)
      xdel(it)=(1.e+00-ratio)*piz
      ydel(it)=ratio
c     print *,'discre ',it,cr,ca,xdel(it),ydel(it),zx
  14  continue
      endif
c update plane layer if necessary
      if (ntp.eq.(nt-1)) then
c compute position of the plane layer
         taup=tap+trp
         iplane=-1
         do i=0,ntp
         if (taup.ge.h(i)) iplane=i
         enddo
c update the layer from the end to the position to update if necessary
         th=0.005
         xt1=abs(h(iplane)-taup)
         xt2=abs(h(iplane+1)-taup)
         if ((xt1.gt.th).and.(xt2.gt.th)) then
         do i=nt,iplane+1,-1
            xdel(i)=xdel(i-1)
            ydel(i)=ydel(i-1)
            h(i)=h(i-1)
            altc(i)=altc(i-1)
            ch(i)=ch(i-1)
         enddo
         else
         nt=ntp
         if (xt2.lt.xt1) iplane=iplane+1
         endif
         h(iplane)=taup
         if ( tr.gt.accu.and.ta.gt.accu) then
         ca=ta*exp(-palt/ha)
         cr=tr*exp(-palt/hr)
         cr=cr/hr
         ca=ca/ha
         ratio=cr/(cr+ca)
         xdel(iplane)=(1.e+00-ratio)*piz
         ydel(iplane)=ratio
         altc(iplane)=palt
         h(iplane)=ca+cr
         ch(iplane)=exp(-h(iplane)/xmus)/2.
         endif
         if ( tr.gt.accu.and.ta.le.accu) then
         ydel(iplane)=1.
         xdel(iplane)=0.
         altc(iplane)=palt
         endif
         if ( tr.le.accu.and.ta.gt.accu) then
         ydel(iplane)=0.
         xdel(iplane)=1.*piz
         altc(iplane)=palt
         endif
      endif
c
c
c     print *,ha,hr,palt,ta,tr,tap,trp
c     do i=0,nt
c     print *,i,h(i),ch(i),xdel(i),ydel(i),altc(i)
c     enddo
c
      pi=acos(-1.)
      phi=phirad
      do l=1,np
        do m=-mu,mu
          xl(m,l)=0.
        enddo
      enddo
c
c     ************ incident angle mus *******
c
c
      aaaa=delta/(2-delta)
      ron=(1-aaaa)/(1+2*aaaa)
c     write(6,*) 'ron ',ron
c
c     rayleigh phase function
c
      beta0=1.
      beta2=0.5*ron
c
c     fourier decomposition
c
      do 17 j=-mu,mu
      i4(j)=0.
   17 continue
      iborm=80
      if( abs (xmus-1.000000) .lt.1.e-06)iborm=0
      do 24 is=0,iborm
c
c    primary scattering
c
      ig=1
      roavion0=0.
      roavion1=0.
      roavion2=0.
      roavion=0.
      do 16 j=-mu,mu
      i3(j)=0.
   16 continue
c
c     kernel computations
c
      call kernel(is,mu,rm,xpl,psl,bp)
      if(is.gt.0)beta0=0.
      do 100 j=-mu,mu
      if ((is-2) .le. 0) then
        spl=xpl(0)
        sa1=beta0+beta2*xpl(j)*spl
        sa2=bp(0,j)
      else
        sa2=bp(0,j)
        sa1=0.
      endif
c
c     primary scattering source function at every level within the layer
c
      do 101 k=0,nt
      c=ch(k)
      a=dble(ydel(k))
      b=dble(xdel(k))
      i2(k,j)=real(dble(c)*(dble(sa2)*b+dble(sa1)*a))
  101 continue
  100 continue
c
c     vertical integration, primary upward radiation
c
 
      do k=1,mu
        i1(nt,k)=0.
        zi1=i1(nt,k)
        yy=rm(k)
        do i=nt-1,0,-1
          jj=i+1
          f=h(jj)-h(i)
          if (abs(f) .gt. accu) then
            a=(dble(i2(jj,k))-dble(i2(i,k)))/dble(f)
            b=dble(i2(i,k))-a*dble(h(i))
            c=exp(-f/yy)
            d=1.0e+00-c
            xx=dble(h(i))-dble(h(jj))*dble(c)
            zi1=real(dble(c*zi1)+(dble(d)*(b+a*dble(yy))+a*xx)*0.5d+00)
          endif
          i1(i,k)=zi1
        enddo
      enddo
c
c     vertical integration, primary downward radiation
c
      do k=-mu,-1
        i1(0,k)=0.
        zi1=i1(0,k)
        yy=rm(k)
        do i=1,nt
          jj=i-1
          f=h(i)-h(jj)
          if (abs(f) .gt. accu) then
            c=exp(f/yy)
            d=1.0e+00-c
            a=(dble(i2(i,k))-dble(i2(jj,k)))/dble(f)
            b=dble(i2(i,k))-a*dble(h(i))
            xx=dble(h(i))-dble(h(jj))*dble(c)
            zi1=c*zi1+real((dble(d)*(b+a*dble(yy))+a*xx)*0.5d+00)
          endif
          i1(i,k)=zi1
        enddo
      enddo
c
c     inm2 is inialized with scattering computed at n-2
c     i3 is inialized with primary scattering
c
      do 20 k=-mu,mu
      if (k .eq. 0) then
        goto 20
      else if (k .lt. 0) then
        index=nt
      else
        index=0
      endif
      inm1(k)=i1(index,k)
      inm2(k)=i1(index,k)
      i3(k)=i1(index,k)
   20 continue
      roavion2=i1(iplane,mu)
      roavion=i1(iplane,mu)
c
c     loop on successive order
c
  503 ig=ig+1
c     write(6,*) 'ig ',ig
c
c     successive orders
c
c     multiple scattering source function at every level within the laye
c
c     if is < ou = 2 kernels are a mixing of aerosols and molecules kern
c     if is >2 aerosols kernels only
c
      if ((is - 2) .le. 0) then
      do k=1,mu
      xpk=xpl(k)
      ypk=dble(xpl(-k))
        do i=0,nt
          ii1=0.d0
          ii2=0.d0
          x=dble(xdel(i))
          y=ydel(i)
          do j=1,mu
            xpj=dble(xpl(j))
            z=dble(gb(j))
            xi1=dble(i1(i,j))
            xi2=dble(i1(i,-j))
            bpjk=dble(bp(j,k))*x+dble(y)*(dble(beta0)+
     &           dble(beta2)*xpj*dble(xpk))
            bpjmk=dble(bp(j,-k))*x+dble(y)*(dble(beta0)+
     &            dble(beta2)*xpj*ypk)
            xdb=z*(xi1*bpjk+xi2*bpjmk)
            ii2=ii2+xdb
            xdb=z*(xi1*bpjmk+xi2*bpjk)
            ii1=ii1+xdb
          enddo
          if (ii2.lt.1.D-30) ii2=0.d0
          if (ii1.lt.1.D-30) ii1=0.d0
          i2(i,k)=real(ii2)
          i2(i,-k)=real(ii1)
        enddo
      enddo
      else
      do k=1,mu
        do i=0,nt
          ii1=0.d0
          ii2=0.d0
          x=dble(xdel(i))
          do j=1,mu
            z=dble(gb(j))
            xi1=dble(i1(i,j))
            xi2=dble(i1(i,-j))
            bpjk=dble(bp(j,k))*x
            bpjmk=dble(bp(j,-k))*x
            xdb=z*(xi1*bpjk+xi2*bpjmk)
            ii2=ii2+xdb
            xdb=z*(xi1*bpjmk+xi2*bpjk)
            ii1=ii1+xdb
          enddo
          if (ii2.lt.1.D-30) ii2=0.d0
          if (ii1.lt.1.D-30) ii1=0.d0
          i2(i,k)=real(ii2)
          i2(i,-k)=real(ii1)
        enddo
      enddo
      endif
c
c     vertical integration, upward radiation
c
      do k=1,mu
        i1(nt,k)=0.
        zi1=i1(nt,k)
        yy=rm(k)
        do i=nt-1,0,-1
          jj=i+1
          f=h(jj)-h(i)
          if (abs(f) .gt. accu) then
             a=(dble(i2(jj,k))-dble(i2(i,k)))/dble(f)
             b=dble(i2(i,k))-a*dble(h(i))
             c=exp(-f/yy)
             d=1.e+00-c
             xx=dble(h(i))-dble(h(jj))*dble(c)
             zi1=c*zi1+real((dble(d)*(b+a*dble(yy))+a*xx)*0.5d+00)
          endif
          i1(i,k)=zi1
        enddo
      enddo
c
c     vertical integration, downward radiation
c
      do k=-mu,-1
        i1(0,k)=0.
        zi1=i1(0,k)
        yy=rm(k)
        do i=1,nt
          jj=i-1
          f=h(i)-h(jj)
          if (abs(f) .gt. accu) then
            c=exp(f/yy)
            d=1.e+00-c
            a=(dble(i2(i,k))-dble(i2(jj,k)))/dble(f)
            b=dble(i2(i,k))-a*dble(h(i))
            xx=dble(h(i))-dble(h(jj))*dble(c)
            zi1=c*zi1+real((dble(d)*(dble(b)+a*dble(yy))+a*xx)*0.5d+00)
          endif
          i1(i,k)=zi1
        enddo
      enddo
c
c     in is the nieme scattering order
c
      do 30 k=-mu,mu
      if (k .eq. 0) then
        goto 30
      else if (k .lt. 0) then
        index=nt
      else
        index=0
      endif
      in(k)=i1(index,k)
   30 continue
      roavion0=i1(iplane,mu)
c
c   convergence test (geometrical serie)
c
      if(ig.gt.2) then
      z=0.d0
      a1=roavion2
      d1=roavion1
      g1=roavion0
      if(a1.ge.accu.and.d1.ge.accu.and.roavion.ge.accu) then
      y=((g1/d1-d1/a1)/((1-g1/d1)**2)*(g1/roavion))
      y=abs(y)
      z=dmax1(dble(y),z)
      endif
      do 99 l=-mu,mu
      if (l.eq.0) goto 99
      a1=inm2(l)
      d1=inm1(l)
      g1=in(l)
      if(a1.le.accu) go to 99
      if(d1.le.accu) go to 99
      if(i3(l).le.accu) go to 99
      y=((g1/d1-d1/a1)/((1-g1/d1)**2)*(g1/i3(l)))
      y=abs(y)
      z=dmax1(dble(y),z)
  99  continue
      if(z.lt.0.0001d0) then
c
c     successful test (geometrical serie)
c
      do 606 l=-mu,mu
      y1=1.
      d1=inm1(l)
      g1=in(l)
      if(d1.le.accu) go to 606
      y1=1-g1/d1
      if(abs(g1-d1).le.accu) then
      go to 606
      endif
      g1=g1/y1
      i3(l)=i3(l)+g1
  606 continue
      d1=roavion1
      g1=roavion0
      y1=1.
      if(d1.ge.accu) then
      if(abs(g1-d1).ge.accu) then
      y1=1-g1/d1
      g1=g1/y1
      endif
      roavion=roavion+g1
      endif
      go to 505
      endif
c
c     inm2 is the (n-2)ieme scattering order
c
      do 26 k=-mu,mu
      inm2(k)=inm1(k)
   26 continue
      roavion2=roavion1
      endif
c
c     inm1 is the (n-1)ieme scattering order
c
      do 27 k=-mu,mu
      inm1(k)=in(k)
   27 continue
      roavion1=roavion0
c
c     sum of the n-1 orders
c
      do 610 l=-mu,mu
      i3(l)=i3(l)+in(l)
  610 continue
      roavion=roavion+roavion0
c
c     stop if order n is less than 1% of the sum
c
      z=0.d0
      do 611 l=-mu,mu
      if (abs(i3(l)).ge.accu) then
      y=abs(in(l)/i3(l))
      z=dmax1(z,dble(y))
      endif
  611 continue
      if(z.lt.0.00001d0) go to 505
c
c      stop if order n is greater than 20 in any case
c
      if ((ig - 20) .le. 0) goto 503
  505 continue
c
c     sum of the fourier component s
c
      delta0s=1
      if(is.ne.0) delta0s=2
      do 612 l=-mu,mu
      i4(l)=i4(l)+delta0s*i3(l)
  612 continue
c
c     stop of the fourier decomposition
c
      do l=1,np
        phi=rp(l)
        do m=-mum1,mum1
          if(m.gt.0) then
          xl(m,l)=xl(m,l)+delta0s*i3(m)*cos(is*(phi+pi))
          else
          xl(m,l)=xl(m,l)+delta0s*i3(m)*cos(is*phi)
          endif
        enddo
      enddo
      if(is.eq.0) then
      do k=1,mum1
      xl(0,1)=xl(0,1)+rm(k)*gb(k)*i3(-k)
      enddo
      endif
      xl(mu,1)=xl(mu,1)+delta0s*i3(mu)*cos(is*(phirad+pi))
      xl(-mu,1)=xl(-mu,1)+delta0s*roavion*cos(is*(phirad+pi))
      z=0.d0
      do 613 l=-mu,mu
       if (abs(i4(l)).lt.accu) goto 613
      x=abs(dble(i3(l))/dble(i4(l)))
      z=dmax1(z,x)
  613 continue
      if(z.gt.0.001d0) go to 24
      goto 243
   24 continue
  243 continue
      nt=snt
c     write(6,*) 'reflectance ', xl(mu,1)/xmus
      return
      end
*
************************************************************************
*
      subroutine print_error(tex)
      character *(*) tex
      logical ier
      integer iwr
      common/sixs_ier/iwr,ier
      ier = .TRUE.
      write(iwr,'(a)')tex
      return
      end
*
************************************************************************
*
      subroutine scatra (taer,taerp,tray,trayp,piza,
     a      palt,nt,mu,rm,gb,xmus,xmuv,
     a                   ddirtt,ddiftt,udirtt,udiftt,sphlbt,
     a                   ddirtr,ddiftr,udirtr,udiftr,sphlbr,
     a                   ddirta,ddifta,udirta,udifta,sphlba)
 
      integer mu
      real rm(-mu:mu),gb(-mu:mu)
c     computations of the direct and diffuse transmittances
c     for downward and upward paths , and spherical albedo
      real xtrans(-1:1)
      real taer,taerp,tray,trayp,piza,palt,xmus,xmuv
      real udiftt,sphlbt,ddirtr,ddiftr,udirtr,udiftr,sphlbr
      real ddirtt,ddiftt,udirtt,ddirta,ddifta,udirta,udifta
      real sphlba,tamol,tamolp
      integer nt,it
c
      ddirtt=1.
      ddiftt=0.
      udirtt=1.
      udiftt=0.
      sphlbt=0.
      ddirtr=1.
      ddiftr=0.
      udirtr=1.
      udiftr=0.
      sphlbr=0.
      ddirta=1.
      ddifta=0.
      udirta=1.
      udifta=0.
      sphlba=0.
 
      do 1 it=1,3
c it=1 rayleigh only, it=2 aerosol only, it=3 rayleigh+aerosol
      if (it.eq.2.and.taer.le.0.) goto 1
c     compute upward,downward diffuse transmittance for rayleigh,aerosol
      if (it.eq.1) then
         if (palt.gt.900) then
         udiftt=(2./3.+xmuv)+(2./3.-xmuv)*exp(-tray/xmuv)
         udiftt=udiftt/((4./3.)+tray)-exp(-tray/xmuv)
         ddiftt=(2./3.+xmus)+(2./3.-xmus)*exp(-tray/xmus)
         ddiftt=ddiftt/((4./3.)+tray)-exp(-tray/xmus)
         ddirtt=exp(-tray/xmus)
         udirtt=exp(-tray/xmuv)
         call csalbr(tray,sphlbt)
         endif
         if (palt.lt.900) then
           tamol=0.
           tamolp=0.
           rm(-mu)=-xmuv
           rm(mu)=xmuv
           rm(0)=xmus
         call iso(tamol,tray,piza,tamolp,trayp,palt,
     a         nt,mu,rm,gb,xtrans)
         udiftt=xtrans(-1)-exp(-trayp/xmuv)
         udirtt=exp(-trayp/xmuv)
         rm(-mu)=-xmus
         rm(mu)=xmus
         rm(0)=xmus
         call iso(tamol,tray,piza,tamolp,trayp,palt,
     a         nt,mu,rm,gb,xtrans)
         ddiftt=xtrans(1)-exp(-tray/xmus)
         ddirtt=exp(-tray/xmus)
         sphlbt=xtrans(0)*2.
         endif
         if (palt.le.0.) then
            udiftt=0.
            udirtt=1.
         endif
      endif
      if (it.eq.2) then
      tamol=0.
      tamolp=0.
      rm(-mu)=-xmuv
      rm(mu)=xmuv
      rm(0)=xmus
      call iso(taer,tamol,piza,taerp,tamolp,palt,
     a         nt,mu,rm,gb,xtrans)
      udiftt=xtrans(-1)-exp(-taerp/xmuv)
      udirtt=exp(-taerp/xmuv)
      rm(-mu)=-xmus
      rm(mu)=xmus
      rm(0)=xmus
      call iso(taer,tamol,piza,taerp,tamolp,palt,
     a         nt,mu,rm,gb,xtrans)
      ddirtt=exp(-taer/xmus)
      ddiftt=xtrans(1)-exp(-taer/xmus)
      sphlbt=xtrans(0)*2.
      if (palt.le.0.) then
         udiftt=0.
         udirtt=1.
       endif
      endif
      if (it.eq.3) then
      rm(-mu)=-xmuv
      rm(mu)=xmuv
      rm(0)=xmus
      call iso(taer,tray,piza,taerp,trayp,palt,
     a         nt,mu,rm,gb,xtrans)
      udirtt=exp(-(taerp+trayp)/xmuv)
      udiftt=xtrans(-1)-exp(-(taerp+trayp)/xmuv)
      rm(-mu)=-xmus
      rm(mu)=xmus
      rm(0)=xmus
      call iso(taer,tray,piza,taerp,trayp,palt,
     a         nt,mu,rm,gb,xtrans)
      ddiftt=xtrans(1)-exp(-(taer+tray)/xmus)
      ddirtt=exp(-(taer+tray)/xmus)
      sphlbt=xtrans(0)*2.
      if (palt.le.0.) then
         udiftt=0.
         udirtt=1.
       endif
      endif
c     write(6,*) ddirtt,ddiftt,it,tray,taer,trayp,taerp
 
      if (it.eq.2) goto 2
      if (it.eq.3) goto 1
      ddirtr=ddirtt
      ddiftr=ddiftt
      udirtr=udirtt
      udiftr=udiftt
      sphlbr=sphlbt
      goto 1
    2 ddirta=ddirtt
      ddifta=ddiftt
      udirta=udirtt
      udifta=udiftt
      sphlba=sphlbt
    1 continue
      return
      end
*
************************************************************************
*
      subroutine   soot
      real ph,phr
      integer i,j
      common /sixs_aerbas/ ph(10,83)
      dimension phr(10,83)
c
c    model: soot
c
            DATA ((PHR(I,J),J=1,83),I=01,01) /
     *0.4897E+00,0.4896E+00,0.4890E+00,0.4881E+00,0.4867E+00,0.4849E+00,
     *0.4827E+00,0.4802E+00,0.4773E+00,0.4743E+00,0.4709E+00,0.4675E+00,
     *0.4638E+00,0.4601E+00,0.4563E+00,0.4526E+00,0.4489E+00,0.4453E+00,
     *0.4419E+00,0.4388E+00,0.4359E+00,0.4334E+00,0.4312E+00,0.4296E+00,
     *0.4285E+00,0.4281E+00,0.4283E+00,0.4293E+00,0.4312E+00,0.4341E+00,
     *0.4380E+00,0.4430E+00,0.4494E+00,0.4571E+00,0.4663E+00,0.4771E+00,
     *0.4896E+00,0.5041E+00,0.5206E+00,0.5392E+00,0.5603E+00,0.5717E+00,
     *0.5838E+00,0.6101E+00,0.6392E+00,0.6714E+00,0.7069E+00,0.7459E+00,
     *0.7886E+00,0.8352E+00,0.8860E+00,0.9411E+00,0.1001E+01,0.1065E+01,
     *0.1135E+01,0.1210E+01,0.1290E+01,0.1376E+01,0.1468E+01,0.1566E+01,
     *0.1670E+01,0.1781E+01,0.1897E+01,0.2019E+01,0.2148E+01,0.2282E+01,
     *0.2421E+01,0.2565E+01,0.2713E+01,0.2865E+01,0.3019E+01,0.3173E+01,
     *0.3327E+01,0.3479E+01,0.3625E+01,0.3765E+01,0.3894E+01,0.4011E+01,
     *0.4111E+01,0.4192E+01,0.4250E+01,0.4284E+01,0.4292E+01/
            DATA ((PHR(I,J),J=1,83),I=02,02) /
     *0.5620E+00,0.5618E+00,0.5611E+00,0.5599E+00,0.5582E+00,0.5560E+00,
     *0.5533E+00,0.5502E+00,0.5467E+00,0.5428E+00,0.5387E+00,0.5342E+00,
     *0.5295E+00,0.5246E+00,0.5197E+00,0.5146E+00,0.5096E+00,0.5046E+00,
     *0.4998E+00,0.4951E+00,0.4907E+00,0.4866E+00,0.4829E+00,0.4797E+00,
     *0.4771E+00,0.4751E+00,0.4738E+00,0.4734E+00,0.4738E+00,0.4753E+00,
     *0.4779E+00,0.4817E+00,0.4868E+00,0.4934E+00,0.5016E+00,0.5114E+00,
     *0.5231E+00,0.5367E+00,0.5524E+00,0.5704E+00,0.5908E+00,0.6019E+00,
     *0.6137E+00,0.6393E+00,0.6678E+00,0.6993E+00,0.7340E+00,0.7720E+00,
     *0.8136E+00,0.8589E+00,0.9081E+00,0.9613E+00,0.1019E+01,0.1080E+01,
     *0.1147E+01,0.1218E+01,0.1293E+01,0.1373E+01,0.1459E+01,0.1549E+01,
     *0.1643E+01,0.1743E+01,0.1847E+01,0.1956E+01,0.2069E+01,0.2185E+01,
     *0.2305E+01,0.2428E+01,0.2553E+01,0.2679E+01,0.2806E+01,0.2931E+01,
     *0.3055E+01,0.3174E+01,0.3289E+01,0.3396E+01,0.3495E+01,0.3582E+01,
     *0.3656E+01,0.3716E+01,0.3758E+01,0.3782E+01,0.3788E+01/
            DATA ((PHR(I,J),J=1,83),I=03,03) /
     *0.5834E+00,0.5832E+00,0.5825E+00,0.5813E+00,0.5795E+00,0.5771E+00,
     *0.5743E+00,0.5710E+00,0.5673E+00,0.5632E+00,0.5587E+00,0.5540E+00,
     *0.5490E+00,0.5438E+00,0.5384E+00,0.5330E+00,0.5275E+00,0.5221E+00,
     *0.5168E+00,0.5117E+00,0.5068E+00,0.5023E+00,0.4981E+00,0.4944E+00,
     *0.4913E+00,0.4889E+00,0.4871E+00,0.4862E+00,0.4862E+00,0.4872E+00,
     *0.4894E+00,0.4928E+00,0.4975E+00,0.5037E+00,0.5115E+00,0.5210E+00,
     *0.5324E+00,0.5457E+00,0.5611E+00,0.5788E+00,0.5988E+00,0.6098E+00,
     *0.6215E+00,0.6468E+00,0.6749E+00,0.7061E+00,0.7405E+00,0.7781E+00,
     *0.8193E+00,0.8641E+00,0.9127E+00,0.9652E+00,0.1022E+01,0.1083E+01,
     *0.1148E+01,0.1217E+01,0.1291E+01,0.1370E+01,0.1453E+01,0.1541E+01,
     *0.1633E+01,0.1730E+01,0.1831E+01,0.1936E+01,0.2045E+01,0.2157E+01,
     *0.2272E+01,0.2390E+01,0.2509E+01,0.2629E+01,0.2749E+01,0.2867E+01,
     *0.2984E+01,0.3096E+01,0.3203E+01,0.3304E+01,0.3395E+01,0.3476E+01,
     *0.3545E+01,0.3599E+01,0.3638E+01,0.3660E+01,0.3666E+01/
            DATA ((PHR(I,J),J=1,83),I=04,04) /
     *0.6060E+00,0.6059E+00,0.6051E+00,0.6038E+00,0.6019E+00,0.5994E+00,
     *0.5964E+00,0.5929E+00,0.5889E+00,0.5846E+00,0.5798E+00,0.5747E+00,
     *0.5693E+00,0.5637E+00,0.5580E+00,0.5521E+00,0.5462E+00,0.5403E+00,
     *0.5345E+00,0.5289E+00,0.5235E+00,0.5185E+00,0.5138E+00,0.5096E+00,
     *0.5059E+00,0.5029E+00,0.5007E+00,0.4993E+00,0.4988E+00,0.4993E+00,
     *0.5010E+00,0.5040E+00,0.5083E+00,0.5142E+00,0.5216E+00,0.5307E+00,
     *0.5418E+00,0.5548E+00,0.5699E+00,0.5873E+00,0.6071E+00,0.6180E+00,
     *0.6295E+00,0.6546E+00,0.6825E+00,0.7134E+00,0.7474E+00,0.7848E+00,
     *0.8255E+00,0.8699E+00,0.9179E+00,0.9698E+00,0.1026E+01,0.1085E+01,
     *0.1150E+01,0.1218E+01,0.1290E+01,0.1367E+01,0.1448E+01,0.1534E+01,
     *0.1623E+01,0.1717E+01,0.1815E+01,0.1916E+01,0.2020E+01,0.2128E+01,
     *0.2237E+01,0.2349E+01,0.2462E+01,0.2576E+01,0.2688E+01,0.2800E+01,
     *0.2909E+01,0.3013E+01,0.3113E+01,0.3206E+01,0.3290E+01,0.3364E+01,
     *0.3427E+01,0.3477E+01,0.3512E+01,0.3532E+01,0.3537E+01/
            DATA ((PHR(I,J),J=1,83),I=05,05) /
     *0.6604E+00,0.6602E+00,0.6593E+00,0.6578E+00,0.6556E+00,0.6528E+00,
     *0.6494E+00,0.6454E+00,0.6409E+00,0.6358E+00,0.6304E+00,0.6245E+00,
     *0.6182E+00,0.6117E+00,0.6050E+00,0.5981E+00,0.5911E+00,0.5841E+00,
     *0.5771E+00,0.5703E+00,0.5636E+00,0.5573E+00,0.5513E+00,0.5458E+00,
     *0.5409E+00,0.5366E+00,0.5331E+00,0.5305E+00,0.5288E+00,0.5281E+00,
     *0.5287E+00,0.5305E+00,0.5338E+00,0.5385E+00,0.5450E+00,0.5532E+00,
     *0.5633E+00,0.5754E+00,0.5897E+00,0.6062E+00,0.6252E+00,0.6356E+00,
     *0.6467E+00,0.6710E+00,0.6980E+00,0.7280E+00,0.7610E+00,0.7972E+00,
     *0.8367E+00,0.8797E+00,0.9261E+00,0.9762E+00,0.1030E+01,0.1087E+01,
     *0.1149E+01,0.1214E+01,0.1283E+01,0.1355E+01,0.1432E+01,0.1512E+01,
     *0.1595E+01,0.1682E+01,0.1772E+01,0.1865E+01,0.1961E+01,0.2058E+01,
     *0.2157E+01,0.2257E+01,0.2358E+01,0.2458E+01,0.2557E+01,0.2654E+01,
     *0.2748E+01,0.2838E+01,0.2923E+01,0.3001E+01,0.3072E+01,0.3134E+01,
     *0.3187E+01,0.3228E+01,0.3257E+01,0.3273E+01,0.3277E+01/
            DATA ((PHR(I,J),J=1,83),I=06,06) /
     *0.6993E+00,0.6991E+00,0.6982E+00,0.6965E+00,0.6942E+00,0.6911E+00,
     *0.6874E+00,0.6830E+00,0.6781E+00,0.6726E+00,0.6666E+00,0.6601E+00,
     *0.6533E+00,0.6461E+00,0.6387E+00,0.6310E+00,0.6232E+00,0.6154E+00,
     *0.6076E+00,0.5998E+00,0.5923E+00,0.5851E+00,0.5782E+00,0.5717E+00,
     *0.5659E+00,0.5607E+00,0.5562E+00,0.5526E+00,0.5500E+00,0.5485E+00,
     *0.5482E+00,0.5491E+00,0.5515E+00,0.5555E+00,0.5611E+00,0.5686E+00,
     *0.5779E+00,0.5893E+00,0.6028E+00,0.6187E+00,0.6369E+00,0.6470E+00,
     *0.6577E+00,0.6812E+00,0.7074E+00,0.7366E+00,0.7687E+00,0.8040E+00,
     *0.8425E+00,0.8843E+00,0.9295E+00,0.9781E+00,0.1030E+01,0.1086E+01,
     *0.1145E+01,0.1208E+01,0.1274E+01,0.1344E+01,0.1417E+01,0.1494E+01,
     *0.1573E+01,0.1656E+01,0.1741E+01,0.1828E+01,0.1918E+01,0.2009E+01,
     *0.2101E+01,0.2194E+01,0.2287E+01,0.2380E+01,0.2470E+01,0.2559E+01,
     *0.2645E+01,0.2726E+01,0.2803E+01,0.2873E+01,0.2937E+01,0.2992E+01,
     *0.3038E+01,0.3075E+01,0.3100E+01,0.3115E+01,0.3118E+01/
            DATA ((PHR(I,J),J=1,83),I=07,07) /
     *0.7916E+00,0.7914E+00,0.7903E+00,0.7883E+00,0.7855E+00,0.7818E+00,
     *0.7773E+00,0.7721E+00,0.7662E+00,0.7595E+00,0.7522E+00,0.7444E+00,
     *0.7360E+00,0.7272E+00,0.7180E+00,0.7085E+00,0.6988E+00,0.6889E+00,
     *0.6790E+00,0.6692E+00,0.6595E+00,0.6500E+00,0.6408E+00,0.6321E+00,
     *0.6239E+00,0.6164E+00,0.6097E+00,0.6038E+00,0.5989E+00,0.5952E+00,
     *0.5926E+00,0.5915E+00,0.5918E+00,0.5936E+00,0.5972E+00,0.6027E+00,
     *0.6101E+00,0.6195E+00,0.6311E+00,0.6451E+00,0.6614E+00,0.6705E+00,
     *0.6803E+00,0.7017E+00,0.7259E+00,0.7529E+00,0.7828E+00,0.8156E+00,
     *0.8514E+00,0.8903E+00,0.9323E+00,0.9774E+00,0.1026E+01,0.1077E+01,
     *0.1131E+01,0.1189E+01,0.1249E+01,0.1312E+01,0.1378E+01,0.1447E+01,
     *0.1518E+01,0.1590E+01,0.1665E+01,0.1741E+01,0.1819E+01,0.1897E+01,
     *0.1976E+01,0.2054E+01,0.2132E+01,0.2209E+01,0.2284E+01,0.2356E+01,
     *0.2426E+01,0.2491E+01,0.2552E+01,0.2607E+01,0.2657E+01,0.2700E+01,
     *0.2736E+01,0.2764E+01,0.2783E+01,0.2795E+01,0.2797E+01/
            DATA ((PHR(I,J),J=1,83),I=08,08) /
     *0.1041E+01,0.1040E+01,0.1038E+01,0.1036E+01,0.1031E+01,0.1026E+01,
     *0.1019E+01,0.1011E+01,0.1002E+01,0.9924E+00,0.9814E+00,0.9694E+00,
     *0.9566E+00,0.9431E+00,0.9288E+00,0.9140E+00,0.8988E+00,0.8832E+00,
     *0.8673E+00,0.8513E+00,0.8353E+00,0.8194E+00,0.8038E+00,0.7885E+00,
     *0.7737E+00,0.7596E+00,0.7462E+00,0.7338E+00,0.7223E+00,0.7121E+00,
     *0.7031E+00,0.6955E+00,0.6895E+00,0.6852E+00,0.6827E+00,0.6820E+00,
     *0.6833E+00,0.6868E+00,0.6924E+00,0.7003E+00,0.7105E+00,0.7165E+00,
     *0.7232E+00,0.7383E+00,0.7559E+00,0.7760E+00,0.7987E+00,0.8240E+00,
     *0.8518E+00,0.8821E+00,0.9149E+00,0.9501E+00,0.9877E+00,0.1028E+01,
     *0.1069E+01,0.1113E+01,0.1159E+01,0.1207E+01,0.1256E+01,0.1306E+01,
     *0.1358E+01,0.1410E+01,0.1463E+01,0.1517E+01,0.1570E+01,0.1623E+01,
     *0.1676E+01,0.1727E+01,0.1778E+01,0.1827E+01,0.1873E+01,0.1918E+01,
     *0.1960E+01,0.1999E+01,0.2035E+01,0.2067E+01,0.2096E+01,0.2120E+01,
     *0.2140E+01,0.2156E+01,0.2167E+01,0.2173E+01,0.2174E+01/
            DATA ((PHR(I,J),J=1,83),I=09,09) /
     *0.1182E+01,0.1181E+01,0.1179E+01,0.1176E+01,0.1171E+01,0.1164E+01,
     *0.1156E+01,0.1147E+01,0.1136E+01,0.1124E+01,0.1110E+01,0.1096E+01,
     *0.1080E+01,0.1064E+01,0.1046E+01,0.1028E+01,0.1009E+01,0.9903E+00,
     *0.9708E+00,0.9510E+00,0.9312E+00,0.9114E+00,0.8919E+00,0.8726E+00,
     *0.8539E+00,0.8357E+00,0.8184E+00,0.8019E+00,0.7866E+00,0.7724E+00,
     *0.7595E+00,0.7481E+00,0.7383E+00,0.7302E+00,0.7239E+00,0.7195E+00,
     *0.7171E+00,0.7168E+00,0.7188E+00,0.7229E+00,0.7294E+00,0.7335E+00,
     *0.7382E+00,0.7494E+00,0.7630E+00,0.7790E+00,0.7974E+00,0.8182E+00,
     *0.8414E+00,0.8668E+00,0.8944E+00,0.9242E+00,0.9561E+00,0.9898E+00,
     *0.1025E+01,0.1063E+01,0.1101E+01,0.1141E+01,0.1183E+01,0.1225E+01,
     *0.1268E+01,0.1311E+01,0.1355E+01,0.1399E+01,0.1442E+01,0.1485E+01,
     *0.1528E+01,0.1569E+01,0.1609E+01,0.1648E+01,0.1685E+01,0.1720E+01,
     *0.1753E+01,0.1783E+01,0.1811E+01,0.1836E+01,0.1858E+01,0.1876E+01,
     *0.1891E+01,0.1903E+01,0.1911E+01,0.1916E+01,0.1917E+01/
            DATA ((PHR(I,J),J=1,83),I=10,10) /
     *0.1325E+01,0.1324E+01,0.1322E+01,0.1318E+01,0.1312E+01,0.1304E+01,
     *0.1294E+01,0.1283E+01,0.1270E+01,0.1256E+01,0.1240E+01,0.1222E+01,
     *0.1204E+01,0.1184E+01,0.1163E+01,0.1142E+01,0.1119E+01,0.1096E+01,
     *0.1073E+01,0.1049E+01,0.1025E+01,0.1001E+01,0.9776E+00,0.9541E+00,
     *0.9312E+00,0.9088E+00,0.8872E+00,0.8666E+00,0.8471E+00,0.8287E+00,
     *0.8118E+00,0.7963E+00,0.7825E+00,0.7704E+00,0.7602E+00,0.7519E+00,
     *0.7457E+00,0.7415E+00,0.7396E+00,0.7399E+00,0.7424E+00,0.7446E+00,
     *0.7473E+00,0.7545E+00,0.7640E+00,0.7758E+00,0.7899E+00,0.8063E+00,
     *0.8248E+00,0.8455E+00,0.8681E+00,0.8928E+00,0.9192E+00,0.9473E+00,
     *0.9771E+00,0.1008E+01,0.1041E+01,0.1074E+01,0.1109E+01,0.1144E+01,
     *0.1179E+01,0.1215E+01,0.1252E+01,0.1288E+01,0.1324E+01,0.1359E+01,
     *0.1393E+01,0.1427E+01,0.1460E+01,0.1491E+01,0.1521E+01,0.1549E+01,
     *0.1575E+01,0.1599E+01,0.1622E+01,0.1641E+01,0.1658E+01,0.1673E+01,
     *0.1685E+01,0.1694E+01,0.1701E+01,0.1704E+01,0.1705E+01/
      do i=1,10
        do j=1,83
          ph(i,j)=phr(i,j)
        enddo
      enddo
      return
      end
*
************************************************************************
*
      subroutine specinterp(wl,taer55,taerp55,
     s     tamoy,tamoyp,pizmoy,pizmp)
      real wl,taer55,taerp55,tamoy,tamoyp,pizmoy,pizmp,roatm
      real dtdir,dtdif,utdir,utdif,sphal,wldis,trayl,traypl
      real ext,ome,gasym,phase,pha,betal,phasel,cgaus,coef
      real wlinf,alphaa,betaa,tsca,coeff
      integer linf,ll,lsup,k
      common /six_disc/ roatm(3,10),dtdir(3,10),dtdif(3,10),
     s utdir(3,10),utdif(3,10),sphal(3,10),wldis(10),trayl(10),
     s traypl(10)
      common /sixs_aer/ext(10),ome(10),gasym(10),phase(10)
      common /sixs_trunc/pha(83),betal(0:80)
      common /sixs_sos/phasel(10,83),cgaus(83)
      linf=1
      do 80 ll=1,9
      if(wl.ge.wldis(ll).and.wl.le.wldis(ll+1)) linf=ll
   80 continue
      if(wl.gt.wldis(10)) linf=9
      lsup=linf+1
      coef=alog(wldis(lsup)/wldis(linf))
      wlinf=wldis(linf)
      alphaa=alog(ext(lsup)*ome(lsup)/(ext(linf)*ome(linf)))/coef
      betaa=ext(linf)*ome(linf)/(wlinf**alphaa)
      tsca=taer55*betaa*(wl**alphaa)/ext(4)
      alphaa=alog(ext(lsup)/ext(linf))/coef
      betaa=ext(linf)/(wlinf**alphaa)
      tamoy=taer55*betaa*(wl**alphaa)/ext(4)
      tamoyp=taerp55*betaa*(wl**alphaa)/ext(4)
      pizmoy=tsca/tamoy
      pizmp=pizmoy
      do k=1,83
        alphaa=alog(phasel(lsup,k)/phasel(linf,k))/coef
        betaa=phasel(linf,k)/(wlinf**alphaa)
        pha(k)=betaa*(wl**alphaa)
      enddo
      call trunca(coeff)
      tamoy=tamoy*(1.-pizmoy*coeff)
      tamoyp=tamoyp*(1.-pizmp*coeff)
      pizmoy=pizmoy*(1.-coeff)/(1.-pizmoy*coeff)
      return
      end
*
************************************************************************
*
      subroutine trunca(coeff)
      real aa,x1,x2,a,x,rm,z1
      real cosang(80),weight(80),ptemp(83),pl(-1:81)
      real rmu(83),ga(83)
      integer nbmu,nang,k,j,kk,i
      real pha,betal,coeff
      common /sixs_trunc/pha(83),betal(0:80)
      nbmu=83
      nang=80
      kk  = 0
      do k=1,nbmu
      ptemp(k)=pha(k)
      enddo
      call gauss(-1.,1.,cosang,weight,nang)
      do 1 j=1,40
      rmu(j+1)=cosang(j)
      ga(j+1)=weight(j)
   1  continue
      rmu(1)=-1.0
      ga(1)=0.
      rmu(42)=0.
      ga(42)=0.
      do 2 j=41,80
      rmu(j+2)=cosang(j)
      ga(j+2)=weight(j)
   2  continue
      rmu(83)=1.0
      ga(83)=0.
      do 3 j=1,nbmu
      if((rmu(j).gt.0.8)) then
      go to 20
      else
      k=j-1
      endif
   3  continue
  20  continue
      do 4 j=1,nbmu
      if((rmu(j).gt.0.94)) then
      go to 21
      else
      kk=j-1
      endif
   4  continue
  21  continue
      aa=(alog10(pha(kk))-alog10(pha(k)))/
     a       (acos(rmu(kk))-acos(rmu(k)))
      x1=alog10(pha(kk))
      x2=acos(rmu(kk))
      do 5 j=kk+1,nbmu
      if(abs(rmu(j)-1.).le.1e-08) a=x1-aa*x2
      a=x1+aa*(acos(rmu(j))-x2)
      ptemp(j)=10**a
    5 continue
      do i=1,83
      pha(i)=ptemp(i)
      enddo
c
      do 10 k=0,80
      betal(k)=0.
   10 continue
      do 11 j=1,83
      x=pha(j)*ga(j)
      rm=rmu(j)
      pl(-1)=0.
      pl(0)=1.
      do 12 k=0,80
      pl(k+1)=((2*k+1.)*rm*pl(k)-k*pl(k-1))/(k+1.)
      betal(k)=betal(k)+x*pl(k)
  12  continue
  11  continue
      do 13 k=0,80
      betal(k)=(2*k+1.)*0.5*betal(k)
  13  continue
      z1=betal(0)
      coeff=1.-z1
      do k=0,80
      betal(k)=betal(k)/z1
      enddo
      return
      end
*
************************************************************************
*
      subroutine   us62

      integer i
      real z6(34),p6(34),t6(34)
      real z,p,t
      common /sixs_atm/z(34),p(34),t(34)
c
c     model: us standard 62 mc clatchey
c
      data(z6(i),i=1, 34)/
     1    0.,    1.,    2.,    3.,    4.,    5.,    6.,    7.,    8.,
     2    9.,   10.,   11.,   12.,   13.,   14.,   15.,   16.,   17.,
     3   18.,   19.,   20.,   21.,   22.,   23.,   24.,   25.,   30.,
     4   35.,   40.,   45.,   50.,   70.,  100.,99999./
      data (p6(i),i=1,34) /
     a1.013e+03,8.986e+02,7.950e+02,7.012e+02,6.166e+02,5.405e+02,
     a4.722e+02,4.111e+02,3.565e+02,3.080e+02,2.650e+02,2.270e+02,
     a1.940e+02,1.658e+02,1.417e+02,1.211e+02,1.035e+02,8.850e+01,
     a7.565e+01,6.467e+01,5.529e+01,4.729e+01,4.047e+01,3.467e+01,
     a2.972e+01,2.549e+01,1.197e+01,5.746e+00,2.871e+00,1.491e+00,
     a7.978e-01,5.520e-02,3.008e-04,0.000e+00/
      data (t6(i),i=1,34) /
     a2.881e+02,2.816e+02,2.751e+02,2.687e+02,2.622e+02,2.557e+02,
     a2.492e+02,2.427e+02,2.362e+02,2.297e+02,2.232e+02,2.168e+02,
     a2.166e+02,2.166e+02,2.166e+02,2.166e+02,2.166e+02,2.166e+02,
     a2.166e+02,2.166e+02,2.166e+02,2.176e+02,2.186e+02,2.196e+02,
     a2.206e+02,2.216e+02,2.265e+02,2.365e+02,2.534e+02,2.642e+02,
     a2.706e+02,2.197e+02,2.100e+02,2.100e+02/
      do 1 i=1,34
      z(i)=z6(i)
      p(i)=p6(i)
      t(i)=t6(i)
    1 continue
      return
      end
*
************************************************************************
*
      subroutine   vegeta (r)
      real sr(1501),r(1501)
      integer l,i
 
c     vegetation average reflectance
c     warning : values of dry sand ground reflectance are given
c     between 0.4 and 2.2 microns. outside this interval the
c     values are set to 0.
 
      data (sr(l),l=1,184)/  58*0.,
     a .00000, .03000, .06000, .06000, .06000, .06100, .06200,
     a .06300, .06400, .06500, .06600, .06750, .06900, .06950,
     a .07000, .07100, .07200, .07300, .07400, .07600, .07800,
     a .07900, .08000, .08150, .08300, .08350, .08400, .08650,
     a .08900, .09100, .09300, .09550, .09800, .10000, .10200,
     a .10300, .10400, .10500, .10600, .10800, .11000, .11250,
     a .11500, .11700, .11900, .12000, .12100, .12300, .12500,
     a .12600, .12700, .12850, .13000, .13150, .13300, .13300,
     a .13300, .13350, .13400, .13350, .13300, .13200, .13100,
     a .12900, .12700, .12400, .12100, .11800, .11500, .11250,
     a .11000, .10750, .10500, .10300, .10100, .09950, .09800,
     a .09600, .09400, .09200, .09000, .08850, .08700, .08500,
     a .08300, .08200, .08100, .08050, .08000, .07900, .07800,
     a .07700, .07600, .07550, .07500, .07450, .07400, .07350,
     a .07300, .07300, .07300, .07300, .07300, .07350, .07400,
     a .07650, .07900, .08950, .10000, .11900, .13800, .15350,
     a .16900, .18400, .19900, .21350, .22800, .24350, .25900,
     a .27450, .29000, .30300, .31600, .33300, .35000, .36400/
      data (sr(l),l=185,310)/  
     a .37800, .39050, .40300, .41950, .43600, .44900, .46200,
     a .47450, .48700, .49800, .50900, .51000, .51100, .51250,
     a .51400, .51650, .51900, .51950, .52000, .52000, .52000,
     a .52100, .52200, .52200, .52200, .52200, .52200, .52250,
     a .52300, .52350, .52400, .52400, .52400, .52400, .52400,
     a .52400, .52400, .52500, .52600, .52600, .52600, .52600,
     a .52600, .52650, .52700, .52700, .52700, .52700, .52700,
     a .52750, .52800, .52800, .52800, .52800, .52800, .52850,
     a .52900, .52900, .52900, .52900, .52900, .52900, .52900,
     a .53000, .53100, .53100, .53100, .53100, .53100, .53100,
     a .53100, .53100, .53100, .53150, .53200, .53200, .53200,
     a .53200, .53200, .53200, .53200, .53200, .53200, .53250,
     a .53300, .53300, .53300, .53300, .53300, .53350, .53400,
     a .53400, .53400, .53400, .53400, .53400, .53400, .53450,
     a .53500, .53500, .53500, .53550, .53600, .53600, .53600,
     a .53650, .53700, .53700, .53700, .53650, .53600, .53600,
     a .53600, .53550, .53500, .53500, .53500, .53450, .53400,
     a .53300, .53200, .53150, .53100, .53050, .53000, .52900/
      data (sr(l),l=311,408)/  
     a .52800, .52800, .52800, .52750, .52700, .52700, .52700,
     a .52650, .52600, .52550, .52500, .52450, .52400, .52300,
     a .52200, .52150, .52100, .52000, .51900, .51850, .51800,
     a .51650, .51500, .51400, .51300, .51250, .51200, .51100,
     a .51000, .50900, .50800, .50750, .50700, .50650, .50600,
     a .50550, .50500, .50350, .50200, .50100, .50000, .49900,
     a .49800, .49700, .49600, .49550, .49500, .49400, .49300,
     a .49250, .49200, .49200, .49200, .49200, .49200, .49200,
     a .49200, .49200, .49200, .49250, .49300, .49400, .49500,
     a .49500, .49500, .49550, .49600, .49600, .49600, .49600,
     a .49600, .49650, .49700, .49700, .49700, .49700, .49700,
     a .49750, .49800, .49800, .49800, .49750, .49700, .49700,
     a .49700, .49700, .49700, .49600, .49500, .49400, .49300,
     a .49250, .49200, .49150, .49100, .48950, .48800, .48700/
        data (sr(l),l=409,534)/
     a .48600, .48400, .48200, .48000, .47800, .47700, .47600,
     a .47400, .47200, .46950, .46700, .46450, .46200, .45650,
     a .45100, .44600, .44100, .43500, .42900, .42500, .42100,
     a .41450, .40800, .40350, .39900, .39200, .38500, .37800,
     a .37100, .36800, .36500, .35700, .34900, .34400, .33900,
     a .33450, .33000, .32550, .32100, .31500, .30900, .30350,
     a .29800, .29350, .28900, .28400, .27900, .27550, .27200,
     a .26950, .26700, .26300, .25900, .25500, .25100, .24700,
     a .24300, .23800, .23300, .23100, .22900, .22650, .22400,
     a .22100, .21800, .21650, .21500, .21500, .21500, .21500,
     a .21500, .21500, .21500, .21700, .21900, .22100, .22300,
     a .22600, .22900, .23150, .23400, .23700, .24000, .24450,
     a .24900, .25250, .25600, .25800, .26000, .26350, .26700,
     a .27000, .27300, .27600, .27900, .28250, .28600, .28950,
     a .29300, .29650, .30000, .30300, .30600, .30900, .31200,
     a .31550, .31900, .32200, .32500, .32800, .33100, .33400,
     a .33700, .33900, .34100, .34300, .34500, .34800, .35100,
     a .35300, .35500, .35750, .36000, .36100, .36200, .36450/
        data (sr(l),l=535,660)/
     a .36700, .36800, .36900, .37050, .37200, .37400, .37600,
     a .37700, .37800, .37850, .37900, .38000, .38100, .38150,
     a .38200, .38300, .38400, .38500, .38600, .38650, .38700,
     a .38800, .38900, .38850, .38800, .38800, .38800, .38800,
     a .38800, .38800, .38800, .38800, .38800, .38800, .38800,
     a .38600, .38400, .38350, .38300, .38200, .38100, .38050,
     a .38000, .37900, .37800, .37700, .37600, .37500, .37400,
     a .37350, .37300, .37200, .37100, .37050, .37000, .36900,
     a .36800, .36750, .36700, .36650, .36600, .36550, .36500,
     a .36500, .36500, .36400, .36300, .36250, .36200, .36150,
     a .36100, .36000, .35900, .35850, .35800, .35750, .35700,
     a .35600, .35500, .35400, .35300, .35150, .35000, .34850,
     a .34700, .34650, .34600, .34550, .34500, .34400, .34300,
     a .34150, .34000, .33850, .33700, .33600, .33500, .33300,
     a .33100, .33050, .33000, .32550, .32100, .31650, .31200,
     a .30400, .29600, .28450, .27300, .24700, .22100, .20350,
     a .18600, .17200, .15800, .14800, .13800, .13350, .12900,
     a .12500, .12100, .11550, .11000, .10600, .10200, .09850/
        data (sr(l),l=661,758)/
     a .09500, .09300, .09100, .09000, .08900, .08750, .08600,
     a .08600, .08600, .08500, .08400, .08400, .08400, .08400,
     a .08400, .08500, .08600, .08650, .08700, .09000, .09300,
     a .09550, .09800, .10150, .10500, .10950, .11400, .11500,
     a .11600, .12000, .12400, .12850, .13300, .13350, .13400,
     a .13750, .14100, .14400, .14700, .14900, .15100, .15350,
     a .15600, .15900, .16200, .16400, .16600, .16800, .17000,
     a .17200, .17400, .17450, .17500, .17650, .17800, .17950,
     a .18100, .18300, .18500, .18600, .18700, .18750, .18800,
     a .19000, .19200, .19400, .19600, .19750, .19900, .20000,
     a .20100, .20300, .20500, .20650, .20800, .21000, .21200,
     a .21250, .21300, .21350, .21400, .21550, .21700, .21800,
     a .21900, .21950, .22000, .22050, .22100, .22250, .22400,
     a .22550, .22700, .22800, .22900, .23000, .23100, .23200/
       data (sr(l),l=759,884)/
     a .23300, .23500, .23700, .23750, .23800, .23850, .23900,
     a .24000, .24100, .24150, .24200, .24250, .24300, .24400,
     a .24500, .24500, .24500, .24550, .24600, .24700, .24800,
     a .24800, .24800, .24900, .25000, .24800, .24600, .24400,
     a .24200, .24000, .23800, .23600, .23400, .23200, .23000,
     a .22800, .22600, .22400, .22200, .22000, .21800, .21600,
     a .21400, .21200, .21000, .20800, .20600, .20400, .20200,
     a .20000, .19800, .19600, .19400, .19200, .19000, .18800,
     a .18600, .18400, .18200, .18000, .17800, .17600, .17400,
     a .17200, .17000, .16800, .16600, .16400, .16200, .16000,
     a .15800, .15600, .15400, .15200, .15000, .14800, .14600,
     a .14400, .14200, .14000, .13800, .13600, .13400, .13200,
     a .13000, .12800, .12600, .12400, .12200, .12000, .11800,
     a .11600, .11400, .11200, .11000, .10800, .10600, .10400,
     a .10200, .10000, .09800, .09600, .09400, .09200, .09000,
     a .08800, .08600, .08400, .08200, .08000, .07800, .07600,
     a .07400, .07200, .07000, .06800, .06600, .06400, .06200,
     a .06000, .05800, .05600, .05400, .05200, .05000, .04800/
       data (sr(l),l=885,1501)/
     a .04600, .04400, .04200, .04000, .03800, .03600, .03400,
     a .03200, .03000, .02800, .02600, .02400, .02200, .02000,
     a .01800, .01600, .01400, .01200, .01000, .01000, .01000,
     a .01000, .01000, .01000, .01000, .01000, .01000, .01000,
     a .01000, .01000, .01000, .01000, .01000, .01000, .01000,
     a .01000, .01000, .01000, .01000, .01000, .01000, .01000,
     a .01000, .01000, .01000, .01000, .01000, .01000, .01000,
     a .01000, .01000, .01000, .01000, .01000, .01000, .01000,
     a .01000,
     a 560*0./
      do 1 i=1,1501
      r(i)=sr(i)
    1 continue
      return
      end
*
************************************************************************
*
       subroutine   wate
       integer i,j
       real phr(10,83)
       real ph
       common /sixs_aerbas/ ph(10,83)
c
c    model: water-soluble
c
            DATA ((PHR(I,J),J=1,83),I=01,01) /
     *0.4115E+00,0.4045E+00,0.3805E+00,0.3495E+00,0.3192E+00,0.2943E+00,
     *0.2768E+00,0.2659E+00,0.2592E+00,0.2538E+00,0.2479E+00,0.2411E+00,
     *0.2336E+00,0.2255E+00,0.2175E+00,0.2098E+00,0.2026E+00,0.1961E+00,
     *0.1903E+00,0.1854E+00,0.1812E+00,0.1778E+00,0.1752E+00,0.1734E+00,
     *0.1723E+00,0.1719E+00,0.1724E+00,0.1736E+00,0.1756E+00,0.1784E+00,
     *0.1820E+00,0.1866E+00,0.1920E+00,0.1985E+00,0.2061E+00,0.2149E+00,
     *0.2249E+00,0.2363E+00,0.2492E+00,0.2638E+00,0.2803E+00,0.2893E+00,
     *0.2988E+00,0.3195E+00,0.3428E+00,0.3688E+00,0.3979E+00,0.4306E+00,
     *0.4671E+00,0.5079E+00,0.5537E+00,0.6048E+00,0.6622E+00,0.7264E+00,
     *0.7985E+00,0.8794E+00,0.9701E+00,0.1072E+01,0.1186E+01,0.1315E+01,
     *0.1460E+01,0.1622E+01,0.1805E+01,0.2011E+01,0.2242E+01,0.2503E+01,
     *0.2796E+01,0.3125E+01,0.3496E+01,0.3913E+01,0.4383E+01,0.4912E+01,
     *0.5510E+01,0.6185E+01,0.6951E+01,0.7825E+01,0.8828E+01,0.9991E+01,
     *0.1136E+02,0.1297E+02,0.1491E+02,0.1711E+02,0.1834E+02/
            DATA ((PHR(I,J),J=1,83),I=02,02) /
     *0.3918E+00,0.3859E+00,0.3654E+00,0.3384E+00,0.3117E+00,0.2895E+00,
     *0.2736E+00,0.2635E+00,0.2571E+00,0.2522E+00,0.2470E+00,0.2411E+00,
     *0.2345E+00,0.2275E+00,0.2204E+00,0.2135E+00,0.2071E+00,0.2012E+00,
     *0.1959E+00,0.1914E+00,0.1875E+00,0.1844E+00,0.1820E+00,0.1804E+00,
     *0.1794E+00,0.1792E+00,0.1797E+00,0.1810E+00,0.1831E+00,0.1860E+00,
     *0.1898E+00,0.1945E+00,0.2001E+00,0.2068E+00,0.2146E+00,0.2236E+00,
     *0.2339E+00,0.2456E+00,0.2589E+00,0.2739E+00,0.2909E+00,0.3001E+00,
     *0.3099E+00,0.3312E+00,0.3552E+00,0.3820E+00,0.4119E+00,0.4455E+00,
     *0.4830E+00,0.5249E+00,0.5718E+00,0.6243E+00,0.6829E+00,0.7486E+00,
     *0.8221E+00,0.9045E+00,0.9968E+00,0.1100E+01,0.1216E+01,0.1346E+01,
     *0.1492E+01,0.1655E+01,0.1839E+01,0.2045E+01,0.2275E+01,0.2534E+01,
     *0.2824E+01,0.3149E+01,0.3513E+01,0.3920E+01,0.4375E+01,0.4884E+01,
     *0.5454E+01,0.6092E+01,0.6807E+01,0.7611E+01,0.8516E+01,0.9543E+01,
     *0.1071E+02,0.1205E+02,0.1357E+02,0.1518E+02,0.1599E+02/
            DATA ((PHR(I,J),J=1,83),I=03,03) /
     *0.3872E+00,0.3816E+00,0.3620E+00,0.3360E+00,0.3102E+00,0.2887E+00,
     *0.2732E+00,0.2633E+00,0.2571E+00,0.2522E+00,0.2471E+00,0.2414E+00,
     *0.2350E+00,0.2283E+00,0.2214E+00,0.2148E+00,0.2085E+00,0.2028E+00,
     *0.1976E+00,0.1932E+00,0.1894E+00,0.1864E+00,0.1840E+00,0.1824E+00,
     *0.1815E+00,0.1813E+00,0.1819E+00,0.1832E+00,0.1853E+00,0.1883E+00,
     *0.1920E+00,0.1968E+00,0.2024E+00,0.2092E+00,0.2170E+00,0.2261E+00,
     *0.2364E+00,0.2483E+00,0.2617E+00,0.2768E+00,0.2939E+00,0.3032E+00,
     *0.3131E+00,0.3346E+00,0.3587E+00,0.3857E+00,0.4159E+00,0.4497E+00,
     *0.4875E+00,0.5297E+00,0.5769E+00,0.6297E+00,0.6887E+00,0.7547E+00,
     *0.8286E+00,0.9114E+00,0.1004E+01,0.1108E+01,0.1224E+01,0.1354E+01,
     *0.1500E+01,0.1664E+01,0.1847E+01,0.2053E+01,0.2284E+01,0.2542E+01,
     *0.2831E+01,0.3154E+01,0.3515E+01,0.3919E+01,0.4370E+01,0.4874E+01,
     *0.5436E+01,0.6064E+01,0.6765E+01,0.7549E+01,0.8430E+01,0.9422E+01,
     *0.1054E+02,0.1182E+02,0.1324E+02,0.1472E+02,0.1544E+02/
            DATA ((PHR(I,J),J=1,83),I=04,04) /
     *0.3737E+00,0.3687E+00,0.3509E+00,0.3269E+00,0.3030E+00,0.2830E+00,
     *0.2686E+00,0.2593E+00,0.2535E+00,0.2490E+00,0.2444E+00,0.2393E+00,
     *0.2335E+00,0.2273E+00,0.2210E+00,0.2148E+00,0.2089E+00,0.2036E+00,
     *0.1987E+00,0.1945E+00,0.1910E+00,0.1881E+00,0.1859E+00,0.1844E+00,
     *0.1836E+00,0.1835E+00,0.1842E+00,0.1855E+00,0.1877E+00,0.1907E+00,
     *0.1945E+00,0.1993E+00,0.2051E+00,0.2118E+00,0.2198E+00,0.2289E+00,
     *0.2394E+00,0.2513E+00,0.2649E+00,0.2802E+00,0.2974E+00,0.3068E+00,
     *0.3168E+00,0.3385E+00,0.3628E+00,0.3901E+00,0.4206E+00,0.4547E+00,
     *0.4928E+00,0.5353E+00,0.5829E+00,0.6361E+00,0.6955E+00,0.7620E+00,
     *0.8363E+00,0.9195E+00,0.1013E+01,0.1117E+01,0.1233E+01,0.1364E+01,
     *0.1510E+01,0.1674E+01,0.1858E+01,0.2063E+01,0.2293E+01,0.2550E+01,
     *0.2838E+01,0.3160E+01,0.3518E+01,0.3919E+01,0.4365E+01,0.4863E+01,
     *0.5416E+01,0.6033E+01,0.6719E+01,0.7483E+01,0.8337E+01,0.9292E+01,
     *0.1036E+02,0.1156E+02,0.1289E+02,0.1423E+02,0.1486E+02/
            DATA ((PHR(I,J),J=1,83),I=05,05) /
     *0.3651E+00,0.3607E+00,0.3449E+00,0.3233E+00,0.3016E+00,0.2832E+00,
     *0.2697E+00,0.2609E+00,0.2552E+00,0.2509E+00,0.2465E+00,0.2418E+00,
     *0.2364E+00,0.2307E+00,0.2249E+00,0.2191E+00,0.2137E+00,0.2086E+00,
     *0.2041E+00,0.2001E+00,0.1968E+00,0.1940E+00,0.1919E+00,0.1905E+00,
     *0.1898E+00,0.1897E+00,0.1904E+00,0.1919E+00,0.1941E+00,0.1971E+00,
     *0.2011E+00,0.2059E+00,0.2118E+00,0.2187E+00,0.2267E+00,0.2361E+00,
     *0.2467E+00,0.2589E+00,0.2727E+00,0.2883E+00,0.3059E+00,0.3155E+00,
     *0.3257E+00,0.3478E+00,0.3726E+00,0.4004E+00,0.4315E+00,0.4662E+00,
     *0.5050E+00,0.5483E+00,0.5967E+00,0.6507E+00,0.7110E+00,0.7783E+00,
     *0.8536E+00,0.9376E+00,0.1032E+01,0.1137E+01,0.1254E+01,0.1385E+01,
     *0.1531E+01,0.1695E+01,0.1878E+01,0.2083E+01,0.2311E+01,0.2566E+01,
     *0.2850E+01,0.3166E+01,0.3518E+01,0.3910E+01,0.4344E+01,0.4825E+01,
     *0.5358E+01,0.5947E+01,0.6597E+01,0.7314E+01,0.8106E+01,0.8978E+01,
     *0.9939E+01,0.1099E+02,0.1211E+02,0.1319E+02,0.1367E+02/
            DATA ((PHR(I,J),J=1,83),I=06,06) /
     *0.3540E+00,0.3501E+00,0.3360E+00,0.3166E+00,0.2969E+00,0.2801E+00,
     *0.2677E+00,0.2594E+00,0.2541E+00,0.2500E+00,0.2461E+00,0.2417E+00,
     *0.2369E+00,0.2317E+00,0.2263E+00,0.2211E+00,0.2160E+00,0.2113E+00,
     *0.2070E+00,0.2033E+00,0.2001E+00,0.1976E+00,0.1956E+00,0.1943E+00,
     *0.1937E+00,0.1937E+00,0.1945E+00,0.1960E+00,0.1982E+00,0.2013E+00,
     *0.2053E+00,0.2102E+00,0.2162E+00,0.2232E+00,0.2313E+00,0.2408E+00,
     *0.2516E+00,0.2639E+00,0.2779E+00,0.2937E+00,0.3115E+00,0.3213E+00,
     *0.3315E+00,0.3540E+00,0.3791E+00,0.4073E+00,0.4387E+00,0.4739E+00,
     *0.5131E+00,0.5569E+00,0.6057E+00,0.6603E+00,0.7211E+00,0.7890E+00,
     *0.8647E+00,0.9493E+00,0.1044E+01,0.1149E+01,0.1267E+01,0.1398E+01,
     *0.1545E+01,0.1708E+01,0.1891E+01,0.2095E+01,0.2322E+01,0.2575E+01,
     *0.2856E+01,0.3169E+01,0.3517E+01,0.3902E+01,0.4328E+01,0.4799E+01,
     *0.5318E+01,0.5890E+01,0.6519E+01,0.7208E+01,0.7963E+01,0.8788E+01,
     *0.9685E+01,0.1065E+02,0.1166E+02,0.1261E+02,0.1301E+02/
            DATA ((PHR(I,J),J=1,83),I=07,07) /
     *0.3121E+00,0.3097E+00,0.3008E+00,0.2882E+00,0.2753E+00,0.2643E+00,
     *0.2562E+00,0.2509E+00,0.2473E+00,0.2445E+00,0.2417E+00,0.2384E+00,
     *0.2348E+00,0.2307E+00,0.2265E+00,0.2223E+00,0.2182E+00,0.2144E+00,
     *0.2109E+00,0.2078E+00,0.2052E+00,0.2030E+00,0.2014E+00,0.2004E+00,
     *0.2000E+00,0.2002E+00,0.2011E+00,0.2027E+00,0.2051E+00,0.2082E+00,
     *0.2123E+00,0.2173E+00,0.2232E+00,0.2303E+00,0.2386E+00,0.2482E+00,
     *0.2591E+00,0.2717E+00,0.2859E+00,0.3019E+00,0.3201E+00,0.3300E+00,
     *0.3404E+00,0.3633E+00,0.3889E+00,0.4176E+00,0.4496E+00,0.4854E+00,
     *0.5253E+00,0.5699E+00,0.6196E+00,0.6749E+00,0.7367E+00,0.8055E+00,
     *0.8822E+00,0.9677E+00,0.1063E+01,0.1169E+01,0.1288E+01,0.1419E+01,
     *0.1566E+01,0.1730E+01,0.1912E+01,0.2115E+01,0.2341E+01,0.2591E+01,
     *0.2869E+01,0.3177E+01,0.3518E+01,0.3895E+01,0.4309E+01,0.4765E+01,
     *0.5265E+01,0.5811E+01,0.6405E+01,0.7049E+01,0.7744E+01,0.8489E+01,
     *0.9280E+01,0.1010E+02,0.1093E+02,0.1165E+02,0.1192E+02/
            DATA ((PHR(I,J),J=1,83),I=08,08) /
     *0.3070E+00,0.3061E+00,0.3027E+00,0.2975E+00,0.2918E+00,0.2865E+00,
     *0.2821E+00,0.2787E+00,0.2760E+00,0.2735E+00,0.2711E+00,0.2684E+00,
     *0.2656E+00,0.2626E+00,0.2594E+00,0.2562E+00,0.2530E+00,0.2500E+00,
     *0.2471E+00,0.2446E+00,0.2423E+00,0.2404E+00,0.2390E+00,0.2380E+00,
     *0.2375E+00,0.2377E+00,0.2385E+00,0.2400E+00,0.2422E+00,0.2453E+00,
     *0.2493E+00,0.2543E+00,0.2604E+00,0.2677E+00,0.2762E+00,0.2861E+00,
     *0.2976E+00,0.3108E+00,0.3258E+00,0.3428E+00,0.3620E+00,0.3725E+00,
     *0.3836E+00,0.4079E+00,0.4351E+00,0.4655E+00,0.4993E+00,0.5371E+00,
     *0.5791E+00,0.6258E+00,0.6776E+00,0.7351E+00,0.7988E+00,0.8694E+00,
     *0.9476E+00,0.1034E+01,0.1130E+01,0.1236E+01,0.1353E+01,0.1482E+01,
     *0.1625E+01,0.1783E+01,0.1957E+01,0.2148E+01,0.2359E+01,0.2590E+01,
     *0.2844E+01,0.3121E+01,0.3424E+01,0.3754E+01,0.4112E+01,0.4498E+01,
     *0.4913E+01,0.5356E+01,0.5826E+01,0.6320E+01,0.6833E+01,0.7358E+01,
     *0.7884E+01,0.8390E+01,0.8846E+01,0.9187E+01,0.9295E+01/
            DATA ((PHR(I,J),J=1,83),I=09,09) /
     *0.3321E+00,0.3315E+00,0.3294E+00,0.3266E+00,0.3238E+00,0.3214E+00,
     *0.3192E+00,0.3169E+00,0.3142E+00,0.3111E+00,0.3075E+00,0.3036E+00,
     *0.2994E+00,0.2950E+00,0.2905E+00,0.2860E+00,0.2817E+00,0.2775E+00,
     *0.2735E+00,0.2698E+00,0.2665E+00,0.2635E+00,0.2609E+00,0.2587E+00,
     *0.2571E+00,0.2561E+00,0.2556E+00,0.2558E+00,0.2568E+00,0.2586E+00,
     *0.2613E+00,0.2650E+00,0.2697E+00,0.2756E+00,0.2827E+00,0.2913E+00,
     *0.3013E+00,0.3131E+00,0.3267E+00,0.3422E+00,0.3600E+00,0.3698E+00,
     *0.3802E+00,0.4030E+00,0.4287E+00,0.4575E+00,0.4899E+00,0.5261E+00,
     *0.5665E+00,0.6115E+00,0.6617E+00,0.7175E+00,0.7795E+00,0.8484E+00,
     *0.9248E+00,0.1010E+01,0.1103E+01,0.1208E+01,0.1323E+01,0.1451E+01,
     *0.1592E+01,0.1749E+01,0.1922E+01,0.2113E+01,0.2324E+01,0.2557E+01,
     *0.2813E+01,0.3095E+01,0.3403E+01,0.3740E+01,0.4106E+01,0.4502E+01,
     *0.4928E+01,0.5383E+01,0.5863E+01,0.6364E+01,0.6878E+01,0.7395E+01,
     *0.7898E+01,0.8366E+01,0.8764E+01,0.9041E+01,0.9119E+01/
            DATA ((PHR(I,J),J=1,83),I=10,10) /
     *0.4248E+00,0.4242E+00,0.4221E+00,0.4189E+00,0.4153E+00,0.4116E+00,
     *0.4081E+00,0.4045E+00,0.4006E+00,0.3964E+00,0.3918E+00,0.3869E+00,
     *0.3818E+00,0.3764E+00,0.3709E+00,0.3654E+00,0.3600E+00,0.3547E+00,
     *0.3495E+00,0.3446E+00,0.3401E+00,0.3359E+00,0.3321E+00,0.3288E+00,
     *0.3260E+00,0.3239E+00,0.3224E+00,0.3218E+00,0.3219E+00,0.3230E+00,
     *0.3251E+00,0.3282E+00,0.3326E+00,0.3383E+00,0.3455E+00,0.3542E+00,
     *0.3646E+00,0.3768E+00,0.3911E+00,0.4075E+00,0.4263E+00,0.4366E+00,
     *0.4476E+00,0.4717E+00,0.4989E+00,0.5293E+00,0.5633E+00,0.6011E+00,
     *0.6431E+00,0.6896E+00,0.7410E+00,0.7977E+00,0.8603E+00,0.9291E+00,
     *0.1005E+01,0.1088E+01,0.1179E+01,0.1278E+01,0.1387E+01,0.1506E+01,
     *0.1636E+01,0.1778E+01,0.1933E+01,0.2100E+01,0.2283E+01,0.2480E+01,
     *0.2693E+01,0.2923E+01,0.3169E+01,0.3433E+01,0.3713E+01,0.4009E+01,
     *0.4319E+01,0.4642E+01,0.4973E+01,0.5308E+01,0.5640E+01,0.5962E+01,
     *0.6262E+01,0.6528E+01,0.6740E+01,0.6876E+01,0.6911E+01/
c
       do i=1,10
         do j=1,83
           ph(i,j)=phr(i,j)
         enddo
       enddo
       return
       end
*
************************************************************************
*
